/*
 * JaTS Project - Java Transformation System
 * http://www.cin.ufpe.br/~jats
 *  
 * Copyright  2005  Informatics Center - Federal University of Pernambuco
 * 
 * JaTS is a free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 */
package cin.jats.engine;

import cin.jats.engine.parser.ParseException;
import cin.jats.engine.parser.nodes.*;
import cin.jats.engine.util.NodeTable;

import java.io.Reader;
import java.io.FileNotFoundException;


/**
 * Esta interface define os mtodos que devem ser comuns aos parsers de C# e de
 * Java.
 *
 * @author Adeline de Sousa
 * @version 20/01/2004
 */
public interface IParser {

    ICompilationUnit parse(Reader entrada) throws ParseException,
            IllegalArgumentException;

    ICompilationUnit parse(String entrada) throws ParseException,
            IllegalArgumentException;

    ICompilationUnit parseFile(String fileName) throws ParseException,
            IllegalArgumentException, FileNotFoundException;

    /**
     * Retorna a tabela de ns que foi construda durante o processo de parsing
     * do arquivo fonte
     */
    public NodeTable getNodeTable();


    /**
     * Reinicializa a tabela
     */
    public void reInitTable();

    /**
     * Realiza o parsing de uma declarao de mtodo. A entrada pode tanto ser
     * uma String quanto um Reader
     */
    public JMethodDeclaration parseMethod(Reader entrada) throws ParseException,
            IllegalArgumentException;

    public JMethodDeclaration parseMethod(String entrada) throws ParseException,
            IllegalArgumentException;
    /**
     * Realiza o parsing de uma declarao de atributo. A entrada pode tanto ser
     * uma String quanto um Reader
     */
    public JVariableDeclaration parseField(Reader entrada) throws ParseException,
            IllegalArgumentException;

    public JVariableDeclaration parseField(String entrada) throws ParseException,
            IllegalArgumentException;


    /**
        * Realiza o parsing de uma declarao de construtor. A entrada pode tanto
        * ser uma String quanto um Reader
        */
       public JConstructorDeclaration parseConstructor(Reader entrada)
               throws ParseException, IllegalArgumentException;

       public JConstructorDeclaration parseConstructor(String entrada)
               throws ParseException, IllegalArgumentException;


       /**
        * Realiza o parsing de uma declarao de import. A entrada pode tanto
        * ser uma String quanto um Reader
        */
       public JImportDeclaration parseImport(Reader entrada)
               throws ParseException, IllegalArgumentException;

       public JImportDeclaration parseImport(String entrada)
               throws ParseException, IllegalArgumentException;


       /**
        * Realiza o parsing de uma declarao de pacote. A entrada pode tanto
        * ser uma String quanto um Reader
        */
       public JPackageDeclaration parsePackage(Reader entrada)
               throws ParseException, IllegalArgumentException;

        public JPackageDeclaration parsePackage(String entrada)
                throws ParseException, IllegalArgumentException;

    /**
     * Realiza o parsing de um identificador. A entrada pode ser uma String ou
     * um reader
     */
    public JIdentifier parseIdentifier(Reader entrada)
            throws ParseException, IllegalArgumentException;

    public JIdentifier parseIdentifier(String entrada)
            throws ParseException, IllegalArgumentException;


    /**
     * Realiza o parsing de um name. A entrada pode ser uma String ou
     * um reader
     */
    public JName parseName(Reader entrada)
            throws ParseException, IllegalArgumentException;

    public JName parseName(String entrada)
            throws ParseException, IllegalArgumentException;


    /**
     * Realiza o parsing de um name. A entrada pode ser uma String ou
     * um reader
     */
    public JLiteral parseLiteral(Reader entrada)
            throws ParseException, IllegalArgumentException;

    public JLiteral parseLiteral(String entrada)
            throws ParseException, IllegalArgumentException;


    /**
     * Realiza o parsing de um tipo. A entrada pode ser uma String ou
     * um reader
     */
    public JType parseType(Reader entrada)
            throws ParseException, IllegalArgumentException;

    public JType parseType(String entrada)
            throws ParseException, IllegalArgumentException;

    public JType parseResultType(Reader entrada) throws ParseException,
			IllegalArgumentException;

	public JType parseResultType(String entrada) throws ParseException,
			IllegalArgumentException;

    /**
     * Realiza o parsing de um tipo. A entrada pode ser uma String ou
     * um reader
     */
    public JBlock parseBlock(Reader entrada)
            throws ParseException, IllegalArgumentException;

    public JBlock parseBlock(String entrada)
            throws ParseException, IllegalArgumentException;

    /**
     * Realiza o parsing de uma expresso. A entrada pode ser uma String ou
     * um reader
     */
    public JExpression parseExpression(Reader entrada)
            throws ParseException, IllegalArgumentException;

    public JExpression parseExpression(String entrada)
            throws ParseException, IllegalArgumentException;

    /**
     * Realiza o parsing de um parmetro. A entrada pode ser uma String ou
     * um reader
     */
    public JFormalParameter parseParameter(Reader entrada)
            throws ParseException, IllegalArgumentException;

    public JFormalParameter parseParameter(String entrada)
            throws ParseException, IllegalArgumentException;


    /**
     * Realiza o parsing de uma lista de parmetros. A entrada pode ser uma
     * String ou  um reader
     */
    public JParameterList parseParameterList(Reader entrada)
            throws ParseException, IllegalArgumentException;

    public JParameterList parseParameterList(String entrada)
            throws ParseException, IllegalArgumentException;

    /**
     * Realiza o parsing de uma lista de modificadores
     */
    public JModifierList parseModifierList(Reader entrada)
            throws ParseException, IllegalArgumentException;

    public JModifierList parseModifierList(String entrada)
            throws ParseException, IllegalArgumentException;

    /**
     *  Realiza o parsing de uma lista de argumentos
     */
    public JExpressionList parseArgumentList(Reader entrada)
            throws ParseException, IllegalArgumentException;

    public JExpressionList parseArgumentList(String entrada)
            throws ParseException, IllegalArgumentException;

    public JFieldDeclaration[] getFields(NodeTable table, boolean var);

    public JMethodDeclaration[] getMethods(NodeTable table, boolean var);

	public JStatementList parseStatementList(Reader rdValue) throws ParseException;
	
	public JStatementList parseStatementList(String strValue) throws ParseException;

	public INode parseMetaExpression(Reader in) throws ParseException;

}
