/*
 * JaTS Project - Java Transformation System
 * http://www.cin.ufpe.br/~jats
 *  
 * Copyright  2005  Informatics Center - Federal University of Pernambuco
 * 
 * JaTS is a free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 */
package cin.jats.engine;

import java.io.*;
import java.util.Properties;

//import qualiti.qcode.printing.PrettyPrinterConfiguration;
import cin.jats.engine.parser.ParseException;
import cin.jats.engine.parser.nodes.*;
// 
import cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException;
import cin.jats.engine.parser.nodes.exceptions.NodesNotMatchedException;
import cin.jats.engine.parser.nodes.exceptions.ExecutionException;
import cin.jats.engine.transformation.JaTSTransformation;
import cin.jats.engine.transformation.MicroTransformation;
import cin.jats.engine.util.CompilationUnitSet;
import cin.jats.engine.util.ResultSet;

/**
 * Esta classe representa a fachada do JaTS como engine de transformao.
 * Diferente do que acontecia antes, ela nao mais implementa a interface
 * ITransformationEngine. Sendo apenas um ponto de acesso s funcinalidades do
 * JaTS.
 *
 * @author    Adeline de Sousa Silva
 * @version   01/03/2004
 */
public class JaTSFacade {

    private static JaTSFacade instance;

    // engine de transformao JaTS
    private JatsEngine engine;

    // componente de entrada-saida
    private JatsIO jatsio;


    public static JaTSFacade getInstance(){

        if (instance == null) {
            instance = new JaTSFacade();
        }
        return instance;
    }

    /**
     * Construtor da Fachada
     */
    private JaTSFacade() {
        this.engine = JatsEngine.getInstance();
        this.jatsio = JatsIO.getInstance();
    }

    public void initPrettyPrinter(Properties prop) {
        // instancia as configuraes do pretty-printer, carregando as
        // informaoes do arquivo
        if (prop != null) {
            jatsio.setPrettyPrinterProperties(prop);
        }
    }


    public void setPrettyPrinterProperties(Properties prop) {
        if (prop != null) {
//            try {
//                jatsio.setPrettyPrinterProperties(prop);
//                //PrettyPrinterConfiguration.saveProperties(prop);
//            }
//            catch (IOException e) {
//                e.printStackTrace();
//            }
        }
        else {
            throw new IllegalArgumentException();

        }
    }


    /**
     * Retorna as propriedades do PrettyPrinter
     * @return
     */
    public Properties getPrettyPrinterProperties() {
        return null;
        //return PrettyPrinterConfiguration.getProperties();
    }


    /**
     * Retorna a CompilationUnit contido no InputStream passado como parametro.
     *
     * @param  input   InputStream representando o tipo
     *
     * @return IType contido no InputStream passado como parametro
     *
     * @exception  ParseException - Caso haja algum erro no parsing
     *        do arquivo, ou na converso da rvore JaTS para a rvore IType
     */
    public ICompilationUnit parse(InputStream input)
            throws ParseException {
        return jatsio.parse(input);
    }


//    /**
//     * Este mtodo aplica a transformao transformation e retorna um conjunto de
//     * objetos IType que representam as classes geradas pela transformao.
//     *
//     * @param  transformation     Transformao a ser aplicada
//     * @param  rset               Result Set, contendo os mapeamentos
//     *        (var -> valor) que sero usados na substituio
//     *
//     * @return   Tipos gerados e/ou modificados pela transformao
//     *
//     * @exception  cin.jats.engine.parser.nodes.exceptions.ExecutionException  - Caso haja algum erro durante a
//     *         aplicao da transformao, como erro de parsindo dos templates,
//     *         erro de matching ou erro de aplicao mesmo
//     */
//    public ICompilationUnit[] apply(JaTSTransformation transformation,
//                                    ResultSet rset) throws ExecutionException,
//            cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException {
//        ICompilationUnit[] retorno = null;
//        if (transformation != null && rset != null) {
//
//            CompilationUnitSet right;
//            right = transformation.getRightTemplates();
//
//            // aplica a transformao
//            CompilationUnitSet tempRetorno = engine.apply(right, rset);
//
//            // retorna os tipos gerados
//            retorno = tempRetorno.toArray();
//
//
//        }
//        else {
//            throw new IllegalArgumentException("Argumentos invlidos");
//        }
//        return retorno;
//    }

    public INode apply(MicroTransformation transformation, INode source,
			ResultSet rset) throws ExecutionException,
			InconsistentNodeException, IllegalArgumentException,
			NodesNotMatchedException {
		INode result = null;
		if (transformation != null && rset != null) {
			INode left = transformation.getSource();
			INode right = transformation.getTarget();
			result = engine.apply(left, source, right, rset);
		} else {
			throw new IllegalArgumentException("Invalid arguments");
		}
		return result;
	}




    public void match(CompilationUnitSet sources, CompilationUnitSet leftTemplates,
                      ResultSet rset)
            throws NodesNotMatchedException, cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException {

		engine.match(leftTemplates,sources,rset);
	}


    /**
     * Realiza o matching entre dois ns JaTS. Estes, devem possuir o mesmo
     * tipo.
     *
     * @param left
     * @param source
     * @param rset
     * @throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException
     * @throws NodesNotMatchedException
     */
    public void match(INode left, INode source, ResultSet rset)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, NodesNotMatchedException {

        if (left.getTypeIdentifier() != source.getTypeIdentifier()) {
            throw new NodesNotMatchedException(
                    NodesNotMatchedException.NODES_HAVE_DIFFERENT_TYPES_MSG,
                    left, source);
        }
        engine.match(left, source, rset);

    }


    /**
     * Retorna um objeto representando o n da rvore sinttica correspondente a
     * String passada como parametro. O tipo do n retornado  igual ao tipo
     * especificado no parametro. Os tipos de ns passveis de serem retornados
     * so:
     * <ul>
     * <li>JaTSNode.IDENTIFIER </li>
     * <li>JaTSNode.NAME </li>
     * <li>JaTSNode.FIELD_DECLARATION</li>
     * <li>JaTSNode.METHOD_DECLARATION</li>
     * <li>JaTSNode.CONSTRUCTOR_DECLARATION </li>
     * <li>JaTSNode.IMPORT_DECLARATION </li>
     * <li>JaTSNode.PACKAGE_DECLARATION </li>
     * <li>JaTSNode.TYPE</li>
     * <li>JaTSNode.MODIFIER_LIST </li>
     * <li>JaTSNode.PARAMETER </li>
     * <li>JaTSNode.PARAMETER_LIST</li>
     * <li>JaTSNode.LITERAL </li>
     * <li>JaTSNode.EXPRESSION</li>
     * </ul>
     *
     * @param  type   Cdigo do n a ser retornado
     * @param  input  String representando o n
     *
     * @return  Um n da rvore sinttica, do tipo especificado em <code>type</code>
     *
     */
    public Object getNode(int type, String input) throws ParseException {
        Object retorno = null;
        if (input != null) {
            retorno = JatsIO.parseString(type, input);
        } else {
            throw new IllegalArgumentException("object cannot be parsed");
        }

        return retorno;
    }


    /**
     * Adiciona uma declarao ao conjunto de declaraes mapeadas  varivel
     * especificada no parmetro. Ateno: o cdigo do tipo passado como
     * parmetro, deve ser o cdigo do tipo da declarao. Ex: <br>
     * Se temos a varivel <code>#FDS</code> e queremos adicionar a ela a
     * declarao: <br>
     * <code> "private Conta conta;"</code>. Ento, o modo correto de
     * invocar este mtodo :
     * <br>
     * <code>
     * addDeclaration("private Conta conta;", "#FDS", JaTSNode.FIELD_DECLARATION);
     * </code>
     *
     * @param strDec  String correpondente a declarao a ser adicionada. Ex:
     * "private int numero;", "public void setNumero();" <br>
     *
     * @param varName Nome da varivel do tipo Conjunto de declaraes
     * (de mtodos, de atributos, de importaes ou de construtores). Ex:
     * "#FDS", "#CDS", "#MTDS" <br>
     *
     * @param type  Tipo da declarao que vai ser adicionada. Pode ser um dos
     * quatro tipos abaixo:
     * <blockquote>
     * 	      JaTSNode.FIELD_DECLARATION <br>
     * 	      JaTSNode.METHOD_DECLARATION <br>
     * 	      JaTSNode.CONSTRUCTOR_DECLARATION <br>
     * 	      JaTSNode.IMPORT_DECLARATION <br>
     * </blockquote>
     */
    public void addDeclaration(String strDec, String varName, int type,
                               ResultSet rset) {
        rset.addDeclaration(strDec, varName, type);
    }


    /**
     * Retorna os mtodos presentes no arquivo passado como parametro
     */
    public JMethodDeclaration[] getMethods(InputStream input) throws ParseException {

        Reader reader = null;
        JCompilationUnit compUnit = null;
        JMethodDeclaration[] retorno = null;

        if (input != null) {

            reader = new InputStreamReader(input);
            compUnit = (JCompilationUnit) jatsio.parse(reader);
            retorno = jatsio.getMethods(compUnit, false);

        } else {
            throw new IllegalArgumentException();

        }
        return retorno;

    }

    /**
     * Retorna os mtodos presentes no arquivo passado como parametro
     */
    public JFieldDeclaration[] getFields(InputStream input)
            throws ParseException {

        Reader reader = null;
        JCompilationUnit compUnit = null;
        JFieldDeclaration[] retorno = null;
        if (input != null) {
            reader = new InputStreamReader(input);
            compUnit = (JCompilationUnit) jatsio.parse(reader);
            retorno = jatsio.getFields(compUnit, false);
        }
        else {
            throw new IllegalArgumentException();
        }
        return retorno;

    }


    /**
     * Remove o mapeamento feito ao Node do resultSet
     */
    public void removeResult(INode var, ResultSet rset) {
        rset.remove(var);
    }

}
