/*
 * JaTS Project - Java Transformation System
 * http://www.cin.ufpe.br/~jats
 *  
 * Copyright  2005  Informatics Center - Federal University of Pernambuco
 * 
 * JaTS is a free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 */
package cin.jats.engine;

import cin.jats.engine.parser.nodes.*;
import cin.jats.engine.parser.nodes.exceptions.NodesNotMatchedException;
import cin.jats.engine.util.ResultSet;


/**
 * Esta classe  responsvel por realizar o casamento entre o programa JaTS e o
 * programa Java e preeencher o resultSet com os mapeamentos realizados durante
 * o casamento
 *
 * @author Adeline de Sousa Silva
 * @author Fernando Castor
 */
public class JatsMatcher {

    /**
     * Construtor da classe
     */
    public JatsMatcher() {

    }

    /**
     * Realiza o casamento entre duas rvores sintticas a do programa JaTS e a
     * do programa Java e preenche o resultSet com os mapeamentos feitos durante
     * o casamento
     */
    public void match(ICompilationUnit left, ICompilationUnit javaSources,
                      ResultSet rset) throws IllegalArgumentException,
            cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, NodesNotMatchedException {

        if (left == null || javaSources == null) {
            throw new IllegalArgumentException("Entrada invalida");
        }

        left.match(javaSources, rset);
    }

    /**
     * Realiza o casamento entre dois ns o do programa JaTS e o do programa
     * Java e preenche o resultSet com os mapeamentos feitos durante
     * o casamento
     */
    public void match(INode left, INode javaSource,
                      ResultSet rset) throws IllegalArgumentException,
            cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, NodesNotMatchedException {

        if (left == null || javaSource == null) {
            throw new IllegalArgumentException("Entrada invalida");
        }

        left.match(javaSource, rset);
    }
}
