/*
 * JaTS Project - Java Transformation System
 * http://www.cin.ufpe.br/~jats
 *  
 * Copyright  2005  Informatics Center - Federal University of Pernambuco
 * 
 * JaTS is a free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 */
package cin.jats.engine;

import cin.jats.engine.parser.nodes.*;
import cin.jats.engine.parser.nodes.exceptions.*;
import cin.jats.engine.util.ResultSet;
import cin.jats.engine.visitors.ReplacementVisitor;


/**
 * Esta classe  responsvel por realizar as substituies das variveis que
 * aparecem do lado direito de uma transformao por valores que a estas esto
 * mapeadas no resultSet resultado do casamento entre o programa Java e o
 * programa JaTS. Alm dos valores mapeados durante o matching entre o programa
 * Java e o programa JaTS, outras variveis podem ter sido adicionadas ao
 * ResultSet. De forma que  possvel realizar a substituio de valores das
 * variveis que aparecem apenas no lado direito da transformao. Essas em
 * geral, dizem respeito a gerao de cdigo.
 *
 * @author Adeline de Sousa Silva
 * @author Fernando Castor
 */
public class JatsReplacer {

    // visitor responsvel por fazer as substituies
    private ReplacementVisitor visitor;

    /**
     * Construtor
     */
    public JatsReplacer() {

        this.visitor = new ReplacementVisitor();

    }


    /**
     * Realiza a substituio em right (lado direito da transformao) das
     * variveis que l existirem e estiverem mapeadas a valores em 'results'.
     */
    public void replace(INode right, ResultSet results)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {


        if (right == null) {
            throw new IllegalArgumentException("lado direito da transformacao nulo");
        }
        this.visitor = new ReplacementVisitor(results);
        right.accept(this.visitor, results);
    }

    /**
     * Realiza a substituio em right (lado direito da transformao) das
     * variveis que l existirem e estiverem mapeadas a valores em 'results'.
     */
    public void replace(ICompilationUnit right, ResultSet results)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {


        if (right == null) {
            throw new IllegalArgumentException("lado direito da transformacao nulo");
        }
        this.visitor = new ReplacementVisitor(results);
        right.accept(this.visitor, results);
    }
}
