/*
 * JaTS Project - Java Transformation System
 * http://www.cin.ufpe.br/~jats
 *  
 * Copyright  2005  Informatics Center - Federal University of Pernambuco
 * 
 * JaTS is a free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 */
package cin.jats.engine;

import cin.jats.engine.parser.nodes.*;
import cin.jats.engine.util.NodeList;
import cin.jats.engine.util.NodeTable;

/**
 * 
 * @author Adeline de Sousa
 * @version 12/08/2002
 */
public class NodeHandler {


    /* Esta classe traz mtodos para , dado uma tabela do tipo NodeTable, retornar
    os identificadores, names, e outros ns */

    private NodeTable table;

    /**
     * Construtor da classe. Recebe a tabela gerada no processo de parsing.
     */
    public NodeHandler(NodeTable newtable) {
        this.table = newtable;
    }


    /**
     * Retorna um array contendo todos os identificadores presentes no arquivo
     * analisado e que se encontra na tabela gerada no processo de parsing.
     *
     * @param var Diz se os ns retornados devem ser variveis ou no
     */
    public JIdentifier[] getIdentifiers(boolean var) {

        JIdentifier[] retorno = null;
        INode[] tmpRetorno = this.getNodes(JaTSNode.IDENTIFIER, var);
        if (tmpRetorno != null) {
            retorno = new JIdentifier[tmpRetorno.length];
            for (int i = 0; i < tmpRetorno.length; i++) {
                // o instanceof no  necessrio, pois em tmpRetorno s existem
                // ns do tipo JIdentifier, devido ao parmetro passado.
                retorno[i] = (JIdentifier) tmpRetorno[i];

            }
        } else {
            throw new IllegalArgumentException("Argumento interno nulo em JatsParser.getIdentifiers");
        }

        return retorno;
    }

    /**
     * Retorna um array contendo todos os objetos do tipo <code>JName</code>
     * presentes no arquivo analisado e que se encontra na tabela gerada no
     * processo de parsing.
     *
     * @param var Diz se os ns retornados devem ser variveis ou no
     */
    public JName[] getNames(boolean var) {

        JName[] retorno = null;
        INode[] tmpRetorno = this.getNodes(JaTSNode.NAME, var);
        if (tmpRetorno != null) {
            retorno = new JName[tmpRetorno.length];
            for (int i = 0; i < tmpRetorno.length; i++) {
                // o instanceof no  necessrio, pois em tmpRetorno s existem
                // ns do tipo JName, devido ao parmetro passado.
                retorno[i] = (JName) tmpRetorno[i];

            }
        } else {
            throw new IllegalArgumentException("Argumento interno nulo em JatsParser.getNames");
        }

        return retorno;
    }


    public JLiteral[] getLiterals(boolean var) {
        JLiteral[] retorno = null;
        INode[] tmpRetorno = this.getNodes(JaTSNode.LITERAL, var);
        if (tmpRetorno != null) {
            retorno = new JLiteral[tmpRetorno.length];
            for (int i = 0; i < tmpRetorno.length; i++) {
                // o instanceof no  necessrio, pois em tmpRetorno s existem
                // ns do tipo JLiteral, devido ao parmetro passado.
                retorno[i] = (JLiteral) tmpRetorno[i];

            }
        } else {
            throw new IllegalArgumentException("Argumento interno nulo em JatsParser.getLiterals");
        }

        return retorno;

    }

    public JType[] getTypes(boolean var) {
        JType[] retorno = null;
        INode[] tmpRetorno = this.getNodes(JaTSNode.TYPE, var);
        if (tmpRetorno != null) {
            retorno = new JType[tmpRetorno.length];
            for (int i = 0; i < tmpRetorno.length; i++) {
                // o instanceof no  necessrio, pois em tmpRetorno s existem
                // ns do tipo JType, devido ao parmetro passado.
                retorno[i] = (JType) tmpRetorno[i];
            }
        } else {
            throw new IllegalArgumentException("Argumento interno nulo em JatsParser.getTypes");
        }

        return retorno;
    }

    public JModifierList[] getModifiers(boolean var) {
        JModifierList[] retorno = null;
        INode[] tmpRetorno = this.getNodes(JaTSNode.MODIFIER_LIST, var);
        if (tmpRetorno != null) {
            retorno = new JModifierList[tmpRetorno.length];
            for (int i = 0; i < tmpRetorno.length; i++) {
                // o instanceof no  necessrio, pois em tmpRetorno s existem
                // ns do tipo JModifierList, devido ao parmetro passado.
                retorno[i] = (JModifierList) tmpRetorno[i];
            }
        } else {
            throw new IllegalArgumentException("Argumento interno nulo em JatsParser.getModifiers");
        }

        return retorno;

    }

    public JFormalParameter[] getParameters(boolean var) {
        JFormalParameter[] retorno = null;
        INode[] tmpRetorno = this.getNodes(JaTSNode.FORMAL_PARAMETER, var);
        if (tmpRetorno != null) {
            retorno = new JFormalParameter[tmpRetorno.length];
            for (int i = 0; i < tmpRetorno.length; i++) {
                // o instanceof no  necessrio, pois em tmpRetorno s existem
                // ns do tipo JFormalParameter, devido ao parmetro passado.
                retorno[i] = (JFormalParameter) tmpRetorno[i];
            }
        } else {
            throw new IllegalArgumentException("Argumento interno nulo em JatsParser.getParameters");
        }

        return retorno;
    }

    public JPackageDeclaration[] getPackages(boolean var) {
        JPackageDeclaration[] retorno = null;
        INode[] tmpRetorno = this.getNodes(JaTSNode.PACKAGE_DECLARATION, var);
        if (tmpRetorno != null) {
            retorno = new JPackageDeclaration[tmpRetorno.length];
            for (int i = 0; i < tmpRetorno.length; i++) {
                // o instanceof no  necessrio, pois em tmpRetorno s existem
                // ns do tipo JPackageDeclaration, devido ao parmetro passado.
                retorno[i] = (JPackageDeclaration) tmpRetorno[i];
            }
        } else {
            throw new IllegalArgumentException("Argumento interno nulo em JatsParser.getPackages");
        }

        return retorno;
    }

    public JImportDeclaration[] getImports(boolean var) {
        JImportDeclaration[] retorno = null;
        INode[] tmpRetorno = this.getNodes(JaTSNode.PACKAGE_DECLARATION, var);
        if (tmpRetorno != null) {
            retorno = new JImportDeclaration[tmpRetorno.length];
            for (int i = 0; i < tmpRetorno.length; i++) {
                // o instanceof no  necessrio, pois em tmpRetorno s existem
                // ns do tipo ImportDeclaration, devido ao parmetro passado.
                retorno[i] = (JImportDeclaration) tmpRetorno[i];
            }
        } else {
            throw new IllegalArgumentException("Argumento interno nulo em JatsParser.getImports");
        }

        return retorno;
    }


    public JFieldDeclaration[] getFields(boolean var) {
        JFieldDeclaration[] retorno = null;
        INode[] tmpRetorno = this.getNodes(JaTSNode.FIELD_DECLARATION, var);
        if (tmpRetorno != null) {
            retorno = new JFieldDeclaration[tmpRetorno.length];
            for (int i = 0; i < tmpRetorno.length; i++) {
                // o instanceof no  necessrio, pois em tmpRetorno s existem
                // ns do tipo JFieldDeclaration, devido ao parmetro passado.
                retorno[i] = (JFieldDeclaration) tmpRetorno[i];
            }
        } else {
            throw new IllegalArgumentException("Argumento interno nulo em JatsParser.getFields");
        }

        return retorno;
    }


    public JMethodDeclaration[] getMethods(boolean var) {
        JMethodDeclaration[] retorno = null;
        INode[] tmpRetorno = this.getNodes(JaTSNode.METHOD_DECLARATION, var);
        if (tmpRetorno != null) {
            retorno = new JMethodDeclaration[tmpRetorno.length];
            for (int i = 0; i < tmpRetorno.length; i++) {
                // o instanceof no  necessrio, pois em tmpRetorno s existem
                // ns do tipo JMethodDeclaration, devido ao parmetro passado.
                retorno[i] = (JMethodDeclaration) tmpRetorno[i];
            }
        } else {
            throw new IllegalArgumentException("Argumento interno nulo em JatsParser.getMethods");
        }

        return retorno;
    }


    /**
     * Retorna as declaraes de construtor presentes no arquivo analisado.
     *
     * @param var Diz se os ns retornados devem ou no ser variveis
     */
    private JConstructorDeclaration[] getConstructors(boolean var) {
        JConstructorDeclaration[] retorno = null;
        INode[] tmpRetorno = this.getNodes(JaTSNode.CONSTRUCTOR_DECLARATION, var);
        if (tmpRetorno != null) {
            retorno = new JConstructorDeclaration[tmpRetorno.length];
            for (int i = 0; i < tmpRetorno.length; i++) {
                // o instanceof no  necessrio, pois em tmpRetorno s existem
                // ns do tipo Constructor, devido ao parmetro passado.
                retorno[i] = (JConstructorDeclaration) tmpRetorno[i];
            }
        } else {
            throw new IllegalArgumentException("Argumento interno nulo em JatsParser.getConstructors");
        }

        return retorno;

    }


    /**
     * Retorna um array contendo apenas os ns do tipo especificado pelo
     * parmetro <code>typeIdentifier</code>. Os ns retornados ou so todos
     * variveis ou no. Para tanto, basta especificar o valor do parmetro
     * <code>var</code>.
     * Os valores possveis para o parmetro typeIdentifier so os valores
     * possveis para o atributo typeIdentifier de cada n. Estes esto listados
     * na interface JaTSNode
     *
     * @param  typeIdentifier Tipo dos ns a serem retornados
     * @param  var  true, para variveis <br>
     *              false, caso contrrio
     *
     * @see cin.jats.engine.parser.nodes.JaTSNode
     */
    public INode[] getNodes(int typeIdentifier, boolean var) {
        NodeList nodes = null;
        INode[] retorno = null;
        if (var) {
            nodes = table.getNodeVarLine(typeIdentifier);
        } else {
            nodes = table.getNodeLine(typeIdentifier);
        }
        retorno = getNodeArray(nodes);
        return retorno;
    }


    /**
     * Converte um objeto do tipo NodeList em um array de ns.
     *
     * @param nodes NodeList a ser convertido em array
     *
     * @return Array contendo os objetos presentes na NodeList passada como
     *         parametro
     */
    private INode[] getNodeArray(NodeList nodes) {
        INode[] retorno;
        int size = nodes.size();
        retorno = new INode[size];
        INode tmpNode = null;
        for (int i = 0; i < size; i++) {
            tmpNode = nodes.elementAt(i);
            retorno[i] = tmpNode;
        }
        return retorno;
    }


    /**
     * Retorna todos as declaraes de atributo presentes no arquivo compilado,
     * inclusive variveis do tipo <code>FieldDeclaration</code>
     */
    public JFieldDeclarationSet getFieldDeclarationSet() {
        JFieldDeclarationSet fds = new JFieldDeclarationSet();
        JFieldDeclaration[] fields = this.getFields(false);
        JFieldDeclaration[] varFields = this.getFields(false);

        for (int i = 0; i < fields.length; i++) {
            fds.add(fields[i]);
        }
        for (int i = 0; i < varFields.length; i++) {
            fds.add(varFields[i]);
        }
        return fds;
    }


    /**
     * Retorna as declaraes de mtodo presentes no arquivo analisado. Tanto as
     * declaraes quanto as variveis do tipo <code>MethodDeclaration</code>
     */
    public JMethodDeclarationSet getMethodDeclarationSet() {
        JMethodDeclarationSet mds = new JMethodDeclarationSet();
        JMethodDeclaration[] methods = this.getMethods(false);
        JMethodDeclaration[] varMethods = this.getMethods(true);
        for (int i = 0; i < methods.length; i++) {
            mds.add(methods[i]);
        }
        for (int i = 0; i < varMethods.length; i++) {
            mds.add(varMethods[i]);
        }
        return mds;
    }

    /**
     * Retorna as declaraes de construtor presentes no arquivo analisado.
     * Tanto as declaraes quanto as variveis do tipo
     * <code>ConstructorDeclaration</code>
     */
    public JConstructorDeclarationSet getConstructorDeclarationSet() {
        JConstructorDeclarationSet cds = new JConstructorDeclarationSet();
        JConstructorDeclaration[] constructors = this.getConstructors(false);
        JConstructorDeclaration[] varConstructors = this.getConstructors(true);
        for (int i = 0; i < constructors.length; i++) {
            cds.add(constructors[i]);
        }
        for (int i = 0; i < varConstructors.length; i++) {
            cds.add(varConstructors[i]);
        }
        return cds;

    }

}
