/*
 * JaTS Project - Java Transformation System
 * http://www.cin.ufpe.br/~jats
 *  
 * Copyright  2005  Informatics Center - Federal University of Pernambuco
 * 
 * JaTS is a free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 */
package cin.jats.engine;


/**
 * 
 * @autor Adeline de Sousa
 * @author Gustavo Santos 
 * 
 * @version 20/01/2004
 */
public class ParserFactory {


    public static final int JATS_PARSER = 1;
    public static final int JAVA_PARSER = 2;

    private ParserFactory() {

    }


    /**
     * Creates a parser for the pattern language.
     * 
     * @return A parser for jats-tl
     */
    public static IParser createTemplateParser() {
        IParser retorno = null;
        retorno = JatsParser.getInstance();


        return retorno;
    }

    /**
     * Creates a parser for the Java language.
     * 
     * @return A parser for Java
     */
    public static IParser createSourceParser() {
        IParser retorno = null;
        retorno = JavaParser.getInstance();
        return retorno;
    }


    /**
     * Creates a parser instance based on the recieved code.
     * The possible types are: <br>
     * <ul>
     * <li>ParserFactory.JATS_PARSER</li>
     * <li>ParserFactory.JAVA_PARSER</li>
     * </ul>
     *
     * @return A language specific parser.
     */
    public static IParser createParser(int parserType) {
        IParser retorno = null;

        try {
           int parserInt = parserType;//Integer.parseInt(parserType);

           if (parserInt == ParserFactory.JAVA_PARSER) {
                retorno = JavaParser.getInstance();
            } else if (parserInt == ParserFactory.JATS_PARSER) {
                retorno = JatsParser.getInstance();
            }
            
        // esses catches nao fazem sentido pra mim
        } catch (Error e) {
            retorno = JatsParser.getInstance();
        } catch (Exception e) {
            retorno = JatsParser.getInstance();
        }

        return retorno;
    }

}
