/*
 * JaTS Project - Java Transformation System
 * http://www.cin.ufpe.br/~jats
 *  
 * Copyright  2005  Informatics Center - Federal University of Pernambuco
 * 
 * JaTS is a free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 */
package cin.jats.engine;

import java.util.Iterator;

import cin.jats.engine.parser.ParseException;
import cin.jats.engine.parser.nodes.*;
import cin.jats.engine.util.JaTSIterator;
import cin.jats.engine.util.ResultSet;
import cin.jats.engine.util.Util;


/**
 * Esta classe  responsvel por realizar um pr-processamento no ResultSet a
 * ser usado pelo replacement visitor e tambm por transformar os tipos
 * QualitiCoder em tipos JaTS
 * 
 * @author Adeline de Sousa Silva
 * 
 * @version 20/03/2002
 */
public class PrePrettyPrinter {

	public PrePrettyPrinter() {

	}

	public ResultSet processResultSet(ResultSet results) {
		JaTSNode var = null;
		String value = "";
		Object tmp = null;
		Object tmp2 = null;

		if (results != null) {
			Iterator it = results.getVariables();
			while (it.hasNext()) {
				tmp = it.next();
				if (tmp instanceof INode) {
					var = (JaTSNode) tmp;
					tmp2 = results.get(var);

					// se o objeto estiver mapeado a uma String, ento substitui
					// o valor
					// da String pelo n correspondente
					if (tmp2 instanceof String) {
						value = (String) tmp2;
						this.replaceValue(var, value, results);
					}
				}
			}
		} else {
			throw new IllegalArgumentException("Parametro results nulo");
		}
		return results;

	}

	/**
	 * Replaces the old value of <code>var </code> with the JaTSNode
	 * correspondent to apply
	 * 
	 * @param var
	 *            Varivel
	 * @param val
	 *            String que representa o valor associado a <code>var</code>
	 * @param results
	 *            ResultSet que contem a varivel e que ter seu par <code>
	 *                     var-valor</code>
	 *            modificado
	 */
	private void replaceValue(INode var, String val, ResultSet results) {
		INode newValue = null;
		if ((var != null) && (Util.checkString(val)) && (results != null)) {
			try {
				newValue = JatsIO.parseString(var.getTypeIdentifier(), val);
				results.put(var, newValue);
			} catch (ParseException e) {
				throw new IllegalArgumentException(
						"Erro de parsing na String passada");
			}
		} else {
			throw new IllegalArgumentException("Argumentos nulos ou invlidos");
		}

	}

	/**
	 * Pre-processa a CompilationUnit, assumindo-se que se trata de um programa
	 * Java. As declaraes presentes no OtherDeclarationSet sero adicionadas
	 * no respectivo conjunto.  importante observar que, uma vez que se trata
	 * de um programa Java, OtherDeclarationset contem apenas declaraes do
	 * tipo: JFieldDeclaration, JConstructorDeclaration ou JMethodDeclaration
	 * 
	 * @param comp
	 *            CompilationUnit representando o programa Java a ser processado
	 *  
	 */
	public static void preprocess(ICompilationUnit comp) {
		if (comp instanceof JCompilationUnit) {
			JCompilationUnit compilationUnit = (JCompilationUnit) comp;
			OtherDeclarationsSet ods = null;
			JTypeDeclaration typeDec = null;
			JaTSIterator itTypes = compilationUnit.getTypeDeclarationList()
					.getIterator();
			JaTSIterator itOtherDecs = null;

			while (itTypes.hasNext()) {
				// pega a declarao de tipo atual
				typeDec = (JTypeDeclaration) itTypes.next();
				// pega o other declarations set desse tipo
				TypeBody body = typeDec.getBody();
				
//				CSTypeBody csBody = null;
				
//				if (body instanceof CSTypeBody) {
//					csBody = (CSTypeBody) body;
//				}

				ods = body.getOtherDeclarationsSet();
				// pega o iterator, pra poder navegar sobre esse conjunto
				itOtherDecs = ods.getIterator();
				INode temp;

				// pra cada elemento desse conjunto, verifica qual o seu tipo e
				// o
				// coloca no conjunto correspondente
				while (itOtherDecs.hasNext()) {
					temp = itOtherDecs.next();

					if (temp instanceof JMethodDeclaration) {
						body.addMethodDeclaration((JMethodDeclaration) temp);
					} else if (temp instanceof JFieldDeclaration) {
						body.addFieldDeclaration((JFieldDeclaration) temp);
					} else if (temp instanceof JConstructorDeclaration) {
						body
								.addConstructorDeclaration((JConstructorDeclaration) temp);
					} else if (temp instanceof JInitializer) {
						body.addInitializer((JInitializer) temp);
					} 
//					else if (temp instanceof IndexerDeclaration) {
//						csBody.addIndexerDeclaration((IndexerDeclaration) temp);
//					} else if (temp instanceof PropertyDeclaration) {
//						csBody.addPropertyDeclaration((PropertyDeclaration) temp);
//					} else if (temp instanceof StructDeclaration) {
//						csBody.addStructDeclaration((StructDeclaration) temp);
//					} else if (temp instanceof DelegateDeclaration) {
//						csBody.addDelegateDeclaration((DelegateDeclaration) temp);
//					}
				}
				body.setOtherDeclarationsSet(new OtherDeclarationsSet());

			}
		} 
//		else {
//			if (comp instanceof CSCompilationUnit) {
//				throw new UnsupportedOperationException();
//			}
//		}
	}
}