/*
 * JaTS Project - Java Transformation System
 * http://www.cin.ufpe.br/~jats
 *  
 * Copyright  2005  Informatics Center - Federal University of Pernambuco
 * 
 * JaTS is a free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 */
package cin.jats.engine;

import java.io.IOException;
import java.io.Writer;
import java.util.Properties;

//import qualiti.qcode.util.Configuration;

import cin.jats.engine.parser.nodes.exceptions.ExecutionException;
import cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException;
import cin.jats.engine.parser.nodes.INode;
import cin.jats.engine.visitors.PrettyPrintingVisitor;

public class PrettyPrinter {

    // instancia unica da classe
    private static PrettyPrinter instance;

    // prettyprinter propriamente dito
    private PrettyPrintingVisitor prettyPrinter;


    /**
     * Construtor da classe
     */
    private PrettyPrinter() {    	
    	//try {
         //   if (Configuration.isJavaAdapter()) {
            	this.prettyPrinter = new PrettyPrintingVisitor();
         //   }
         //   else {
         //   	this.prettyPrinter = new CSPrettyPrintingVisitor();
         //   }
        //} catch (Error e) {
        	//this.prettyPrinter = new CSPrettyPrintingVisitor();
       // }
    }
    
    public static PrettyPrinter getInstance() {
        if (instance == null) {
            instance = new PrettyPrinter();
        }
        return instance;
    }

    public void setPrettyPrinterProperties(Properties prop) {
        this.prettyPrinter.setProperties(prop);
    }


    /**
     * Realiza o pretty-printig da arvore cuja raiz  node, no writer passado
     * como parametro
     *
     * @param node   Raiz da arvore sintatica
     * @param output Writer que ira receber o resultado
     *
     * @exception  cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException
     * @exception  IllegalArgumentException
     * @exception  IOException
     */
    public void prettyPrint(INode node, Writer output)
            throws InconsistentNodeException, IllegalArgumentException, IOException,
            ExecutionException {

        if (node == null || output == null) {

            throw new IllegalArgumentException("node ou output sao null");

        }

        // passar aqui o PrintData que ser usado para extrair as informaes
        // de configurao do pp
        node.accept(prettyPrinter, null);

        prettyPrinter.getPrettyPrintingResults(output);

    }

    public void prettyPrint(INode node, Writer output,
                            boolean jatsTypesEnabled) throws InconsistentNodeException,
            IllegalArgumentException, IOException, cin.jats.engine.parser.nodes.exceptions.ExecutionException {
        prettyPrinter.setJatsTypesEnabled(jatsTypesEnabled);
        this.prettyPrint(node, output);
        prettyPrinter.setJatsTypesEnabled(true);
    }

}
