package cin.jats.engine.parser.jats;


public class JParseException extends ParseException {

    private int errorLine;

    /**
     * This constructor is used by the method "generateParseException"
     * in the generated parser.  Calling this constructor generates
     * a new object of this type with the fields "currentToken",
     * "expectedTokenSequences", and "tokenImage" set.  The boolean
     * flag "specialConstructor" is also set to true to indicate that
     * this constructor was used to create this object.
     * This constructor calls its super class with the empty string
     * to force the "toString" method of parent class "Throwable" to
     * print the error message in the form:
     *     ParseException: <result of getMessage>
     */
    public JParseException(Token currentTokenVal, int[][] expectedTokenSequencesVal,
                           String[] tokenImageVal, int errorLine) {
        super(currentTokenVal, expectedTokenSequencesVal, tokenImageVal);
        this.errorLine = errorLine;
    }


    public JParseException(ParseException pe, int line) {
        if (pe != null) {
            this.expectedTokenSequences = pe.expectedTokenSequences;
            this.currentToken = pe.currentToken;
            this.tokenImage = pe.tokenImage;
            this.specialConstructor = pe.specialConstructor;
        }
        this.errorLine = line;
    }


    public JParseException(String msg, ParseException pe) {
        super(msg);
        if (pe != null) {
            this.expectedTokenSequences = pe.expectedTokenSequences;
            this.currentToken = pe.currentToken;
            this.tokenImage = pe.tokenImage;
            this.specialConstructor = pe.specialConstructor;
        }
    }


    public int getErrorLine() {
        return errorLine;
    }

    public void setErrorLine(int errorLine) {
        this.errorLine = errorLine;
    }

    /**
     * This method has the standard behavior when this object has been
     * created using the standard constructors.  Otherwise, it uses
     * "currentToken" and "expectedTokenSequences" to generate a parse
     * error message and returns it.  If this object has been created
     * due to a parse error, and you do not catch it (it gets thrown
     * from the parser), then this method is called during the printing
     * of the final stack trace, and hence the correct error message
     * gets displayed.
     */
    public String getMessage() {
        String msg = "";
        msg = "Parsing error detected at line: " + errorLine + "\n";
        msg += super.getMessage();
        return msg;
    }
}
