/**
 * Qualiti Software Processes
 *
 * Qualiti Coder
 *
 * Tipo: cin.jats.engine.parser.ParserHelper
 *
 * Esta classe contm mtodos que auxiliam a construo da rvore sinttica
 * do JaTS/ C#TS
 *
 * Autor: Adeline de Sousa
 * Versao: 16/06/2004
 */
package cin.jats.engine.parser.jats;

import cin.jats.engine.parser.nodes.*;

import cin.jats.engine.util.NodeList;

import java.util.Vector;

/**
 * Esta classe contm mtodos que auxiliam a construo da rvore sinttica
 * do JaTS
 */
public class ParserHelper {


    /**
     * Monta uma String a partir dos tokens inicial e final
     */
    public static String generateString(Token first, Token last) {

        String resultString = "";
        Token current = first;
        if (first != null && last != null) {
            while (current != null && !current.equals(last.next)) {
                Token special = current.specialToken;
                String antesDoToken = "";
                while (special != null) {
                    antesDoToken = special.image + antesDoToken;
                    special = special.specialToken;
                }

                resultString = resultString + antesDoToken + current.image;
                current = current.next;
            }
            resultString = resultString.trim();
        }
        return resultString;
    }

    /**
     * Monta uma String  formada pelos "Special Tokens" , ou seja, pelos tokens
     * correspondentes a comentrios, que se encontrem entre os tokens inicial
     * e final
     */
    public static String generateSpecialTokenString(Token first, Token last) {

        String resultString = "";
        Token current = first;
        if (first != null && last != null) {
            while (current != null && !current.equals(last.next)) {
                Token special = current.specialToken;
                String antesDoToken = "";
                while (special != null) {
                    antesDoToken = special.image + antesDoToken;
                    special = special.specialToken;
                }

                resultString = resultString + antesDoToken;
                current = current.next;
            }

            resultString = resultString.trim();
        }
        return resultString;
    }


    /**
     * Metodo auxiliar para fazer a arvore de uma expresso.
     *
     * @param theOperators
     * @param firstNode
     * @param leftHandExpressions
     * @return
     */
    public static JExpression makeBinaryExpressionTree(Vector theOperators,
                                                       JExpression firstNode,
                                                       Vector leftHandExpressions) {

        JExpression retorno = null;
        if (leftHandExpressions.size() > 0) {
            try {
                retorno = buildBinaryExpressionTree(theOperators, firstNode,
                        leftHandExpressions);
            } catch (IllegalArgumentException e) {

            }
        } else {
            retorno = firstNode;
        }
        return retorno;
    }

    /**
     * Mtodo que contri a rvore realmente.
     *
     * @param operadores
     * @param firstNode
     * @param listOfNodes
     * @return
     * @throws IllegalArgumentException
     */
    private static JExpression buildBinaryExpressionTree(Vector operadores,
                                                         JExpression firstNode, Vector listOfNodes)
            throws IllegalArgumentException {

        JExpression retorno = null;
        if (firstNode != null && listOfNodes != null && listOfNodes.size() > 0) {
            int numOfRightHandExpressions = listOfNodes.size();
            JBinaryExpression first = new JBinaryExpression(), current = null;
            int currOperator = 0;

            if (operadores.elementAt(0) instanceof Integer) {
                currOperator = ((Integer) operadores.elementAt(0)).intValue();
            }

            operadores.removeElementAt(0);
            first.setOperator(currOperator);
            first.setLeftHandOperand(firstNode);
            current = first;

            while (numOfRightHandExpressions > 1) {
                JBinaryExpression tempBin = new JBinaryExpression();
                Object tempObj = listOfNodes.elementAt(0);
                listOfNodes.removeElementAt(0);

                if (operadores.elementAt(0) instanceof Integer) {
                    currOperator = ((Integer) operadores.elementAt(0)).intValue();
                }
                operadores.removeElementAt(0);

                tempBin.setOperator(currOperator);
                if (tempObj instanceof JExpression) {
                    tempBin.setLeftHandOperand((JExpression) tempObj);
                } else {
                    throw new IllegalArgumentException("Um dos objetos na lista" +
                            " de expressoes nao eh uma expressao.");
                }
                current.setRightHandOperand(tempBin);
                current = tempBin;
                numOfRightHandExpressions--;
            }
            current.setRightHandOperand((JExpression) listOfNodes.elementAt(0));
            retorno = first;

        }

        return retorno;
    }

    /**
     * Mtodo auxiliar para a gerao de rvores sintticas na produo
     * PrimaryExpression. Este mtodo  responsvel por gerar um n
     * correspondendo a um acesso a  atributo.
     * Para tanto,  necessrio passar o objeto onde est sendo feito o
     * acesso e o atributo que est sendo acessado.
     *
     * @param  referencedObject  Expresso que representa o objeto onde o
     *         atributo est sendo acessado
     * @param  fieldName  Atributo sendo acessado.
     *
     * @return Um objeto do tipo FieldAccess, representando este acesso a
     *          atributo
     */
    public static JExpression generateFieldAccess(JaTSNode referencedObject,
                                                  JIdentifier fieldName,
                                                  int indentLevel) {

        if (fieldName == null)
            throw new IllegalArgumentException();

        JFieldAccess fieldAcc2 = new JFieldAccess();
        if (referencedObject != null) {
            fieldAcc2.setReferencedObject(referencedObject);
        }
        fieldAcc2.setFieldName(fieldName);
//        fieldAcc2.setIndentLevel(indentLevel);
        return fieldAcc2;
    }

    /**
     * Mtodo auxiliar para a gerao de rvores sintticas na produo
     * PrimaryExpression. Este mtodo  responsvel por gerar um n
     * correspondendo a uma invocao de mtodo.
     */
    public static JExpression generateMethodInvocation(INode referencedObject,
                                                       JExpressionList expList,
                                                       int indentLevel) throws ParseException {

        if (referencedObject == null || expList == null)
            throw new IllegalArgumentException();

        JMethodInvocation methInv = null;
        boolean ehMetodo = referencedObject instanceof JMethodInvocation;

        if (!ehMetodo && !(referencedObject instanceof JIdentifier)) {
            throw new ParseException("Erro no metodo generateMethodInvocation");
        }

        if (ehMetodo) {
            methInv = (JMethodInvocation) referencedObject;
        } else {
            JIdentifier tempNome = (JIdentifier) referencedObject;
            methInv = new JMethodInvocation();
            JName nomeMetodo = new JName();
            nomeMetodo.addIdentifier(tempNome);
            methInv.setMethodName(nomeMetodo);
        }
        methInv.setArgumentList(expList);
//        methInv.setIndentLevel(indentLevel);
        return methInv;
    }


    /**
     * Mtodo auxiliar para a gerao de rvores sintticas na produo
     * PrimaryExpression. Este mtodo  responsvel por gerar um n
     * correspondendo a uma invocao de mtodo.
     */
    public static JExpression generateArrayAccess(JaTSNode referencedObject,
                                                  JExpressionList expressionList,
                                                  int indentLevel) {

        JArrayAccess arrayAccess = new JArrayAccess();
        if (referencedObject == null || expressionList == null) {
            throw new IllegalArgumentException();
        }
        arrayAccess.setArrayName(referencedObject);
        arrayAccess.setIndexers(expressionList);
//        arrayAccess.setIndentLevel(indentLevel);

        return arrayAccess;
    }


    /**
     * Retorna a expresso primria correspondente a lista de no passada como
     * parametro. O que indica que tipo de expresso deve ser retornada,  o
     * seu ltimo elemento.
     * <table>
     * <tr><td><b>Ultimo elemento</b></td>
     * <td><b>Expressao Primaria Resultante</b></td> </tr>
     * <tr><td>Identificador</td> <td>Acesso a atributo</td> </tr>
     * <tr><td>Exp. de alocao</td> <td>Exp. de alocao</td> </tr>
     * <tr><td>Lista de argumentos</td> <td>Invocao de mtodo</td> </tr>
     * <tr><td>Lista de expresses</td> <td>Acesso a elemento de array</td> </tr>
     * </table>
     *
     * Ex: A lista de ns: <br>
     * <pre>
     * [elem1, getX, (), getY, (), array,[0]]
     * </pre>
     * Gera a seguinte expresso: <br>
     * <pre>
     * ((elem1.getX()).getY()).array[0];
     * </pre>
     *
     * @param nodes   Lista de ns a partir da qual ser gerada a expresso
     * @return   Expresso primria resultante da anlise da lista de ns
     *           passada como parametro
     * @throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException
     */
    public static JExpression getPrimaryExpression(NodeList nodes)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException {

        INode lastElement = null;
        if (nodes != null) {

            lastElement = nodes.last();
            if (nodes.size() > 1) {
                // o elemento resultante eh uma invocao de mtodo ou um acesso a array
                if (lastElement instanceof JExpressionList) {
                    JExpressionList expList = (JExpressionList) lastElement;
                    // eh uma invocao de mtodo
                    if (expList.getType() == JExpressionList.ARGUMENTS) {
                        NodeList nl = dropLastElement(nodes);
                        INode name = nl.last();
                        return new JMethodInvocation(getPrimaryExpression(dropLastElement(nl)), JName.unwrapName(name), expList);
                    }

                    //  um acesso a array
                    else {
                        return new JArrayAccess(getPrimaryExpression(dropLastElement(nodes)), expList);
                    }
                }
                // o elemento resultante eh um acesso a atributo
                else if (lastElement instanceof JIdentifier) {
                    return new JFieldAccess(getPrimaryExpression(dropLastElement(nodes)), (JIdentifier) lastElement);
                }

                // o elemento resultante  uma expresso de alocao
                else if (lastElement instanceof JAllocationExpression) {
                    JAllocationExpression allocExp = (JAllocationExpression) lastElement;
                    allocExp.setReferencedObject(getPrimaryExpression(dropLastElement(nodes)));
                    return allocExp;
                }
            }
            else if (nodes.size() == 1) {
                return (JExpression) nodes.elementAt(0);
            }
            else {
                return null;
            }

        } else {
            throw new IllegalArgumentException("NodeList null");
        }

        return null;
    }

    /**
     * Retorna uma lista de ns cujo ltimo elemento foi removido. Caso a lista
     * contenha apenas um elemento, retorna uma lista vazia.
     *
     * @param nodes Lista da qual se quer obter a subsequencia q vai do elemento
     *              zero at o elemento na posicao size - 1.
     *
     * @return Lista passada como parametro, sem o ltimo elemento
     *
     * @throws IllegalArgumentException Levantada caso a lista passada como
     *               parametro seja nula ou tenha tamanho zero.
     */
    private static NodeList dropLastElement(NodeList nodes) {
        if (nodes != null &&  nodes.size() > 1) {
            return nodes.subSequence(0, nodes.size() - 1);
        }
        else if (nodes != null && nodes.size() == 1){
            return new NodeList();
        }
        else {
            throw new IllegalArgumentException("");
        }
    }
}
