/*
 * JaTS Project - Java Transformation System
 * http://www.cin.ufpe.br/~jats
 *  
 * Copyright  2005  Informatics Center - Federal University of Pernambuco
 * 
 * JaTS is a free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 */
package cin.jats.engine.parser.nodes;

import cin.jats.engine.util.JaTSIterator;
import cin.jats.engine.util.NodeList;
import cin.jats.engine.util.ResultSet;
import cin.jats.engine.visitors.IVisitor;
import cin.jats.engine.parser.nodes.exceptions.*;
import cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException;
import cin.jats.engine.parser.nodes.exceptions.IllegalMatchingException;
import cin.jats.engine.parser.nodes.exceptions.NodesNotMatchedException;
import cin.jats.engine.parser.nodes.exceptions.ExecutionException;

import java.util.Iterator;

/**
 * Classe que  guarda as informacoes a respeito das declaracoes  de  metodo.
 * Essa classe  formada basicamente por tres conjuntos : um pra
 * metodos, outro para declaracoes do tipo <b>TypeDeclaration</b> e outra
 * para declaracoes do tipo <b>TypeDeclarationSet</b>
 *
 * @author Adeline de Sousa Silva
 * @version 21/02/2003
 */
public class BodyTypeDeclarations extends AbstractNode {

    // metodos do programa java
    private TypeDeclarationSet typesSet;

    // declaracao no programa jats que casa com um unico metodo do programa
    // java Ex: #class:ClassDeclaration , #interface:InterfaceDeclaration
    private NodeList typeVarSet;

    // declaracao no programa jats que casa com um conjunto de  metodos no do
    // programa java #classes : ClassDeclarationSet
    private NodeList typesVarSet;


    public BodyTypeDeclarations() {
        this.typesSet = new TypeDeclarationSet();
        this.typeVarSet = new NodeList();
        this.typesVarSet = new NodeList();
    }

    /**
     * Adiciona uma declaracao de metodo ao conjunto typesSet
     *
     * @param methDec Declaracao de metodo a ser adicionada
     */
    public void addTypeDeclaration(ITypeDeclaration methDec)
            throws IllegalArgumentException {

        if (methDec != null) {
            this.typesSet.add(methDec);
        } else {
            throw new IllegalArgumentException("methDec nulo");
        }
    }

    /**
     * Adiciona uma declaracao '#type:TypeDeclarationSet' ao conjunto typeVarSet
     *
     * @param types Declaracao '#type:TypeDeclarationSet' a ser adicionada
     */
    public void addTypesVarDeclaration(TypeDeclarationSet types)
            throws IllegalArgumentException {

        if ((types != null) && types.isVariable() &&
                types.getVariableType() == JaTSNode.TYPE_DECLARATION_SET) {
            this.typesVarSet.addElement(types);
        } else {
            throw new IllegalArgumentException("types nulo");
        }
    }

    /**
     * Adiciona uma declaracao '#type:TypeDeclaration' ao conjunto typeVarSet
     *
     * @param type Declaracao '#type:TypeDeclaration' a ser adicionada
     */
    public void addTypeVarDeclaration(ITypeDeclaration type)
            throws IllegalArgumentException {

        if ((type != null) && type.isVariable()) {
            this.typeVarSet.addElement(type);
        } else {
            throw new IllegalArgumentException("type nulo");
        }
    }

    /**
     * Retorna o conjunto de metodos do programa java
     *
     * @return Conjunto de metodos do programa java
     */
    public TypeDeclarationSet getTypesSet() {
        return this.typesSet;
    }

    /**
     * Retorna o conjunto de construcoes do tipo '#type:TypeDeclarationSet' do programa jats
     *
     * @return Conjunto de construcoes do tipo '#type:TypeDeclarationSet' do programa jats
     */
    public NodeList getTypesVarSet() {
        return this.typesVarSet;
    }

    /**
     * Retorna o conjunto de construcoes do tipo '#md:ITypeDeclaration' do programa jats
     *
     * @return Conjunto de construcoes do tipo '#md:ITypeDeclaration' do programa jats
     */
    public NodeList getTypeVarSet() {
        return this.typeVarSet;
    }


    /**
     * Seta o conjunto de metodos do programa java
     *
     * @param types Conjunto de metodos do programa java
     * @throws IllegalArgumentException - Levantada quando o conjunto passado
     *                                  for vazio  ou se houver algum elemento invlido no conjunto.
     */
    public void setTypesSet(TypeDeclarationSet types) {

        if (types != null) {
            JaTSIterator jit = types.getIterator();
            while (jit.hasNext()) {
                if (!(jit.next() instanceof ITypeDeclaration)) {
                    throw new IllegalArgumentException("Elemento invalido");

                }
            }
            this.typesSet = types;
        } else {
            throw new IllegalArgumentException("Conjunto de metodos nulo");
        }

    }

    /**
     * Seta o conjunto de construcoes do tipo 'BodyTypeDeclarations' do programa
     * jats
     *
     * @param typesSet Construcoes do tipo 'BodyTypeDeclarations' do
     *                 programa jats
     * @throws IllegalArgumentException - Levantada quando o conjunto passado
     *                                  for vazio  ou se houver algum elemento invlido no conjunto.
     */
    public void setTypesVarSet(NodeList typesSet) {

        if (typesSet != null) {
            Iterator it = typesSet.iterator();
            while (it.hasNext()) {
                if (!(it.next() instanceof TypeDeclarationSet)) {
                    throw new IllegalArgumentException("Elemento invalido");
                }
            }

            this.typesVarSet = typesSet;
        } else {
            throw new IllegalArgumentException("Conjunto metodos vazio");
        }

    }

    /**
     * Seta o conjunto de construcoes do tipo ITypeDeclaration do programa jats
     *
     * @param typeSet Construcoes do tipo ITypeDeclaration do programa jats
     * @throws IllegalArgumentException - Levantada quando o conjunto passado
     *                                  for vazio  ou se houver algum elemento invlido no conjunto.
     */
    public void setTypeVarSet(NodeList typeSet) {

        if (typeSet != null) {
            Iterator it = typeSet.iterator();
            while (it.hasNext()) {
                if (!(it.next() instanceof ITypeDeclaration)) {
                    throw new IllegalArgumentException("Elemento invalido");
                }
            }
            this.typeVarSet = typeSet;
        } else {
            throw new IllegalArgumentException("Conjunto de metodos nulo");
        }

    }

    /**
     * Mtodo responsvel por receber um <code>visitor</code>, aplic-lo ao n
     * atual e envi-lo atravs da rvore, invocando os mtodos accept de
     * <code>this.typesSet</code>, <code>this.typeVarSet</code> e
     * <code>this.typesVarSet</code>.
     *
     * @param visitor Um objeto do tipo <code>IVisitor</code>, funcionando de acordo
     *                com o padro de projeto homnimo, ou seja, implementa uma operao
     *                sobre vrios tipos diferentes de objetos fora do cdigo desses
     *                objetos. A partir dele ser invocado o mtodo <code>visit</code>,
     *                que recebe como argumento <code>this</code>.
     * @param data    Um objeto qualquer que ser utilizado pelo visitor.
     * @return Um objeto qualquer que possa vir a ser retornado pelo visitor.
     * @throws IllegalArgumentException  Levantada quando o parmetro
     *                                   qualquer um dos argumentos tiver valor <code>null</code>.
     * @throws InconsistentNodeException Levantada
     *                                   quando algum componente essencial do n tiver um valor,
     *                                   tornando o n, como um todo, inconsistente.
     * @throws ExecutionException        Levantada
     *                                   sempre que a execuo de um n falhar.
     */
    public Object accept(IVisitor visitor, Object data)
            throws InconsistentNodeException, IllegalArgumentException,
            ExecutionException {

        Object retorno = null;

        // Variveis que casam com vrias declaraes de metodos
        if (this.typesVarSet != null) {

            int num = typesVarSet.size() - 1;
            while (num >= 0) {
                this.typesVarSet.elementAt(num).accept(visitor, data);
                num--;
            }
        }

        // Variveis que casam com uma nica declarao de metodo
        if (this.typeVarSet != null) {
            int num = typeVarSet.size() - 1;

            while (num >= 0) {
                this.typeVarSet.elementAt(num).accept(visitor, data);
                num--;
            }
        }

        // Declaraes de metodo
        if (this.typesSet != null) {
            this.typesSet.accept(visitor, data);
        } else {
            throw new InconsistentNodeException(this);
        }

        // ... invoque o accept da declarao executvel associada a este n,
        // caso haja alguma...
        this.invokeAcceptOnExecutableDeclaration(visitor, data);
        retorno = visitor.visit(this, data);
        return retorno;
    }

    /**
     * Este mtodo realiza o casamento entre este n, que faz parte da rvore
     * sinttica de um programa JaTS, e o n fornecido como argumento, que faz
     * parte da rvore sinttica de um programa em Java. Para JaTS, dois ns
     * casam em duas situaes: quando eles so completamente idnticos
     * e quando o primeiro deles  uma varivel JaTS e, ou essa varivel ainda
     * no foi mapeada a nenhum valor no conjunto resultado do casamento
     * (<code>results</code>) ou o valor ao qual ela foi mapeada  idntico quele
     * ao qual ela est sendo mapeada agora (<code>node</code>). "primeiro", no contexto
     * acima, se refere ao n que faz parte do programa JaTS que est sendo
     * casado.
     *
     * @param node    O objeto do tipo <code>JaTSNode</code> com o qual <code>this</code> ser casado.
     * @param results O conjunto resultado do casamento.  usado para armazenar
     *                os pares <code>(varivel,cdigo)</code> que so gerados pelo
     *                casamento.
     * @throws IllegalArgumentException  Levantada quando o parmetro
     *                                   qualquer um dos argumentos tiver valor <code>null</code>.
     * @throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException Levantada
     *                                   quando algum componente essencial do n tiver um valor,
     *                                   tornando o n, como um todo, inconsistente.
     * @throws NodesNotMatchedException  Lanada sempre que o casamento falhar
     *                                   porque <code>node</code> no  um n compatvel com <code>this</code>, ou
     *                                   seja, nem <code>this</code>  uma varivel no casada nem <code>this</code> 
     *                                   idntico a <code>this</code>.
     */
    public void match(INode node, ResultSet results)
            throws IllegalArgumentException, cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException,
            NodesNotMatchedException {

        if (results == null) {
            throw new IllegalArgumentException("results ou node sao nulos");
        }
        if (node == null || !(node instanceof BodyTypeDeclarations)) {
            throw new NodesNotMatchedException(NodesNotMatchedException.NODES_HAVE_DIFFERENT_TYPES_MSG,
                    this, node);
        }

        BodyTypeDeclarations typeDecSet = (BodyTypeDeclarations) node;
        TypeDeclarationSet nodeTypeDecs =
                (TypeDeclarationSet) typeDecSet.getTypesSet().clone();
        if (nodeTypeDecs == null) {
            throw new InconsistentNodeException(typeDecSet);
        }
        int nodeTypeDecsSize = nodeTypeDecs.size();
        boolean achou = false;


        // Casamento das declaraes do tipo varivel:ITypeDeclaration com os
        // metodos que ainda esto em nodeTypeDecs. Primeiro so casadas as
        // variveis que J FORAM mapeadas a algum valor. Isso  necessrio para
        // garantir que nenhuma declarao no fonte Java  mapeada a mais de uma
        // varivel.
        int k = 0;
        int max3 = this.typeVarSet.size();

        // Lista de variveis do programa JaTS que ainda no foram casados
        NodeList unmatchedTypeDecVars = new NodeList();

        // Lista de variveis do tipo TypeDeclarationSet do programa JaTS que
        // ainda no foram casadas.
        NodeList unmatchedTypeDecSetVars = new NodeList();

        // Com essa implementao padro,  necessrio verificar essa condio
        // para garantir que nenhuma varivel em this.typeVarSet
        // ficar sem ser casada.
        if (max3 > nodeTypeDecs.size()) {
            throw new NodesNotMatchedException(NodesNotMatchedException.NODES_HAVE_DIFFERENT_SIZES_MSG,
                    this, typeDecSet);
        } else {
            while (k < max3) {
                JaTSNode matchesSingleType = (JaTSNode) this.typeVarSet.elementAt(k);
                if (!matchesSingleType.isVariable() ||
                        !(matchesSingleType.getVariableType() ==
                        JaTSNode.CLASS_DECLARATION
                        ||
                        matchesSingleType.getVariableType() ==
                        JaTSNode.INTERFACE_DECLARATION) ||
                        !(matchesSingleType instanceof ITypeDeclaration)) {
                    throw new InconsistentNodeException(this);
                }
                // se a varivel j est casada, ento varre a lista de ns do
                // tipo ITypeDeclaration at encontrar o valor que lhe foi
                // mapeado e o remove da lista, para que este no possa ser
                // mapeado a outra varivel
                if (results.alreadyMatched(matchesSingleType)) {
                    int l = 0;
                    nodeTypeDecsSize = nodeTypeDecs.size();
                    achou = false;
                    Object tempNode = null;

                    tempNode = results.get(matchesSingleType);

                    if (!(tempNode instanceof ITypeDeclaration)) {
                        throw new IllegalMatchingException(matchesSingleType);
                    }
                    while ((l < nodeTypeDecsSize) && !achou) {
                        // Em nodeTypeDecs s existem ns do tipo
                        // ITypeDeclaration, por isso o cast sem instanceof
                        ITypeDeclaration tmp =
                                (ITypeDeclaration) nodeTypeDecs.elementAt(l);
                        try {
                            // Se encontrou o valor, remove o n da lista de
                            // declaraes de metodo do programa Java
                            tmp.match((ITypeDeclaration) tempNode, results);
                            nodeTypeDecs.removeElementAt(l);
                            achou = true;
                        } catch (NodesNotMatchedException e) {
                            l++;
                        }
                    }
                }
                // a varivel ainda no foi casada, adiciona-a  lista de
                // variveis que ainda no foram casadas
                else {
                    unmatchedTypeDecVars.addElement(matchesSingleType);
                }

                k++;
            }
        }

        // Casamento de variveis do tipo TypeDeclarationSet. Similar ao
        // anterior: apenas para remover da lista de declaraes de metodo
        // do tipo fonte Java as declaraes que j esto casadas a variveis.
        // Nesse caso, a variveis do tipo TypeDeclarationSet.
        int typesVarSetSize = this.typesVarSet.size();
        int n = 0;
        INode tempTypesVar = null;

        while (n < typesVarSetSize) {
            tempTypesVar = this.typesVarSet.elementAt(n);
            if (!(tempTypesVar instanceof TypeDeclarationSet)) {
                throw new IllegalMatchingException(tempTypesVar);
            }
            TypeDeclarationSet typeSetVar =
                    (TypeDeclarationSet) tempTypesVar;

            if (results.alreadyMatched(typeSetVar)) {
                Object tempMds = null;
                tempMds = results.get(typeSetVar);

                if (!(tempMds instanceof TypeDeclarationSet)) {
                    throw new IllegalMatchingException();
                }

                TypeDeclarationSet type = (TypeDeclarationSet) tempMds;
                JaTSIterator ji = type.getIterator();

                while (ji.hasNext()) {
                    INode typesVarDecSetTemp = ji.next();

                    int c1 = 0;
                    int maxTDS = nodeTypeDecs.size();
                    boolean achou2 = false;

                    while (c1 < maxTDS && !achou2) {
                        JaTSNode fDecTemp = (JaTSNode) nodeTypeDecs.elementAt(c1);

                        try {
                            typesVarDecSetTemp.match(fDecTemp, results);
                            nodeTypeDecs.removeElementAt(c1);
                            achou2 = true;
                        } catch (NodesNotMatchedException nnme) {
                            c1++;
                        }
                    }
                }
            } else {
                unmatchedTypeDecSetVars.addElement(typeSetVar);
            }
            n++;
        }


        // Casamento padro de declaraes de metodo no lado esquerdo da
        // transformao (this) com declaraes de metodo no programa fonte
        // (contidas em nodeTypeDecs).
        this.typesSet.match(nodeTypeDecs, results);

        // Casamento das variveis do tipo ITypeDeclaration que ainda no esto
        // mapeadas a nenhum valor.
        int m = 0;
        int max5 = unmatchedTypeDecVars.size();
        if (max5 <= nodeTypeDecs.size()) {
            while (m < max5) {
                JaTSNode tempVar = (JaTSNode) unmatchedTypeDecVars.elementAt(m);
                if (!(tempVar instanceof ITypeDeclaration)) {
                    throw new IllegalMatchingException(tempVar);
                }
                JaTSNode tempDec = (JaTSNode) nodeTypeDecs.elementAt(0);
                // tenta casar os ns restantes
                tempVar.matchesAsAVariable(tempDec, results,
                        JaTSNode.TYPE_DECLARATION);
                nodeTypeDecs.removeElementAt(0);
                m++;
            }
        } else {
            // no pode restar variveis do tipo "ITypeDeclaration" sem estar
            // casadas
            throw new NodesNotMatchedException("Some fields coudn't be matched " +
                    "in BodyTypeDeclararions", this, node);
        }


        // Casamento entre as declaraes do tipo varivel:TypeDeclarationSet
        // e os metodos restantes em nodeTypeDecs.
        int typesVarSize = unmatchedTypeDecSetVars.size();
        if (typesVarSize > 0) {
            // Se houver apenas uma varivel do tipo TypeDeclarationSet, ento
            // esta ser casada com as declaraes de metodo restante
            if (typesVarSize == 1) {
                INode matchesTheRest = unmatchedTypeDecSetVars.elementAt(0);

                if (matchesTheRest == null || !matchesTheRest.isVariable() ||
                        matchesTheRest.getVariableType() !=
                        JaTSNode.TYPE_DECLARATION_SET ||
                        !(matchesTheRest instanceof TypeDeclarationSet))
                    throw new InconsistentNodeException(this);

                TypeDeclarationSet typeDecsVar = (TypeDeclarationSet) matchesTheRest;

                typeDecsVar.matchesAsAVariable(nodeTypeDecs, results,
                        NO_TYPE);
            } else {
                // Se houver mais que uma varivel do tipo TypeDeclarationSet
                // no casada, o sistema no  capaz de decidir como proceder e
                // o casamento falha.
                TypeDeclarationSet unmatchedvarDecs = new TypeDeclarationSet(unmatchedTypeDecSetVars);
                throw new NodesNotMatchedException("The system could't decide " +
                        "how to perform the matching. (" + typesVarSize + " VarSets)",
                        unmatchedvarDecs, nodeTypeDecs);
            }
        } else if (nodeTypeDecs.size() > 0) {
            throw new NodesNotMatchedException("Some fields could not be matched " +
                    "(number of felds: " + nodeTypeDecs.size() + ")", this, typeDecSet);
        }
    }

    /**
     *  responsvel por processar o n atual. O processamento de ns do tipo
     * BodyTypeDeclarations se resume a invocar o mtodo <code>process</code> no
     * n correspondente  lista de declaraes de metodos filha do n.
     *
     * @param data Um objeto que pode conter informaes necessrias 
     *             execuo.
     * @return O resultado do processamento do n.
     * @throws ExecutionException        Se algum erro
     *                                   ocorrer durante o processamento das estruturas executveis do
     *                                   programa.
     * @throws IllegalArgumentException  Levantada
     *                                   caso o parmetro <code>data</code> seja <code>null</code> numa
     *                                   situao em que ele  necessrio.
     * @throws InconsistentNodeException Levantada se um n inconsistente (incompleto) for encontrado.
     *                                   Se algum dos componentes (filhos) essenciais ao correto
     *                                   funcionamento do n estiver faltando.
     */
    public Object process(Object data) throws ExecutionException,
            InconsistentNodeException, IllegalArgumentException {

        Object retorno = this;

        // Ns do tipo BodyTypeDeclarations nunca podem ser executveis.

        INode mDecList = this.getTypesSet();
        if (mDecList != null) {
            this.processNode(mDecList, data);
        }

        return retorno;
    }


}
