/*
 * JaTS Project - Java Transformation System
 * http://www.cin.ufpe.br/~jats
 *  
 * Copyright  2005  Informatics Center - Federal University of Pernambuco
 * 
 * JaTS is a free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 */
package cin.jats.engine.parser.nodes;

import cin.jats.engine.util.NodeTable;
import cin.jats.engine.util.ResultSet;
import cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException;
import cin.jats.engine.parser.nodes.exceptions.ExecutionException;

/**
 * 
 * @autor Adeline de Sousa 
 * @version 16/02/2004
 *
 */
public interface ICompilationUnit extends JaTSNode {

	boolean hasHeader();

	String getHeader();

	void setHeader(String header);

	JPreconditionDeclaration getPrecondition();

	/**
	 * Seta a pr-condio de uma transformao da qual essa unidade de
	 * compilao faz parte.
	 * 
	 * @param pre
	 *            A pr-condio.
	 * @throws IllegalArgumentException -
	 *             Levantada quando a pr-condio passada for <code>null</code>.
	 */
	void setPrecondition(JPreconditionDeclaration pre);

	// ve como estes dois mtodos vao ficar no caso de C#!
	String getTypeName();


	/**
	 * Esse mtodo foi adicionado para substituir o getPackage().
	 * 
	 * @return A localizacao desse arquivo java (no caso, o pacote ou
	 *         namespace).
	 *  
	 */
	String getLocation();

	/**
	 * Retorna a tabela de ns associado a esta CompilationUnit.
	 * 
	 * @return A tabela de ns associado a esta CompilationUnit
	 */
	NodeTable getTable();

	/**
	 * Seta a tabela de ns associado a esta CompilationUnit.
	 * 
	 * @param table
	 *            Tabela de ns associado a esta CompilationUnit
	 */
	void setTable(NodeTable table);

	/**
	 * Retorna o numero de declaraes de tipo presentes nesta CompilationUnit.
	 * 
	 * @return Numero de declaraes de tipo presentes nesta CompilationUnit
	 */
	int size();
	
	String print() throws InconsistentNodeException, ExecutionException;

	ITypeDeclaration getTypeDeclaration(int pos);

    boolean getPreconditionValue(ResultSet rset) throws ExecutionException,
            cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException;
	

}