/*
 * JaTS Project - Java Transformation System
 * http://www.cin.ufpe.br/~jats
 *  
 * Copyright  2005  Informatics Center - Federal University of Pernambuco
 * 
 * JaTS is a free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 */
package cin.jats.engine.parser.nodes;

import cin.jats.engine.util.Position;
import cin.jats.engine.visitors.IVisitor;
import cin.jats.engine.parser.nodes.exceptions.ExecutionException;


/**
 * Classe  representa as expresses da linguagem Java. Ela deveria ser abstrada,
 * pois no deveria poder ser instanciada diretamente, entretanto, como agora
 * podemos ter expresses executveis retornando expresses, faz-se necessrio
 * poder instanciar uma Expresso genrica.
 *
 * @author Fernando Castor
 * @author Adeline Sousa
 */
public abstract class JExpression extends AbstractNode {

    private Position position;

	protected JExpression() {
        this.setTypeIdentifier(JaTSNode.EXPRESSION);
    }

    /**
     * Mtodo responsvel por receber um <code>visitor</code>, aplic-lo ao n
     * atual e envi-lo atravs da rvore.
     *
     * @param visitor Um objeto do tipo <code>IVisitor</code>, funcionando
     *                de acordo com o padro de projeto homnimo, ou seja, implementa uma
     *                operao sobre vrios tipos diferentes de objetos fora do cdigo desses
     *                objetos. A partir dele ser invocado o mtodo <code>visit</code>,
     *                que recebe como argumento <code>this</code>.
     * @param data    Um objeto qualquer que ser utilizado pelo visitor.
     * @return Um objeto qualquer que possa vir a ser retornado pelo visitor.
     * @throws IllegalArgumentException  Levantada quando o parmetro
     *                                   qualquer um dos argumentos tiver valor <code>null</code>.
     * @throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException Levantada
     *                                   quando algum componente essencial do n tiver um valor,
     *                                   tornando o n, como um todo, inconsistente.
     * @throws ExecutionException        Levantada
     *                                   sempre que a execuo de um n falhar.
     */
    public Object accept(IVisitor visitor, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {

        Object retorno = null;
        // invoca o visitor da expresso executavel associada
        this.invokeAcceptOnExecutableDeclaration(visitor, data);

        //retorno = visitor.visit(this, data);
        return retorno;
    }

    public Object evaluate(Object data) throws ExecutionException {
        return this;
    }
    
	/**
	 * @return Returns the position.
	 */
	public Position getPosition() {
		return position;
	}

	/**
	 * @param position The position to set.
	 */
	public void setPosition(Position position) {
		this.position = position;
	}
    
}
