/*
 * JaTS Project - Java Transformation System
 * http://www.cin.ufpe.br/~jats
 *  
 * Copyright  2005  Informatics Center - Federal University of Pernambuco
 * 
 * JaTS is a free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 */
package cin.jats.engine.parser.nodes;

import cin.jats.engine.JatsIO;
import cin.jats.engine.parser.ParseException;
import cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException;
import cin.jats.engine.parser.nodes.exceptions.NodesNotMatchedException;
import cin.jats.engine.parser.nodes.exceptions.ExecutionException;
import cin.jats.engine.util.*;
import cin.jats.engine.visitors.IVisitor;
import cin.jats.engine.visitors.ReplacementVisitor;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

//CORRIGIR, para contemplar a nova sintaxe do JaTS
public class JNodeString extends JStatement {

    private String nodeValue;
    private ExpressionsMap expressions;
    private ExpressionsMap replacedExpressions;
    public final static int MIN_LENGTH = 4;
    public static final String LEFT_EXEC_DEC = "#<";
    public static final String RIGHT_EXEC_DEC = ">#";

    /**
     * Constructor for the JNodeString object
     */
    public JNodeString() {
    }


    /**
     * Constructor for the JNodeString object
     *
     * @param nodeValue Description of the Parameter
     */
    public JNodeString(String nodeValue) {
        this.nodeValue = nodeValue;
    }

    /**
     * Retorna a String que representa este objeto JNodeString
     *
     * @return Valor do JNodeString
     */
    public String getNodeValue() {
        return nodeValue;
    }


    /**
     * Description of the Method
     *
     * @param visitor Description of the Parameter
     * @param data    Description of the Parameter
     * @return Description of the Return Value
     */
    public Object accept(IVisitor visitor, Object data)
            throws InconsistentNodeException, IllegalArgumentException, ExecutionException {
        Object retorno = null;

        retorno = visitor.visit(this, data);

        return retorno;

    }

    /**
     * Este mtodo realiza o casamento entre este n, que faz parte da rvore
     * sinttica de um programa JaTS, e o n fornecido como argumento, que faz
     * parte da rvore sinttica de um programa em Java. Para JaTS, dois ns
     * casam em duas situaes: quando eles so completamente idnticos
     * e quando o primeiro deles  uma varivel JaTS e, ou essa varivel ainda
     * no foi mapeada a nenhum valor no conjunto resultado do casamento
     * (<code>results</code>) ou o valor ao qual ela foi mapeada  idntico
     * quele ao qual ela est sendo mapeada agora (<code>node</code>).
     * "primeiro", no contexto acima, se refere ao n que faz parte do programa
     * JaTS que est sendo casado.
     *
     * @param node    O objeto do tipo <code>JaTSNode</code> com o qual
     *                <code>this</code> ser casado.
     * @param results O conjunto resultado do casamento.  usado para armazenar
     *                os pares <code>(varivel,cdigo)</code> que so gerados pelo
     *                casamento.
     * @throws IllegalArgumentException Levantada quando o parmetro
     *                                  qualquer um dos argumentos tiver valor <code>null</code>.
     * @throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException
     *                                  Levantada
     *                                  quando algum componente essencial do n tiver um valor,
     *                                  tornando o n, como um todo, inconsistente.
     * @throws NodesNotMatchedException
     *                                  Lanada sempre que o casamento falhar
     *                                  porque <code>node</code> no  um n compatvel com <code>this</code>, ou
     *                                  seja, nem <code>this</code>  uma varivel no casada nem
     *                                  <code>this</code>  idntico a <code>this</code>.
     */
    public void match(INode node, ResultSet results) throws IllegalArgumentException, InconsistentNodeException, NodesNotMatchedException {
        //To change body of implemented methods use File | Settings | File Templates.
    }


    /**
     * Mtodo responsvel pelo processamento das expresses executveis
     * presentes neste n.
     *
     * @param data
     * @return
     * @throws cin.jats.engine.parser.nodes.exceptions.ExecutionException
     * @throws InconsistentNodeException
     * @throws IllegalArgumentException
     */
    public Object process(Object data) throws ExecutionException,
            cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException {
        Object retorno = this;

        if (this.isExecutable()) {
            retorno = processExecutableNode(data);
        } else {
            Map tmpExpressions = new HashMap();
            ReplacementVisitor rvis = new ReplacementVisitor((ResultSet) data);

            if (expressions != null && expressions.size() > 0) {
                Object tmpValue = null;
                Iterator it = expressions.getKeys();
                Iterator it2 = this.replacedExpressions.getKeys();
                String strExp2 = null;
                String strExp = null;
                INode tmpNode = null;
                String strResult = null;

                while (it.hasNext()) {
                    strExp = (String) it.next();
                    strExp2 = (String) it2.next();
                    Object obj = expressions.get(strExp);
                    if (obj instanceof INode) {
                        tmpNode = (INode) obj;

                        if (tmpNode.isExecutable()) {
                            JExecutableDeclaration execDec =
                                    (JExecutableDeclaration) tmpNode.getExecutableDeclaration();
                            execDec.initReplacement(rvis);
                            tmpValue = tmpNode.process(data);
                            strResult = tmpValue.toString();
                        }

                        if (tmpValue != null) {
                            tmpExpressions.put(strExp2, strResult);
                        }

                    } else if (obj instanceof String) {
                        tmpExpressions.put(strExp2, obj);
                    }
                }

                this.setNodeValue(VariableReplacer.substituirTagsString(tmpExpressions, this.nodeValue));
            }
        }

        return retorno;
    }


    /**
     * Recupera expresses executveis presentes na string e monta um mapeamento
     * delas com objetos JaTS que a representam.
     * Por enquanto, expresses executveis explicitamente tipadas necessitam de
     * um espao em branco aps o tipo para serem reconhecidas.
     *
     * @param text
     * @return ExpressionsMap Mapeamento de expresses em strings que as reresentam
     */
    public static ExpressionsMap getNodeExpressions(String text) {

        //mapeamento entre strings e expressoes executaveis correspondentes
        ExpressionsMap retorno = new ExpressionsMap();

        int indexLeftBrackets = -1;
        int indexRightBrackets = -1;
        int indexSpace = -1;
        int indexColon = -1;
        String tmp = null;
        String expType = null;
        String fullExp = null;
        int code = 0;
        INode tmpNode = null;
        boolean terminou = false;

        if (Util.checkString(text)) {
            tmp = text.trim();

            //remove comentarios e inutilidades similares
            tmp = Util.removeComments(tmp);

            while (!terminou) {
                //pega a posio do inicio da expresso
                indexLeftBrackets = tmp.indexOf(LEFT_EXEC_DEC);

                if (indexLeftBrackets > -1) {
                    //pega a posio do final da expresso
                    indexRightBrackets = tmp.indexOf(RIGHT_EXEC_DEC, indexLeftBrackets);

                    //pega a posicao da "tipagem", se houver
                    indexColon = tmp.indexOf(':', indexRightBrackets);

                    //caso a expresso seja tipada, tem que ter um espao depois
                    indexSpace = tmp.indexOf(' ', indexRightBrackets);

                    //se for tipada
                    if (indexColon > 0)
                        fullExp = tmp.substring(indexLeftBrackets, indexSpace);
                    //se NAO for tipada
                    else
                        fullExp = tmp.substring(indexLeftBrackets, indexRightBrackets + 2);//gas


                    if (indexRightBrackets > -1) {
                        //se for tipada e o tipo for DESTA mesmo
                        if (indexColon > -1 && indexColon == indexRightBrackets + 2) {
                            //pega o tipo
                            expType = tmp.substring(indexColon + 1, indexSpace);
                            code = AbstractNode.getTypeCode(expType);
                        }

                        //se no for tipada,  um IDENTIFIER
                        if (code == 0) {
                            code = JaTSNode.IDENTIFIER;
                        }

                        //faz o parsing da expresso, monta o n e bota no mapeamento
                        try {
                            tmpNode = JatsIO.parseString(code, fullExp);
                            retorno.put(fullExp, tmpNode);
                        } catch (ParseException e) {
                            
                            e.printStackTrace();
                        }
                    }

                    //se essa era tipada
                    if (indexColon > 0) {
                        tmp = tmp.substring(indexSpace + 1, tmp.length());
                    }
                    //se essa NAO era tipada
                    else {
                        tmp = tmp.substring(indexRightBrackets + 3, tmp.length());
                    }
                }

                //condio de parada
                if (tmp.length() < MIN_LENGTH || indexLeftBrackets == -1) {
                    terminou = true;
                }

            }

        }

        return retorno;
    }


    /**
     * Monta um mapeamento entre as variveis passadas e seus valores
     * correspondentes. Estes valores devem estar presentes no ResultSet
     * fornecido.
     *
     * @param rset
     * @param vv
     * @return Map
     */
    public static Map getMap(ResultSet rset, VectorVariables vv) {
        Iterator it = rset.getVariables();
        HashMap retorno = new HashMap();
        JaTSNode var = null;
        Object value = null;
        int count = 0;
        int max = vv.size();
        while (count < max && it.hasNext()) {
            var = (JaTSNode) it.next();
            if (var != null && vv.contains(var)) {
                value = rset.get(var);
                if (value != null) {
                    if (!Util.containsVar(retorno, var)) {
                        retorno.put(var.printVariable(), value.toString());
                        count++;
                    }
                }

            }
        }
        return retorno;
    }


    /**
     * Gets the variables attribute of the JNodeString class
     *
     * @param text Description of the Parameter
     * @return The variables value
     */
    public static VectorVariables getVariables(String text) {
        VectorVariables retorno = new VectorVariables();
        String tmp = text;
        if (Util.checkString(text)) {
            tmp = Util.removeComments(text);
            int stringSize = text.length();
            int indexHash = -1;
            int indexSpace = -1;
            int indexDot = -1;
            int endIndex = 0;
            int indexLeftBrackets = -1;
            int indexRightBrackets = -1;
            if (tmp.length() > 2 && !tmp.equals("{ }") && !tmp.equals("{}")) {
                if (tmp.startsWith("{") && tmp.endsWith("}")) {
                    tmp = tmp.substring(1, tmp.length() - 1);
                }
                //talvez tenha que matar o ";" aqui...
                tmp = tmp.trim();
            }
            boolean terminou = false;
            while (!terminou && tmp.length() >= 1) {

                indexHash = tmp.indexOf('#');
                indexLeftBrackets = tmp.indexOf(LEFT_EXEC_DEC);
                if (indexLeftBrackets > -1) {
                    indexRightBrackets = tmp.indexOf(RIGHT_EXEC_DEC, indexLeftBrackets);
                }
                if (indexHash != -1) {
                    indexDot = tmp.indexOf('.', indexHash);
                    indexSpace = tmp.indexOf(' ', indexHash);
                    if (indexDot != -1 && indexSpace != -1) {
                        endIndex = Math.min(indexDot, indexSpace);
                    } else if (indexDot != -1 && indexSpace == -1) {
                        endIndex = indexDot;
                    } else if (indexSpace == -1) {
                        endIndex = tmp.length();
                    } else {
                        endIndex = indexSpace;
                    }

                    // insere a varivel no vector de retorno, se ela no fizer
                    // parte de uma expresso executvel
                    if (!((indexLeftBrackets != -1 && indexRightBrackets != -1)
                            && (indexLeftBrackets < indexHash
                            && endIndex < indexRightBrackets))) {
                        insertVariable(tmp, indexHash, endIndex, retorno);
                    }

                    // atualiza o tamanho da string
                    stringSize = tmp.length();

                    try {
                        tmp = tmp.substring(endIndex + 1, stringSize);
                    } catch (Exception exception) {
                        tmp = "";
                    }
                } else {
                    terminou = true;
                }
            }

        }
        return retorno;
    }

    public void setExpressions(ExpressionsMap expressions) {
        this.expressions = expressions;
    }

    /**
     * Sets the nodeValue attribute of the JNodeString object
     *
     * @param nodeValue The new nodeValue value
     */
    public void setNodeValue(String nodeValue) {
        this.nodeValue = nodeValue;
    }

    public void setReplacedExpressions(ExpressionsMap replacedExpressions) {
        this.replacedExpressions = replacedExpressions;
    }

    /**
     * Retorna as variveis JaTS presentes no corpo do mtodo
     *
     * @param text Description of the Parameter
     * @return The blockVariables value
     */
    public static VectorVariables getBlockVariables(String text) {
        VectorVariables retorno = new VectorVariables();
        String tmp = text;
        if (Util.checkString(text)) {
            tmp = Util.removeComments(text);
            int stringSize = text.length();
            int indexHash = -1;
            int indexSpace = -1;
            int indexDot = -1;
            int endIndex = 0;
            int indexLeftBrackets = -1;
            int indexRightBrackets = -1;
            if (tmp.length() > 2 && !tmp.equals("{ }") && !tmp.equals("{}")) {
                if (tmp.startsWith("{") && tmp.endsWith("}")) {
                    tmp = tmp.substring(1, tmp.length() - 1);
                }
                tmp = tmp.trim();
            }
            boolean terminou = false;
            while (!terminou && tmp.length() >= 1) {
                indexHash = tmp.indexOf('#');
                indexLeftBrackets = tmp.indexOf(LEFT_EXEC_DEC);
                if (indexLeftBrackets > -1) {
                    indexRightBrackets = tmp.indexOf(RIGHT_EXEC_DEC, indexLeftBrackets);
                }
                if (indexHash != -1) {
                    indexDot = tmp.indexOf('.', indexHash);
                    indexSpace = tmp.indexOf(' ', indexHash);
                    if (indexDot != -1 && indexSpace != -1) {
                        endIndex = Math.min(indexDot, indexSpace);
                    } else if (indexDot != -1 && indexSpace == -1) {
                        endIndex = indexDot;
                    } else if (indexSpace == -1) {
                        endIndex = tmp.length();
                    } else {
                        endIndex = indexSpace;
                    }

                    // insere a varivel no vector de retorno, se ela no fizer
                    // parte de uma expresso executvel
                    if (!((indexLeftBrackets != -1 && indexRightBrackets != -1)
                            && (indexLeftBrackets < indexHash
                            && endIndex < indexRightBrackets))) {
                        insertVariable(tmp, indexHash, endIndex, retorno);
                    }

                    // atualiza o tamanho da string
                    stringSize = tmp.length();

                    try {
                        tmp = tmp.substring(endIndex + 1, stringSize);
                    } catch (Exception exception) {
                        tmp = "";
                    }
                } else {
                    terminou = true;
                }
            }

        }
        return retorno;
    }

    /**
     * Mtodo auxiliar para inserir a varivel no vector de retorno. Faz-se o
     * parsing da substring na posio indicada pelos ndices passados como
     * parmetro e, em seguida, insere-se o n retornado no vector.
     *
     * @param tmp       String contendo a substring que ser analisada e
     *                  convertida em varivel
     * @param indexHash Indice do hash que d incio ao nome da varivel
     * @param endIndex  Indice da posio que d fim ao nome da varivel
     * @param retorno   Vetor onde ser inserido o par texto - expresso
     */
    protected static void insertVariable(String tmp, int indexHash, int endIndex,
                                         VectorVariables retorno) {
        String fullName = null;
        int indexSemiColon = -1;
        int indexColon = -1;
        String varName = null;
        String strVarType = null;
        int code = JaTSNode.IDENTIFIER;
        fullName = tmp.substring(indexHash, endIndex);
        fullName = fullName.trim();
        indexSemiColon = fullName.indexOf(';');
        if (indexSemiColon != -1 && indexSemiColon < fullName.length()) {
            fullName = fullName.substring(0, indexSemiColon);
        }

        indexColon = fullName.indexOf(':');
        if (indexColon != -1 && indexColon < fullName.length()) {
            varName = fullName.substring(0, indexColon);
            strVarType = fullName.substring(indexColon + 1, fullName.length());
            code = AbstractNode.getTypeCode(strVarType);
            if (code == JaTSNode.BLOCK) {
                INode var = new JBlock();
                var.setVariableName(varName);
                var.setVariable(true);
                if (!retorno.contains(var)) {
                    retorno.addElement(var);
                }
            } else {
                try {
                    if (Util.checkString(fullName)) {
                        INode var = JatsIO.parseString(code, fullName);
                        if (!retorno.contains(var)) {
                            retorno.addElement(var);
                        }
                    }
                } catch (ParseException pex) {
                }
            }
        } else {
            varName = fullName;
            code = JaTSNode.IDENTIFIER;
            try {
                if (Util.checkString(fullName)) {
                    INode var = JatsIO.parseString(code, fullName);
                    if (!retorno.contains(var)) {
                        retorno.addElement(var);
                    }
                }
            } catch (Throwable t) {
                t.printStackTrace();
            }
        }
    }

}
