/*
 * JaTS Project - Java Transformation System
 * http://www.cin.ufpe.br/~jats
 *  
 * Copyright  2005  Informatics Center - Federal University of Pernambuco
 * 
 * JaTS is a free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 */
package cin.jats.engine.parser.nodes;

/**
 * Esta classe  uma abstrao para um JStatement da linguagem Java. Pode
 * representar desde um if, at uma invocao de mtodo
 * 
 * @author Adeline de Sousa
 * @version 28/03/2003
 */
public abstract class JStatement extends JExpression implements Commentable {

	// estruturas de deciso
	public static final int IF = 1;

	public static final int SWITCH = 2;

	// lao
	public static final int FOR = 4;

	public static final int FOREACH = 32;

	// invocao de mtodo, acesso a varivel, atribuio
	public static final int STATEMENT_EXPRESSION = 64;

	// jump statement: break, continue
	public static final int BREAK = 512;

	public static final int CONTINUE = 1024;

	// try
	public static final int TRY = 2048;

	// block
	public static final int BLOCK = 4098;


	
    // armazena o tipo do statement (se if, while, throw, etc.)
    private int statementType;

    // armazena o comentrio referente a este statement
    private JComment comment;

    /**
     * Construtor da classe
     */
    protected JStatement() {
        this.setTypeIdentifier(JaTSNode.STATEMENT);

    }

    /* (non-Javadoc)
	 * @see cin.jats.engine.parser.nodes.IStatement#getStatementType()
	 */
    public int getStatementType() {
        return statementType;
    }

    /* (non-Javadoc)
	 * @see cin.jats.engine.parser.nodes.IStatement#setStatementType(int)
	 */
    public void setStatementType(int statementType) {
        this.statementType = statementType;
    }

    /* (non-Javadoc)
	 * @see cin.jats.engine.parser.nodes.IStatement#validType(int)
	 */
    public boolean validType(int type) {
        return (((type & IF) > 0) ||
                ((type & SWITCH) > 0) ||
                ((type & FOR) > 0) ||
                ((type & STATEMENT_EXPRESSION) > 0) ||
                ((type & BREAK) > 0) ||
                ((type & CONTINUE) > 0) ||
                ((type & TRY) > 0));

    }

    /* (non-Javadoc)
	 * @see cin.jats.engine.parser.nodes.IStatement#hasComment()
	 */
    public boolean hasComment() {
        return (this.comment != null);
    }

    /* (non-Javadoc)
	 * @see cin.jats.engine.parser.nodes.IStatement#getComment()
	 */
    public JComment getComment() {
        return this.comment;
    }

    /* (non-Javadoc)
	 * @see cin.jats.engine.parser.nodes.IStatement#setComment(cin.jats.engine.parser.nodes.JComment)
	 */
    public void setComment(JComment javadoc) {
        if (javadoc != null) {
            this.comment = javadoc;
        }

    }

    /* (non-Javadoc)
	 * @see cin.jats.engine.parser.nodes.IStatement#setComment(java.lang.String)
	 */
    public void setComment(String javadoc) {
        if (javadoc != null && javadoc.length() > 0) {
            this.comment = new JComment(javadoc);

        }
    }

    /* (non-Javadoc)
	 * @see cin.jats.engine.parser.nodes.IStatement#getSignature()
	 */
    public Signature getSignature() {
        throw new UnsupportedOperationException("not available for statements");
    }


}
