/*
 * JaTS Project - Java Transformation System
 * http://www.cin.ufpe.br/~jats
 *  
 * Copyright  2005  Informatics Center - Federal University of Pernambuco
 * 
 * JaTS is a free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 */
package cin.jats.engine.parser.nodes;

import cin.jats.engine.util.JaTSIterator;

/**
 * Esta interface define mtodos genricos para uma coleo de elementos do tipo
 * JaTSNode
 * 
 * @author Adeline de Sousa
 *
 * @version  0.1 -  13/08/2001
 * @since JDK 1.3
 */
public interface JaTSCollection extends JaTSNode {

    /**
     * Adiciona um elemento  lista.
     *
     * @param node Um objeto qualquer do tipo <code>JaTSNode</code>.
     * @throws java.lang.IllegalArgumentException
     *          Levantada quando o argumento
     *          <code>node</code> for <code>null</code>.
     */
    public void add(INode node) throws IllegalArgumentException;


    /**
     * Retorna o n que se encontra na <code>pos</code>-sima posio da lista.
     *
     * @param pos Um inteiro correspondendo  posio desejada.
     * @return Um objeto do tipo <code>JaTSNode</code> que se encontra na posio
     *         desejada.
     */
    public INode elementAt(int pos);


    /**
     * Remove o elemento que se encontra na  posio <code>pos</code> desta lista.
     *
     * @param pos Um inteiro correspondendo  posio do n que deve ser
     *            removido.
     * @throws java.lang.IllegalArgumentException
     *          Levantada quando o
     *          argumento <code>pos</code> for negativo.
     */
    public void removeElementAt(int pos) throws IllegalArgumentException;

    /**
     * Adiciona o elemento na posio <code>pos</code> desta lista e d um shift
     * para a direita de todos os elementos que se encontram a direita a partir
     * desta posio.
     *
     * @param elem Elemento a ser adicionado
     * @param pos  Um inteiro correspondendo  posio do n que deve ser
     *             removido.
     * @throws java.lang.IllegalArgumentException
     *          Levantada quando o
     *          argumento <code>pos</code> for negativo.
     */
    public void insertElementAt(INode elem, int pos) throws IllegalArgumentException;


    /**
     * Retorna o nmero de elementos da lista.
     *
     * @return Um inteiro correspondendo ao nmero de elementos da lista.
     */
    public int size();


    /**
     * Diz se a lista est vazia.
     *
     * @return Um boolean que, caso tenho o valor <code>true</code>, indica
     *         que a lista est vazia.
     */
    public boolean isEmpty();


    /**
     * Retorna um objeto do tipo <code> JaTSIterator</code> .
     *
     * @return Objeto do tipo <code> JaTSIterator</code> .
     */
    public JaTSIterator getIterator();

}
