/*
 * JaTS Project - Java Transformation System
 * http://www.cin.ufpe.br/~jats
 *  
 * Copyright  2005  Informatics Center - Federal University of Pernambuco
 * 
 * JaTS is a free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 */
package cin.jats.engine.parser.nodes;

import cin.jats.engine.parser.nodes.exceptions.ExecutionException;
import cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException;
import cin.jats.engine.parser.nodes.exceptions.NodesNotMatchedException;
import cin.jats.engine.transformation.MicroTransformation;
import cin.jats.engine.util.ResultSet;
import cin.jats.engine.visitors.IVisitor;

/**
 * @author gas
 *
 */
public class MicroTransformationNode extends AbstractNode {

	private MicroTransformation transformation;
	
	/* (non-Javadoc)
	 * @see cin.jats.engine.parser.nodes.INode#accept(cin.jats.engine.visitors.IVisitor, java.lang.Object)
	 */
	public Object accept(IVisitor visitor, Object data)
			throws InconsistentNodeException, IllegalArgumentException,
			ExecutionException {
		
        // ... invoque o accept da declarao executvel associada a este n,
        // caso haja alguma...
        this.invokeAcceptOnExecutableDeclaration(visitor, data);
        // E ento visite este n.
        
        return visitor.visit(this, data);
        
	}

	/* (non-Javadoc)
	 * @see cin.jats.engine.parser.nodes.INode#match(cin.jats.engine.parser.nodes.INode, cin.jats.engine.util.ResultSet)
	 */
	public void match(INode node, ResultSet results)
			throws IllegalArgumentException, InconsistentNodeException,
			NodesNotMatchedException {
        throw new NodesNotMatchedException("An MicrotransformationNode should " +
                "not be matched", this, node);

	}

	/**
	 * Gets the transformation attribute
	 * 
	 * @return A microtransformation
	 */
	public MicroTransformation getTransformation() {
		return transformation;
	}

	/**
	 * Sets the transformation attribute
	 * 
	 * @param transformation
	 */
	public void setTransformation(MicroTransformation transformation) {
		this.transformation = transformation;
	}
	
	public boolean getPreconditionValue(ResultSet rset) throws IllegalArgumentException, ExecutionException, InconsistentNodeException {
		return this.transformation.getPreconditionValue(rset);
	}

}
