/*
 * JaTS Project - Java Transformation System
 * http://www.cin.ufpe.br/~jats
 *  
 * Copyright  2005  Informatics Center - Federal University of Pernambuco
 * 
 * JaTS is a free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 */
package cin.jats.engine.parser.nodes;

import cin.jats.engine.parser.nodes.exceptions.ExecutionException;
import cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException;
import cin.jats.engine.parser.nodes.exceptions.NodesNotMatchedException;
import cin.jats.engine.util.ResultSet;
import cin.jats.engine.visitors.IVisitor;


/**
 * 
 * @author Gustavo Santos
 * @version 01/06/2005
 */
public class Pattern extends AbstractNode {
	
	private JaTSNode body;
	private JIdentifier name;
	
	private String flatPattern;

	/**
	 * @return Returns the body.
	 */
	public JaTSNode getBody() {
		return body;
	}
	
	/**
	 * @param body The body to set.
	 */
	public void setBody(JaTSNode body) {
		this.body = body;
	}

	/**
	 * @param name The name to set.
	 */
	public void setName(JIdentifier name) {
		this.name = name;
	}

	/**
	 * @return Returns the name.
	 */
	public JIdentifier getName() {
		return name;
	}

	public Object accept(IVisitor visitor, Object data) throws InconsistentNodeException, IllegalArgumentException, ExecutionException {
		
		if (this.name != null) {
			this.name.accept(visitor, data);
		}
		
		this.body.accept(visitor, data);
		
		return visitor.visit(this, data);
	}

	public void match(INode node, ResultSet results) throws IllegalArgumentException, InconsistentNodeException, NodesNotMatchedException {

		this.body.match(node, results);
		results.put(this.name, node);
	}
	
	
    public Object process(Object data) 
    		throws ExecutionException, InconsistentNodeException, IllegalArgumentException {
    	return this.processNode(this.body, data);
    }

	/**
	 * @return Returns the flatPattern.
	 */
	public String getFlatPattern() {
		return flatPattern;
	}

	/**
	 * @param flatPattern The flatPattern to set.
	 */
	public void setFlatPattern(String flatPattern) {
		this.flatPattern = flatPattern;
	}
}
