/*
 * JaTS Project - Java Transformation System
 * http://www.cin.ufpe.br/~jats
 *  
 * Copyright  2005  Informatics Center - Federal University of Pernambuco
 * 
 * JaTS is a free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 */
package cin.jats.engine.parser.nodes;

import cin.jats.engine.parser.nodes.exceptions.ExecutionException;
import cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException;
import cin.jats.engine.visitors.IVisitor;

/**
 * A concatenation expression that yields an identifier.
 * i.e. the expression get::#att() where #att is mapped to the 
 * value 'client' results the value getClient().
 * Note, the first character is automatically changed to 
 * uppercase mode.
 * 
 * @author Gustavo Santos
 * @version 30/11/2005
 *
 */
public class PrefixedIdentifier extends JIdentifier {

	private String prefix;
	private JIdentifier rootId;
	
	
    public Object process(Object data) throws ExecutionException,
			InconsistentNodeException, IllegalArgumentException {

//		if (rootId.isExecutable()) {
//			rootId = (JIdentifier) processExecutableNode(rootId);
//		}
		rootId = (JIdentifier) rootId.process(data);

		rootId = rootId.addPrefix(this.prefix);

		return rootId;
	}

    public Object accept(IVisitor visitor, Object data)
			throws InconsistentNodeException, IllegalArgumentException,
			ExecutionException {

		Object retorno = null;
		
		this.rootId.accept(visitor, data);

		// ... invoque o accept da declarao executvel associada a este n,
		// caso haja alguma...
		this.invokeAcceptOnExecutableDeclaration(visitor, data);

		// e ento visite este n.
		retorno = visitor.visit(this, data);

		return retorno;
	}

	public String getPrefix() {
		return prefix;
	}
	public void setPrefix(String prefix) {
		this.prefix = prefix;
	}
	public JIdentifier getRootId() {
		return rootId;
	}
	public void setRootId(JIdentifier rootId) {
		this.rootId = rootId;
	}
}
