/*
 * JaTS Project - Java Transformation System
 * http://www.cin.ufpe.br/~jats
 *  
 * Copyright  2005  Informatics Center - Federal University of Pernambuco
 * 
 * JaTS is a free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 */
package cin.jats.engine.parser.nodes;

import cin.jats.engine.parser.nodes.exceptions.ExecutionException;
import cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException;
import cin.jats.engine.parser.nodes.exceptions.NodesNotMatchedException;
import cin.jats.engine.util.NodeList;
import cin.jats.engine.util.ResultSet;
import cin.jats.engine.visitors.IVisitor;

/**
 * @author gas
 *
 */
public class WhereClause extends AbstractNode {
	
	private Class clauseType;
	private NodeList transformationNodes;

	
	public WhereClause() {
		this.transformationNodes = new NodeList();
	}
	/* (non-Javadoc)
	 * @see cin.jats.engine.parser.nodes.INode#accept(cin.jats.engine.visitors.IVisitor, java.lang.Object)
	 */
	public Object accept(IVisitor visitor, Object data)
			throws InconsistentNodeException, IllegalArgumentException,
			ExecutionException {
		Object retorno = null;
		
        int max = this.transformationNodes.size() - 1;

        for (int i = max; i >= 0; i--) {
            this.transformationNodes.elementAt(i).accept(visitor, data);
        }
        
		// ... invoque o accept da declarao executvel associada a este n,
        // caso haja alguma...
        this.invokeAcceptOnExecutableDeclaration(visitor, data);
        // E ento visite este n.
        retorno = visitor.visit(this, data);

        return retorno;
	}

	/* (non-Javadoc)
	 * @see cin.jats.engine.parser.nodes.INode#match(cin.jats.engine.parser.nodes.INode, cin.jats.engine.util.ResultSet)
	 */
	public void match(INode node, ResultSet results)
			throws IllegalArgumentException, InconsistentNodeException,
			NodesNotMatchedException {
        throw new NodesNotMatchedException("An WhereClause should " +
                "not be matched", this, node);
	}

	public Class getClauseType() {
		return clauseType;
	}

	public void setClauseType(Class clauseType) {
		this.clauseType = clauseType;
	}

	public NodeList getTransformationNodes() {
		return transformationNodes;
	}

	public void addTransformationNode(MicroTransformationNode transf) {
		this.transformationNodes.addElement(transf);
	}

	public void setTransformationNodes(NodeList transformationNodes) {
		this.transformationNodes = transformationNodes;
	}

}
