/*
 * JaTS Project - Java Transformation System
 * http://www.cin.ufpe.br/~jats
 *  
 * Copyright  2005  Informatics Center - Federal University of Pernambuco
 * 
 * JaTS is a free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 */
package cin.jats.engine.parser.nodes.exceptions;

import cin.jats.engine.parser.nodes.INode;


/**
 * Esta classe define uma exceo que  levantada sempre que a execuo de um
 * determinado n falhar devido a problemas de acesso a uma certa entidade como
 * um atributo ou um mtodo. Um exemplo disso pode ser visto se tentarmos acessar
 * o valor de um atributo declarado como privado.
 *
 * @author Fernando Castor
 * @see cin.jats.engine.parser.nodes.JaTSNode
 * @see cin.jats.engine.parser.nodes.exceptions.ExecutionException
 */
public class IllegalAccessExecutionException extends ExecutionException {

    /**
     * Construtor vazio da classe.
     */
    public IllegalAccessExecutionException() {
    }

    /**
     * Construtor que recebe apenas o n onde o erro foi detectado.
     *
     * @param theNode Um objeto do tipo <code>JaTSNode</code> a partir do
     *                qual no foi possvel realizar uma certa operao porque o atributo/
     *                mtodo/etc que deveria ser "executado" no era acessvel.
     */
    public IllegalAccessExecutionException(INode theNode) {
        super(theNode);
    }

    /**
     * Este construtor recebe como parmetro apenas uma mensagem de erro.
     *
     * @param message Uma mensagem de erro indicando qual foi o problema
     *                encontrado.
     */
    public IllegalAccessExecutionException(String message) {
        super(message);
    }

    /**
     * Este construtor recebe, alm da mensagem de erro, o n onde o erro ocorreu.
     *
     * @param mensagem Uma mensagem de erro indicando qual foi o problema
     *                 encontrado.
     * @param theNode  Um objeto do tipo <code>JaTSNode</code> a partir do qual no
     *                 foi possvel realizar execuo.
     */
    public IllegalAccessExecutionException(String mensagem, INode theNode) {
        super(mensagem, theNode);
    }
}
