/*
 * JaTS Project - Java Transformation System
 * http://www.cin.ufpe.br/~jats
 *  
 * Copyright  2005  Informatics Center - Federal University of Pernambuco
 * 
 * JaTS is a free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 */
package cin.jats.engine.parser.nodes.exceptions;

import cin.jats.engine.parser.nodes.INode;


/**
 * Esta classe define uma exceo que  levantada sempre que uma varivel
 * est mapeada a um valor que no  vlido em uma determinada situao.
 * Ex: Se a varivel #a estiver mapeada ao identificador "Foo" e, durante
 * o processo de substituio das variveis, for encontrada #a no lugar
 * onde uma lista de modificadores deveria estar, essa exceo  levan-
 * tada, uma vez que o tipo esperado para #a era difirente do que foi
 * encontrado e os dois so incompatveis.
 *
 * @author Fernando Castor
 * @see cin.jats.engine.parser.nodes.JaTSNode
 */
public class IllegalMatchingException extends IllegalArgumentException {


    /**
     * O n correspondente  varivel que est casada a um n de tipo diferente
     * do necessrio.
     */
    private INode node;

    /**
     * Construtor vazio da classe.
     */
    public IllegalMatchingException() {
        node = null;

    }

    /**
     * Construtor que recebe apenas o n correspondente  varivel casada
     * invalidamente.
     *
     * @param theNode Um objeto do tipo <code>JaTSNode</code>, correspondendo a
     *                uma varivel JaTS que foi casada a um n de um tipo diferente do
     *                esperado.
     */
    public IllegalMatchingException(INode theNode) {
        node = theNode;
    }

    /**
     * Este construtor recebe como parmetro apenas uma mensagem de erro.
     *
     * @param message Uma mensagem de erro indicando qual foi o problema
     *                encontrado.
     */
    public IllegalMatchingException(String message) {
        super(message);
        node = null;
    }

    /**
     * Este construtor recebe, alm da mensagem de erro, o n onde o erro ocorreu.
     *
     * @param mensagem Uma mensagem de erro indicando qual foi o problema
     *                 encontrado.
     * @param theNode  Um objeto do tipo <code>JaTSNode</code>, correspondendo a
     *                 uma varivel JaTS que foi casada a um n de um tipo diferente do
     *                 esperado.
     */
    public IllegalMatchingException(String mensagem, INode theNode) {
        super(mensagem);
        node = theNode;
    }

    /**
     * Este mtodo retorna o n a partir do qual a exceo foi lanada, caso
     * este tenha sido fornecido como parmetro ao construtor da exceo.
     *
     * @return Um objeto do tipo <code>JaTSNode</code> correspondendo ao n a
     *         partir do qual a exceo foi lanada. Caso este no tenha sido
     *         fornecido como parmetro ao construtor da exceo, este mtodo
     *         retorna null;
     */
    public INode getNode() {
        return this.node;
    }
}
