/*
 * JaTS Project - Java Transformation System
 * http://www.cin.ufpe.br/~jats
 *  
 * Copyright  2005  Informatics Center - Federal University of Pernambuco
 * 
 * JaTS is a free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 */
package cin.jats.engine.parser.nodes.exceptions;

import cin.jats.engine.parser.nodes.INode;


/**
 * Esta classe define uma exceo que  levantada sempre que um componente
 * essencial de um n estiver faltando, deixando-o inconsistente.
 *
 * @author Fernando Castor
 * @see cin.jats.engine.parser.nodes.JaTSNode
 */
public class InconsistentNodeException extends Exception {

    public static final int DEFAULT = 0;
    public static final int INVALID_CONSTRUCTION = 1;

    private int type;

    /**
     * O n em que a inconsistncia foi detectada.
     */
    protected INode node;

    /**
     * Construtor vazio da classe.
     */
    public InconsistentNodeException() {
        type = DEFAULT;
        node = null;
    }

    /**
     * Construtor que recebe apenas o n onde o erro foi detectado.
     *
     * @param theNode Um objeto do tipo <code>JaTSNode</code> contendo alguma
     *                espcie de inconsistncia.
     */
    public InconsistentNodeException(INode theNode) {
        type = DEFAULT;
        node = theNode;
    }

    /**
     * Este construtor recebe como parmetro apenas uma mensagem de erro.
     *
     * @param message Uma mensagem de erro indicando qual foi o problema
     *                encontrado.
     */
    public InconsistentNodeException(String message) {
        super(message);
        type = DEFAULT;
        node = null;
    }

    /**
     * Este construtor recebe, alm da mensagem de erro, o n onde o erro ocorreu.
     *
     * @param mensagem Uma mensagem de erro indicando qual foi o problema
     *                 encontrado.
     * @param theNode  Um objeto do tipo <code>INode</code> contendo alguma
     *                 espcie de inconsistncia.
     */
    public InconsistentNodeException(String mensagem, INode theNode) {
        super(mensagem);
        type = DEFAULT;
        node = theNode;
    }

    /**
     * Este contrutor recebe o no onde ocorreu o erro e o tipo de erro
     *
     * @param theNode
     * @param exceptionType
     */
    public InconsistentNodeException(INode theNode, int exceptionType) {
        type = exceptionType;
        node = theNode;
    }


    /**
     * Este mtodo retorna o n a partir do qual a exceo foi lanada, caso
     * este tenha sido fornecido como parmetro ao construtor da exceo.
     *
     * @return Um objeto do tipo <code>INode</code> correspondendo ao n a
     *         partir do qual a exceo foi lanada. Caso este no tenha sido
     *         fornecido como parmetro ao construtor da exceo, este mtodo
     *         retorna null;
     */
    public INode getNode() {
        return this.node;
    }

    /**
     * @return
     */
    public int getType() {
        return this.type;
    }

    /**
     * @param type
     */
    public void setType(int type) {
        this.type = type;
    }

    public String getMessage() {
        String retorno = "";
        if (node != null) {
            retorno = "No inconsistente " /*+ node */;
        }
        return retorno;

    }
}
