/*
 * JaTS Project - Java Transformation System
 * http://www.cin.ufpe.br/~jats
 *  
 * Copyright  2005  Informatics Center - Federal University of Pernambuco
 * 
 * JaTS is a free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 */
package cin.jats.engine.parser.nodes.exceptions;

import cin.jats.engine.parser.nodes.exceptions.*;
import cin.jats.engine.parser.nodes.JaTSNode;
import cin.jats.engine.parser.nodes.INode;


/**
 * Essa exceo  levantada sempre que uma pr-condio invlida (que no avalia
 * para um <code>booleano</code>)  encontrada.
 *
 * @author Fernando Castor
 * @see cin.jats.engine.parser.nodes.JaTSNode
 */
public class InvalidPreconditionException extends cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException {


   
    /**
     * Construtor vazio da classe.
     */
    public InvalidPreconditionException() {
        node = null;

    }

    /**
     * Construtor que recebe apenas o n onde o erro foi detectado.
     *
     * @param theNode Um objeto do tipo <code>JaTSNode</code> contendo alguma
     *                espcie de inconsistncia.
     */
    public InvalidPreconditionException(JaTSNode theNode) {
        node = theNode;
    }

    /**
     * Este construtor recebe como parmetro apenas uma mensagem de erro.
     *
     * @param message Uma mensagem de erro indicando qual foi o problema
     *                encontrado.
     */
    public InvalidPreconditionException(String message) {
        super(message);
    }

    /**
     * Este construtor recebe, alm da mensagem de erro, o n onde o erro ocorreu.
     *
     * @param mensagem Uma mensagem de erro indicando qual foi o problema
     *                 encontrado.
     * @param theNode  Um objeto do tipo <code>JaTSNode</code> contendo alguma
     *                 espcie de inconsistncia.
     */
    public InvalidPreconditionException(String mensagem, JaTSNode theNode) {
        super(mensagem);
        node = theNode;
    }

    /**
     * Este mtodo retorna o n a partir do qual a exceo foi lanada, caso
     * este tenha sido fornecido como parmetro ao construtor da exceo.
     *
     * @return Um objeto do tipo <code>JaTSNode</code> correspondendo ao n a
     *         partir do qual a exceo foi lanada. Caso este no tenha sido
     *         fornecido como parmetro ao construtor da exceo, este mtodo
     *         retorna null;
     */
    public INode getNode() {
        return this.node;
    }
}
