/*
 * JaTS Project - Java Transformation System
 * http://www.cin.ufpe.br/~jats
 *  
 * Copyright  2005  Informatics Center - Federal University of Pernambuco
 * 
 * JaTS is a free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 */
package cin.jats.engine.parser.nodes.metajats;

import cin.jats.engine.util.ResultSet;

/**
 * Binary expressions are boolean logic expressions witch
 * enable the user to express queries over the code.
 * 
 * @author Gustavo Santos
 * @created 12/12/2005
 * 
 * @version 12/12/2005
 *
 */
public class BinaryMetaExpression extends MetaExpression {
	
	private MetaExpression lhs;
	private MetaExpression rhs;
	private LogicMetaOperator operator;

	/**
	 * Empty constructor for BinaryMetaExpression objects
	 *
	 */
	public BinaryMetaExpression() {
	}
	
	/**
	 * Constructor for BinaryMetaExpression objects
	 *
	 */
	public BinaryMetaExpression(MetaExpression lhs, MetaExpression rhs,
			LogicMetaOperator operator) {
		this.lhs = lhs;
		this.rhs = rhs;
		this.operator = operator;
	}

	/**
	 * @return Returns the lhs.
	 */
	public MetaExpression getLhs() {
		return lhs;
	}

	/**
	 * @param lhs The lhs to set.
	 */
	public void setLhs(MetaExpression lhs) {
		this.lhs = lhs;
	}

	/**
	 * @return Returns the operator.
	 */
	public LogicMetaOperator getOperator() {
		return operator;
	}

	/**
	 * @param operator The operator to set.
	 */
	public void setOperator(LogicMetaOperator operator) {
		this.operator = operator;
	}

	/**
	 * @return Returns the rhs.
	 */
	public MetaExpression getRhs() {
		return rhs;
	}

	/**
	 * @param rhs The rhs to set.
	 */
	public void setRhs(MetaExpression rhs) {
		this.rhs = rhs;
	}

//	@Override
	public boolean evaluate(ResultSet rs, Object obj) {
		boolean lhsRes = lhs.evaluate(rs, obj);
		boolean result = false;
		
		if (lhsRes == true) {
			if (this.operator.getValue() == LogicMetaOperator.AND_OP) {
				result = rhs.evaluate(rs, obj);
			} else if (this.operator.getValue() == LogicMetaOperator.OR_OP) {
			   result = true;
			}
		} else {
			if (this.operator.getValue() == LogicMetaOperator.OR_OP) {
				   result = rhs.evaluate(rs, obj);
		    }
		}
		
		return result;
	}

}
