/*
 * JaTS Project - Java Transformation System
 * http://www.cin.ufpe.br/~jats
 *  
 * Copyright  2005  Informatics Center - Federal University of Pernambuco
 * 
 * JaTS is a free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 */
package cin.jats.engine.parser.nodes.metajats;

import java.util.ArrayList;
import java.util.List;

import cin.jats.engine.parser.nodes.INode;

/**
 * A JaTS program is a sequence of statements defining a
 * transformation strategy. Meta programs also support
 * java statements between meta JaTS statements and pattern
 * declarations.
 * 
 * @author Gustavo Santos
 * @created 20/12/2005
 *
 */
public class JaTSProgram {

	private List statements;
	
	/**
	 * Empty constructor for JaTSProgram objects.
	 */
	public JaTSProgram() {
		this.statements = new ArrayList();
	}
	
	/**
	 * Adds a statements to the statement list.
	 * 
	 * @param statement A meta program statement
	 */
	public void addStatement(Object statement) {
		this.statements.add(statement);
	}
	
	/**
	 * Number of root statements in the program.
	 * 
	 * @return Number of statements
	 */
	public int size() {
		return this.statements.size();
	}
	
	/**
	 * Retrieves the statement at position 'index'.
	 * 
	 * @param index Statement index
	 * @return An INode representing the required statement
	 */
	public INode statementAt(int index) {
		return (INode) this.statements.get(index);
	}

}
