/*
 * JaTS Project - Java Transformation System
 * http://www.cin.ufpe.br/~jats
 *  
 * Copyright  2005  Informatics Center - Federal University of Pernambuco
 * 
 * JaTS is a free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 */
package cin.jats.engine.parser.nodes.metajats;

import cin.jats.engine.parser.nodes.INode;
import cin.jats.engine.parser.nodes.JBlock;
import cin.jats.engine.parser.nodes.JName;
import cin.jats.engine.parser.nodes.JStatementList;
import cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException;
import cin.jats.engine.parser.nodes.exceptions.NodesNotMatchedException;
import cin.jats.engine.util.ResultSet;
import cin.jats.engine.util.VariableNotFoundException;


/**
 * 
 * @author Gustavo Santos
 * @created 13/12/2005
 * 
 * @version 15/12/2005
 *
 */
public class MetaVariable extends MetaExpression {

	private String jatsVariable;
	private INode concreteValue;
	
	/**
	 * Empty constructor for MetaVariable objects
	 *
	 */
	public MetaVariable() {
	}
	
	/**
	 * Constructor for MetaVariable objects
	 *
	 * @param value a String representing the variable
	 */
	public MetaVariable(String value) {
		this.jatsVariable = value;
	}

//	@Override
	public boolean evaluate(ResultSet rs, Object obj) {
		boolean result = false;
		INode value = null;
		if (this.jatsVariable.startsWith("#")) {
			try {
				value = (INode) rs.get(jatsVariable);
				value.match((INode)obj, rs);
				result = true;
			} catch (VariableNotFoundException e) {
			} catch (IllegalArgumentException e) {
				e.printStackTrace();
			} catch (InconsistentNodeException e) {
			} catch (NodesNotMatchedException e) {
			}
		} else {
			value = enactVar();
			try {
				//the variable has a mapped value
				value = (INode) rs.get(jatsVariable);
				value.match((INode)obj, rs);
				result = true;
			} catch (VariableNotFoundException e) {
				try {
					//there is no mapped value, indeed
					//the variable matches succefuly
					value.match((INode)obj, rs);
					result = true;
				} catch (IllegalArgumentException e1) {
				} catch (InconsistentNodeException e1) {
				} catch (NodesNotMatchedException e1) {
				}
			} catch (IllegalArgumentException e) {
				e.printStackTrace();
			} catch (InconsistentNodeException e) {
			} catch (NodesNotMatchedException e) {
			}
		}
		
		return result;
	}

	private INode enactVar() {
		INode value = null;
		String type = "";
		String varName = "";
		
		type = this.jatsVariable.substring(0,
				this.jatsVariable.indexOf(":"));
		varName = this.jatsVariable.substring(
				this.jatsVariable.indexOf(":")+1);
		
		
		if (type.equals("Block")) {
			value = new JBlock();
			value.setVariableName(varName);
		} else if (type.equals("Name")) {
			value = new JName();
			value.setVariableName(varName);
		} else if (type.equals("StatementList")) {
			value = new JStatementList();
			value.setVariableName(varName);
		}
		
		return value;
		
	}

	/**
	 * @return Returns the concreteValue.
	 */
	public INode getConcreteValue() {
		return concreteValue;
	}

	/**
	 * @param concreteValue The concreteValue to set.
	 */
	public void setConcreteValue(INode concreteValue) {
		this.concreteValue = concreteValue;
	}

}
