/*
 * JaTS Project - Java Transformation System
 * http://www.cin.ufpe.br/~jats
 *  
 * Copyright  2005  Informatics Center - Federal University of Pernambuco
 * 
 * JaTS is a free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 */
package cin.jats.engine.parser.nodes.metajats;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import cin.jats.engine.parser.nodes.INode;
import cin.jats.engine.parser.nodes.JName;
import cin.jats.engine.util.ResultSet;

/**
 * 
 * @author Gustavo Santos
 * @created 12/12/2005
 * 
 * @version 12/12/2005
 *
 */
public class TraverseMetaExpression extends MetaExpression {
	
	private List matchNodes;
	private List operators;
	

	/**
	 * Empty constructor for TraverseMetaExpression objects.
	 *
	 */
	public TraverseMetaExpression() {
		this.matchNodes = new ArrayList();
	}
	
	/**
	 * Constructor for TraverseMetaExpression objects.
	 *
	 * @param matchNodes matching expressions
	 * @param operators ordered list of operators
	 */
	
	public TraverseMetaExpression(List matchNodes, List operators) {
		this.matchNodes = matchNodes;
		this.operators = operators;
	}

//	@Override
	public boolean evaluate(ResultSet rs, Object obj) {
		boolean result = false;
		
		for (Iterator iter = this.matchNodes.iterator(); iter.hasNext();) {
			MetaExpression element = (MetaExpression) iter.next();
			
		} 
		
		MetaVariable initVar = (MetaVariable) this.matchNodes.get(0);
		INode value = initVar.getConcreteValue();
		
		if (value != null) {
			if (this.operators.size() > 0) {
				
				result = evaluateSubExp(0, value, rs);
			
			} else {
				result = true;
			}
		} 
		
		return result;
	}

	private boolean evaluateSubExp(int opIndex, INode startNode, ResultSet rs) {
		boolean result = false;
		
		TraverseMetaOperator op = 
			(TraverseMetaOperator) this.operators.get(opIndex);
		INode tempNode = null;
		
		// <<
		if (op.getValue() == TraverseMetaOperator.DIRECT_CHILD_OP) {
			tempNode = startNode.getParent();
			if (((MetaExpression)this.matchNodes.get(opIndex+1)).evaluate(rs, tempNode)) {
				
				if (this.operators.size() > (opIndex+1)) {
					result = evaluateSubExp(opIndex+1, tempNode, rs);
				} else {
					result = true;
				}
			}
		} else if (op.getValue() == TraverseMetaOperator.CHILD_OP) {
			tempNode = startNode.getParent();
			while (tempNode != null) {
				if (((MetaExpression) this.matchNodes.get(opIndex + 1))
						.evaluate(rs, tempNode)) {

					if (this.operators.size() > (opIndex + 1)) {
						result = evaluateSubExp(opIndex + 1, tempNode, rs);
						if (result) break;
					} else {
						result = true;
					}
				}
				tempNode = tempNode.getParent();
			}
		} else if (op.getValue() == TraverseMetaOperator.DEC_OP) {
			
			if (!(startNode instanceof JName)) {
				throw new IllegalArgumentException("It was not possible to resolve |dec| edge");
			}
			
			tempNode = ((JName)startNode).getDeclarationRef();
			if (((MetaExpression)this.matchNodes.get(opIndex+1)).evaluate(rs, tempNode)) {
				
				if (this.operators.size() > (opIndex+1)) {
					result = evaluateSubExp(opIndex+1, tempNode, rs);
				} else {
					result = true;
				}
			}
		}
		
		return result;
	}

}
