/*
 * JaTS Project - Java Transformation System
 * http://www.cin.ufpe.br/~jats
 *  
 * Copyright  2005  Informatics Center - Federal University of Pernambuco
 * 
 * JaTS is a free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 */
package cin.jats.engine.transformation;

import cin.jats.engine.util.CompilationUnitSet;

/**
 * A classe <code>JaTSTransformation</code> representa uma transformacao do JaTS. Uma
 * transformacao   composta por:  um conjunto de templates para o lado esquerdo,
 * outro conjunto de templates para o lado direito
 * 
 * @author  Kellen Christine, Adeline
 * @version 1.3, 24/01/2003
 * @since   JDK1.2
 */
public class JaTSTransformation {

    private CompilationUnitSet leftTemplates;
    private CompilationUnitSet rightTemplates;


    /**
     * Construtor da classe, recebe como parametros:
     *
     * @param left           O CompilationUnitSet referente ao lado esquerdo da
     *                       transformao
     * @param right         O CompilationUnitSet referente ao lado esquerdo da
     *                       transformao
     */
    public JaTSTransformation(CompilationUnitSet left, CompilationUnitSet right) {

        this.leftTemplates = left;
        this.rightTemplates = right;
    }


    /**
     * Retorna o lado esquerdo da transformacao
     *
     * @return  Lado esquerdo da transformacao
     */
    public CompilationUnitSet getLeftTemplates() {
        return this.leftTemplates;
    }


    public int hashCode() {
        return super.hashCode();
    }

    /**
     * Retorna o lado direito da transformacao
     *
     * @return  Lado direito da transformacao
     */
    public CompilationUnitSet getRightTemplates() {
        return this.rightTemplates;
    }

}