/*
 * JaTS Project - Java Transformation System
 * http://www.cin.ufpe.br/~jats
 *  
 * Copyright  2005  Informatics Center - Federal University of Pernambuco
 * 
 * JaTS is a free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 */
package cin.jats.engine.transformation;

import cin.jats.engine.parser.nodes.INode;
import cin.jats.engine.parser.nodes.JPreconditionDeclaration;
import cin.jats.engine.parser.nodes.exceptions.ExecutionException;
import cin.jats.engine.parser.nodes.exceptions.PreconditionDoesNotHoldException;
import cin.jats.engine.util.ResultSet;
import cin.jats.engine.visitors.ReplacementVisitor;

/**
 * A classe <code>MicroTransformation</code> representa uma micro transformacao
 * do JaTS. Uma microtransformacao   composta por:  um lado esquerdo da
 * transformao, um lado direito, onde cada lado  um JaTSNode.
 * Uma microtransformao pode ser aplicada a partes de uma rvore sinttica.
 * 
 * @author Adeline  de Sousa
 * @author Gustavo Santos
 * @version  02/04/2003
 * @since   JDK1.3
 */
public class MicroTransformation {

	
	private Class type;
	
    // template fonte
    private INode source;

    // template destino
    private INode target;
    
    private JPreconditionDeclaration precondition;


    /**
     * Construtor da classe, recebe como parametros:
     *
     * @param left           Lado esquerdo da transformao
     * @param right          Lado direito da transformao
     */
    public MicroTransformation(INode left, INode right) {

        this.source = left;
        this.target = right;
    }


    /**
     * Retorna o lado esquerdo da transformacao
     *
     * @return  Lado esquerdo da transformacao
     */
    public INode getSource() {
        return this.source;
    }

    public void setSource(INode source) {
        this.source = source;
    }

    /**
     * Retorna o lado direito da transformacao
     *
     * @return  Lado direito da transformacao
     */
    public INode getTarget() {
        return this.target;
    }


    public void setTarget(INode target) {
        this.target = target;
    }


    public int hashCode() {
        return super.hashCode();
    }


	public Class getType() {
		return type;
	}


	public void setType(Class type) {
		this.type = type;
	}


	/**
	 * @return Returns the precondition.
	 */
	public INode getPrecondition() {
		return precondition;
	}


	/**
	 * @param precondition The precondition to set.
	 */
	public void setPrecondition(JPreconditionDeclaration precondition) {
		this.precondition = precondition;
	}

    public boolean getPreconditionValue(ResultSet rset)
			throws ExecutionException,
			cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException,
			IllegalArgumentException {
		// por definio,  true
		boolean result = true;
		ReplacementVisitor rvis = new ReplacementVisitor(rset);

		if (this.precondition != null) {
			rvis.visit(this.precondition, null);
			try {
				this.precondition.process(rset);
				result = true;
			} catch (PreconditionDoesNotHoldException ex) {
				result = false;
			}
			// para que a pr-condiao nao seja mais impressa
			this.precondition = null;
		}
		return result;
	}	

}