/*
 * JaTS Project - Java Transformation System
 * http://www.cin.ufpe.br/~jats
 *  
 * Copyright  2005  Informatics Center - Federal University of Pernambuco
 * 
 * JaTS is a free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 */
package cin.jats.engine.util;

import java.util.Iterator;
import java.io.Serializable;
import cin.jats.engine.parser.nodes.ICompilationUnit;
import cin.jats.engine.parser.nodes.INode;


public class CompilationUnitSet implements Serializable {

    private NodeList declarations = new NodeList();

    /**
     * Adiciona uma JCompilationUnit ao conjunto
     *
     * @param compUnit JCompilationUnit a ser adicionada
     *
     * @exception IllegalArgumentException - se a CompilationUnit passada for nula.
     */
    public void addCompilationUnit(ICompilationUnit compUnit) throws IllegalArgumentException {
        declarations.addElement(compUnit);
    }


    /**
     * Retorna a JCompilationUnit que se encontra na i-sima posio
     *
     * @return a JCompilationUnit que se encontra na i-sima posio
     */
    public ICompilationUnit getCompilationUnit(int i) throws IllegalArgumentException {

        INode comp = declarations.elementAt(i);

        ICompilationUnit retorno = null;

        // Em teoria, nunca vai acontecer de um objeto do NodeList
        // declarations ser de um tipo diferente de JCompilationUnit, por isso
        // no  lanada nenhuma exceo.
        if (comp instanceof ICompilationUnit) {
            retorno = (ICompilationUnit) comp;
        }
        return retorno;
    }


    /**
     * Remove a CompilationUnit na i-sima posio.
     *
     * @param i posio da CompilationUnit a ser removida
     */
    public void removeCompilationUnit(int i) throws IllegalArgumentException {
        declarations.removeElementAt(i);
    }


    /**
     * Retorna o nmerode elementos presentes neste Conjunto
     *
     * @return Numero de elementos do conjunto
     */
    public int size() {
        return declarations.size();
    }

    public ICompilationUnit[] toArray() {
        ICompilationUnit[] retorno = new ICompilationUnit[declarations.size()];
        for (int i = 0; i < retorno.length; i++) {
            retorno[i] = (ICompilationUnit) declarations.elementAt(i);
        }
        return retorno;
    }

    public static CompilationUnitSet
            getCompilationUnitSet(ICompilationUnit[]
            compUnitArray) {
        CompilationUnitSet retorno = new CompilationUnitSet();
        if (compUnitArray != null && compUnitArray.length > 0) {
            for (int i = 0; i < compUnitArray.length; i++) {
                retorno.addCompilationUnit(compUnitArray[i]);
            }
        }
        return retorno;
    }


    /**
     * Retorna um <code>JaTSIterator</code>, para que se possa percorrer pelos
     * elementos deste conjunto. Nesse caso, os elementos retornados j so do
     * tipo INode, no havendo a necessidade de casting
     *
     * @return um objeto do tipo <code>JaTSIterator</code>, para que se possa
     *        percorrer pelos elementos deste conjunto
     */
    public JaTSIterator getIterator() {
        return this.declarations.getIterator();
    }


    /**
     * Retorna um <code>Iterator</code>, para que se possa percorrer pelos
     * elementos deste conjunto
     *
     * @return um objeto do tipo <code>Iterator</code>, para que se possa
     *        percorrer pelos elementos deste conjunto
     */
    public Iterator iterator() {
        return this.declarations.iterator();
    }
}
