/*
 * JaTS Project - Java Transformation System
 * http://www.cin.ufpe.br/~jats
 *  
 * Copyright  2005  Informatics Center - Federal University of Pernambuco
 * 
 * JaTS is a free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 */
package cin.jats.engine.util;

import java.util.Iterator;
import java.util.Vector;

/**
 * Esta classe serve como um mapeamento entre uma String que representa uma
 * expresso e a expresso executvel correspondente. Ela ser usada ao invs de
 * um map propriamente dito, devido ao fato de precisarmos de uma relao de
 * ordem  entre os elementos do mapeamento.
 *
 * @author Adeline de Sousa
 * @version  0.1 -  08/07/2002
 * @since JDK 1.3
 */
public class ExpressionsMap {

    private Vector map;

    public ExpressionsMap() {
        this.map = new Vector();
    }

    /**
     * Adiciona um mapeamento string -> expressao ao map. A string no pode
     * estar contida no map, pois ela  a chave da busca.
     */
    public void put(String strExp, Object exp) {
        // se o elemento ainda no existe, cria a tupla
        if (!this.contains(strExp)) {
            Tuple t = new Tuple(strExp, exp);
            this.map.addElement(t);
        }
        // se ele j existe, simplesmente sobrescreve o valor
        else {
            Tuple tmp = this.getTuple(strExp);
            tmp.setExp(exp);
        }
    }


    /**
     * Retorna o objeto que est mapeado aa String passada como parmetro
     *
     * @param strExp
     */
    public Object get(String strExp) {
        Iterator it = this.map.iterator();
        Object retorno = null;
        boolean achou = false;

        if (Util.checkString(strExp)) {
            while (it.hasNext() && !achou) {
                Tuple tuple = (Tuple) it.next();
                String exp = tuple.getStrExp();
                if (exp.equals(strExp)) {
                    retorno = tuple.getExp();
                    achou = true;
                }
            }
        } else {
            throw new IllegalArgumentException("Argumento nulo");
        }
        return retorno;
    }


    /**
     * Retorna a tupla correspondente a chave passada como parametro
     */
    private Tuple getTuple(String strExp) {
        Iterator it = this.map.iterator();
        Tuple retorno = null;
        boolean achou = false;

        if (Util.checkString(strExp)) {
            while (it.hasNext() && !achou) {
                Tuple tuple = (Tuple) it.next();
                String exp = tuple.getStrExp();
                if (exp.equals(strExp)) {
                    retorno = tuple;
                    achou = true;
                }
            }
        } else {
            throw new IllegalArgumentException("Argumento nulo");
        }
        return retorno;
    }

    /**
     * Retorna um Iterator, para que se possa percorrer a estrutura do map. Os
     * elementos retornados so do tipo String
     *
     * @return um objeto do tipo <code> java.util.Iterator</code>, para que se
     *         possa percorrer a estrutura do map. Os elementos retornados so
     *         do tipo String
     */
    public Iterator getKeys() {
        Vector tmpVector = new Vector();
        Iterator it = this.map.iterator();
        while (it.hasNext()) {
            Tuple tuple = (Tuple) it.next();
            String exp = tuple.getStrExp();
            tmpVector.addElement(exp);
        }
        return tmpVector.iterator();
    }


    /**
     * Retorna true, se a string passada como parmetro estiver presente no map.
     *
     * @param exp String que queremos verificar se j pertence ao map
     */
    public boolean contains(String exp) {
        boolean retorno = false;
        Iterator it = this.getKeys();
        if (Util.checkString(exp)) {
            // varre as strings presentes, pra ver se ela j no foi adicionada
            while (it.hasNext() && !retorno) {
                String tmp = (String) it.next();
                // se as strings forem iguais, retorna true e sai do lao
                retorno = exp.equals(tmp);
            }
        }
        return retorno;

    }

    /**
     * Retorna o nmero de pares contidos no map.
     */
    public int size() {
        return this.map.size();

    }


    public static void main(String[] args) {
        ExpressionsMap map = new ExpressionsMap();


        String obj1 = "[[a.getName()]]";
        String obj2 = "[[conta.addPrefix(\"" + "get" + "\")]]";
        String obj3 = "[[#x.addPrefix(\"" + "get" + "\")]]";
        String obj4 = "[[#a.getName()]]";

        String exp1 = "exp1";
        String exp3 = "exp3";
        String exp4 = "exp4";

        map.put(exp1, obj1);
        map.put(exp1, obj2);
        map.put(exp3, obj3);
        map.put(exp4, obj4);


        System.out.println("Testando o iterator e o get");
        Iterator it = map.getKeys();
        while (it.hasNext()) {
            String str = (String) it.next();
            System.out.println("chave : " + str + " valor: " + map.get(str));
        }

    }

}