/*
 * JaTS Project - Java Transformation System
 * http://www.cin.ufpe.br/~jats
 *  
 * Copyright  2005  Informatics Center - Federal University of Pernambuco
 * 
 * JaTS is a free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 */
package cin.jats.engine.util;

import java.util.Vector;
import java.util.Iterator;

import cin.jats.engine.parser.nodes.INode;

/**
 * Esta classe implementa uma pilha de objetos do tipo <code>INode</code>.
 *
 * @see INode
 * @author Fernando Castor
 * @since JDK 1.3
 */
public class NodeStack {


    // A lista onde sero armazenados os elementos.
    private Vector elements;

    /**
     * Construtor vazio da classe.
     */
    public NodeStack() {
        elements = new Vector();
    }

    /**
     * Diz se a pilha est vazia.
     *
     * @return Um boolean que, caso tenho o valor <code>true</code>, indica
     * que a lista est vazia.
     */
    public boolean isEmpty() {
        return this.elements.isEmpty();
    }

    /**
     * Remove o elemento que se encontra na  posio <code>pos</code> desta lista.
     *
     * @exception java.lang.IllegalArgumentException Levantada quando o
     * argumento <code>pos</code> for negativo.
     */
    public INode pop() {
        INode retorno = null;

        if (!elements.isEmpty()) {
            retorno = top();
            elements.removeElementAt(elements.size() - 1);
            //System.out.println("Removendo " + retorno.getClass().getName() + " na pilha");
        }
        return retorno;
    }

    /**
     * Adiciona um elemento  pilha.
     *
     * @param node Um objeto qualquer do tipo <code>INode</code>.
     *
     * @exception java.lang.IllegalArgumentException Levantada quando o argumento
     * <code>node</code> for <code>null</code>.
     */
    public void push(INode node) throws IllegalArgumentException {
        if (node == null) {
            throw new IllegalArgumentException();
        }
        //System.out.println("Inserindo " + node.getClass().getName() + " na pilha");
        this.elements.addElement(node);
    }

    /**
     * Retorna o objeto que se encontra no topo da pilha.
     *
     * @return Um <code>INode</code> correspondente ao elemento no topo da pilha.
     */
    public INode top() {
        int i = 0;
        INode retorno = null;

        if (!elements.isEmpty()) {
            i = elements.size() - 1;
            retorno = (INode) elements.elementAt(i);
        }
        return retorno;
    }


    /**
     * Returns a string representation of the object. In general, the
     * <code>toString</code> method returns a string that
     * "textually represents" this object. The result should
     * be a concise but informative representation that is easy for a
     * person to read.
     * It is recommended that all subclasses override this method.
     * <p>
     * The <code>toString</code> method for class <code>Object</code>
     * returns a string consisting of the name of the class of which the
     * object is an instance, the at-sign character `<code>@</code>', and
     * the unsigned hexadecimal representation of the hash code of the
     * object. In other words, this method returns a string equal to the
     * value of:
     * <blockquote>
     * <pre>
     * getClass().getName() + '@' + Integer.toHexString(hashCode())
     * </pre></blockquote>
     *
     * @return  a string representation of the object.
     */
    public String toString() {

        String retorno = "pilha";
        Iterator it = this.elements.iterator();
        while (it.hasNext()) {
            Object obj = it.next();
            retorno += obj + "\nobj.class= " + obj.getClass().getName() + "\n";
        }

        return retorno;

    }
}
