/*
 * JaTS Project - Java Transformation System
 * http://www.cin.ufpe.br/~jats
 *  
 * Copyright  2005  Informatics Center - Federal University of Pernambuco
 * 
 * JaTS is a free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 */
package cin.jats.engine.util;

import java.util.Vector;

/**
 * Esta classe implementa uma pilha de objetos.
 *
 * @author Fernando Castor
 */
public class ObjectStack {


    // A lista onde sero armazenados os elementos.
    private Vector elements;

    /**
     * Construtor vazio da classe.
     */
    public ObjectStack() {
        elements = new Vector();
    }

    /**
     * Diz se a pilha es vazia.
     *
     * @return Um boolean que, caso tenho o valor <code>true</code>, indica
     * que a lista est vazia.
     */
    public boolean isEmpty() {
        return this.elements.isEmpty();
    }

    /**
     * Remove o elemento que se encontra na  posio <code>pos</code> desta lista.
     * @exception java.lang.IllegalArgumentException Levantada quando o
     * argumento <code>pos</code> for negativo.
     */
    public Object pop() {
        Object retorno = null;

        if (!elements.isEmpty()) {
            retorno = top();
            elements.removeElementAt(elements.size() - 1);
        }

        return retorno;
    }

    /**
     * Adiciona um elemento  pilha.
     *
     * @param obj Um objeto qualquer.
     *
     * @exception java.lang.IllegalArgumentException Levantada quando o argumento
     *<code>obj</code> for <code>null</code>.
     */
    public void push(Object obj) throws IllegalArgumentException {
        if (obj == null) {
            throw new IllegalArgumentException();
        }
        this.elements.addElement(obj);
    }

    /**
     * Retorna o objeto que se encontra no topo da pilha.
     *
     * @return Um <code>Object</code> correspondente ao elemento no topo da pilha.
     */
    public Object top() {
        int i = 0;
        Object retorno = null;

        if (!elements.isEmpty()) {
            i = elements.size() - 1;
            retorno = elements.elementAt(i);
        }

        return retorno;
    }
}
