/*
 * JaTS Project - Java Transformation System
 * http://www.cin.ufpe.br/~jats
 *  
 * Copyright  2005  Informatics Center - Federal University of Pernambuco
 * 
 * JaTS is a free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 */
package cin.jats.engine.util;

import cin.jats.engine.parser.nodes.INode;

/**
 * Esta classe implementa um par (var,valor), onde a varivel  um objeto do
 * tipo INode e o valor  um Object
 *
 * @author Adeline de Sousa
 * @since JDK 1.3
 */
public class Pair implements Cloneable {

    // varivel
    private INode var;

    // valor
    private Object value;

    /**
     * Cosntrutor do objeto.
     *
     * @param var            Varivel JaTS
     * @param value          Valor associado
     */
    public Pair(INode var, Object value) {
        this.setVar(var);
        this.setValue(value);
    }

    /**
     * Retorna a varivel deste par (var,valor)
     * @return
     */
    public INode getVar() {
        return var;
    }

    /**
     * Seta a varivel do par (var,valor)
     * @param var
     */
    public void setVar(INode var) {
        this.var = var;
    }

    public Object getValue() {
        return value;
    }

    public void setValue(Object value) {
        this.value = value;
    }

    /**
     * Para comparar se  igual, usa apenas o atributo var.
     */
    public boolean equals(Object obj) {
        boolean retorno = false;
        if (obj instanceof Pair) {
            Pair pair = (Pair) obj;
            retorno = (Util.equals(pair.getVar(), this.getVar()));

        }
        return retorno;
    }

    /**
     * Retorna o hashCode da varivel.
     */
    public int hashCode() {
        return var.hashCode();
    }

    public String toString() {
        String retorno = "";
        if (var != null && value != null) {
            retorno = "var: " + var + "value: " + value;
        } else if (var != null) {
            retorno = "var: " + var;
        }
        return retorno;
    }

    /**
     * Creates and returns a copy of this object.  The precise meaning
     * of "copy" may depend on the class of the object. The general
     * intent is that, for any object <tt>x</tt>, the expression:
     * <blockquote>
     * <pre>
     * x.clone() != x</pre></blockquote>
     * will be true, and that the expression:
     * <blockquote>
     * <pre>
     * x.clone().getClass() == x.getClass()</pre></blockquote>
     * will be <tt>true</tt>, but these are not absolute requirements.
     * While it is typically the case that:
     * <blockquote>
     * <pre>
     * x.clone().equals(x)</pre></blockquote>
     * will be <tt>true</tt>, this is not an absolute requirement.
     * Copying an object will typically entail creating a new instance of
     * its class, but it also may require copying of internal data
     * structures as well.  No constructors are called.
     * <p>
     * The method <tt>clone</tt> for class <tt>Object</tt> performs a
     * specific cloning operation. First, if the class of this object does
     * not implement the interface <tt>Cloneable</tt>, then a
     * <tt>CloneNotSupportedException</tt> is thrown. Note that all arrays
     * are considered to implement the interface <tt>Cloneable</tT>.
     * Otherwise, this method creates a new instance of the class of this
     * object and initializes all its fields with exactly the contents of
     * the corresponding fields of this object, as if by assignment; the
     * contents of the fields are not themselves cloned. Thus, this method
     * performs a "shallow copy" of this object, not a "deep copy" operation.
     * <p>
     * The class <tt>Object</tt> does not itself implement the interface
     * <tt>Cloneable</tt>, so calling the <tt>clone</tt> method on an object
     * whose class is <tt>Object</tt> will result in throwing an
     * exception at run time. The <tt>clone</tt> method is implemented by
     * the class <tt>Object</tt> as a convenient, general utility for
     * subclasses that implement the interface <tt>Cloneable</tt>, possibly
     * also overriding the <tt>clone</tt> method, in which case the
     * overriding definition can refer to this utility definition by the
     * call:
     * <blockquote>
     * <pre>
     * super.clone()</pre></blockquote>
     *
     * @return     a clone of this instance.
     * @exception  OutOfMemoryError            if there is not enough memory.
     * @see        Cloneable
     */
    public Object clone() {

        INode cvar = this.getVar();
        Object cvalue = this.getValue();
        return new Pair(cvar, cvalue);


    }
}
