/*
 * JaTS Project - Java Transformation System
 * http://www.cin.ufpe.br/~jats
 *  
 * Copyright  2005  Informatics Center - Federal University of Pernambuco
 * 
 * JaTS is a free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 */
package cin.jats.engine.util;

import java.util.StringTokenizer;
import java.util.Vector;

public class Position {

	private Vector coordinates;	
	private String flatCoord;

	/**
	 * Constructor for Position objects
	 */
	public Position() {
	}

	/**
	 * Constructor for Position objects
	 * 
	 * @param position The flat coordinates
	 */
	public Position(String position) {
		this.flatCoord = position;
		this.coordinates = this.parseCoordinates(position);
	}
	
	/**
	 * Build a list of PositionElements from a flat string
	 * 
	 * @param flat The original string
	 * @return Vector of elements
	 */
	private Vector parseCoordinates(String flat) {
		Vector result = new Vector();
		StringTokenizer tk = new StringTokenizer(flat, ";");
		String temp = "";
		PositionElement pe = null;
		while (tk.hasMoreTokens()) {
			temp = tk.nextToken();
			pe = stringToPositionElement(temp);
			result.add(pe);
		}
		return result;
		
	}

	
	/**
	 * Mounts a PositionElement object from a string
	 * 
	 * @param posElm The original string
	 * @return PositionElement
	 */
	private PositionElement stringToPositionElement(String posElm) {
		PositionElement pe = new PositionElement();
		
	    if (posElm.startsWith("class")) {
		    pe.setType(PositionElement.CLASS);
		} else if (posElm.startsWith("met")) {
		    pe.setType(PositionElement.METHOD);			
		} else if (posElm.startsWith("par")) {
		    pe.setType(PositionElement.PARAMETER);			
		} else if (posElm.startsWith("stm")) {
		    pe.setType(PositionElement.STATEMENT);			
		}
	    
	    pe.setValue(posElm.substring(posElm.indexOf("[")+1, posElm.indexOf("]")));

	    return pe;
	}

	/**
	 * @return Returns the coordinates.
	 */
	public Vector getCoordinates() {
		return coordinates;
	}

	/**
	 * @param coordinates The coordinates to set.
	 */
	public void setCoordinates(Vector coordinates) {
		this.coordinates = coordinates;
	}

	/**
	 * @return Returns the flatCoord.
	 */
	public String getFlatCoord() {
		return flatCoord;
	}

	/**
	 * @param flatCoord The flatCoord to set.
	 */
	public void setFlatCoord(String flatCoord) {
		this.flatCoord = flatCoord;
	}

	public Position retractLevel() {
		Position result;
		Vector newCoords = new Vector();
		String newFlat = "";
		
		for (int i = 1; i < this.coordinates.size(); i++) {
			newCoords.addElement(this.coordinates.elementAt(i));
		}
		
		StringTokenizer tk = new StringTokenizer(this.flatCoord, ";");
		
		if (tk.hasMoreTokens())
			tk.nextToken();
		
		while (tk.hasMoreTokens()) {
			if (newFlat.length() > 0) {
				newFlat.concat(";");
			}
			newFlat.concat(tk.nextToken());
		}
		
		result = new Position();
		result.setCoordinates(newCoords);
		result.setFlatCoord(newFlat);
		
		return result;
	}

}

