/*
 * JaTS Project - Java Transformation System
 * http://www.cin.ufpe.br/~jats
 *  
 * Copyright  2005  Informatics Center - Federal University of Pernambuco
 * 
 * JaTS is a free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 */
package cin.jats.engine.util;

import java.io.Writer;
import java.util.*;

import cin.jats.engine.JatsIO;
import cin.jats.engine.parser.ParseException;
import cin.jats.engine.parser.nodes.*;


/**
 * Esta classe define funoes utilitrias de uso geral.
 * 
 * @author Adeline
 * @version 1.0
 * @since JDK 1.3
 */
public abstract class Util {

    // terminao default para variaveis do tipo comentrio
    public static final String COMMENT_VAR = "_#COMMENT_VAR";
    public static final String TYPE_COMMENT_VAR = "_#TYPE_COMMENT_VAR";


    /**
     * Este mtodo verifica se dois objetos so iguais.
     * Os objetos obj1 e obj2 devem ser do mesmo tipo
     *
     * @param obj1   Objeto a ser comparado
     * @param obj2   Objeto a ser comparado
     *
     * @return true, se <code>obj1</code> for igual a <code>obj2</code> <br>
     *         false, caso contrario
     */
    public static boolean equals(Object obj1, Object obj2) {
        boolean retorno = false;
        if (obj1 != null && obj2 != null) {
            retorno = obj1.equals(obj2);
        } else if (obj1 == null && obj2 == null) {
            retorno = true;
        }

        return retorno;
    }

    /**
     * Verifica se uma String  vlida. Para isso verifica se esta
     *  diferente de null e se  diferente de String vazia
     *
     * @param str    String a ser verificada
     *
     * @return   true, caso a string seja vlida
     *           false, caso contrrio
     */
    public static boolean checkString(String str) {
        boolean retorno = false;
        if (str != null && !str.trim().equals("")) {
            retorno = true;
        }
        return retorno;

    }


    public static void main(String[] args) {
        String texto="gets the #ATTR# attribute";

        String attr="#ATTR#";

        String newAttr= "name";

        String teste="0123456789abcdefghij";

        System.out.println("fst = " + teste.substring(0,10));
        System.out.println("snd = " + teste.substring(10));

        System.out.println("texto = " + texto);
        String novoTexto = replaceString(attr,newAttr,texto);

        System.out.println("novoTexto = " + novoTexto);


    }

    public static String replaceString(String oldStr, String newStr, String text) {
        String retorno = "";
        int oldStrIndex = -2;

        if (checkString(oldStr) && checkString(newStr) && checkString(text)) {
            if (oldStr.length() <= text.length()) {
                oldStrIndex = text.indexOf(oldStr);
                if (oldStrIndex != -1) {
                    retorno = text.substring(0, oldStrIndex) + newStr
                            + text.substring(oldStrIndex + oldStr.length(),
                                    text.length());
                }
            }

        }
        return retorno;

    }

    /**
     * Verifica se os objetos contidos em uma NodeList so todos de um mesmo
     * tipo. Isto  particularmente til em classes que utilizam o padro
     * Decorator.
     *
     * @param  nodes     Lista de ns
     * @param  nodeType  Tipo permitido para os ns da lista
     *
     * @exception  IllegalArgumentException - Levantada caso um dos ns da lista
     *              no seja do tipo passado como parmetro
     */
    public static boolean verifyNodeList(NodeList nodes, int nodeType) {

        boolean result = false;
        if (nodes != null) {
            // pra entrar no lao
            result = true;
            JaTSIterator jit = nodes.getIterator();
            while (jit.hasNext() && result) {
                INode jaTSNode = jit.next();
                result = (jaTSNode.getTypeIdentifier() == nodeType);

            }
        }

        return result;

    }


    /**
     * Retorna a String passada como parametro com a primeira letra
     * minscula
     *
     * @param str    String a ser convertida
     *
     * @return  String passada como parametro com a primeira letra
     * minscula
     */
    public static String toVariableName(String str) {
        String retorno = str;
        if (checkString(str)) {
            // captura a primeira letra
            char firstChar = str.charAt(0);
            // obtm o restante da String
            String tmpStr = str.substring(1, str.length());
            // converte a primeira letra em minusculo
            char newFirstChar = Character.toLowerCase(firstChar);
            // retorna a mesma String com a primeira letra em minusculo
            retorno = newFirstChar + tmpStr;
        }
        return retorno;

    }

    /**
     * Verifica se esta string corresponde ao nome de uma variavel, ou seja, se
     * seu nome comeca com um "#".
     *
     * @param  str   String a ser verificada
     *
     * @return  true, se a string passada comecar com '#' <br>
     *          false, caso contrario
     */
    public static boolean isAVariableName(String str) {
        boolean retorno = false;
        if (Util.checkString(str)) {
            retorno = str.startsWith("#");
        }
        return retorno;
    }


    /**
     * Retorna o logaritmo do numero passado como parmetro na base 2.
     *
     * @param num Numero para o qual se deseja obter o logaritmo na base 2
     *
     * @return O logaritmo na base 2 do nmero <code>num</code>
     */
    public static double log2(double num) {

        return Math.log(num) / Math.log(2);

    }

    /**
     * Remove os comentrio na hora de realizar o processamento da String
     */
    public static String removeComments(String block) {
        String retorno = block;
        if (checkString(block)) {
            StringBuffer buffer = new StringBuffer();
            StringTokenizer tokenizer = new StringTokenizer(block, "\n\r\t");
            String linha = "";
            while (tokenizer.hasMoreTokens()) {
                linha = tokenizer.nextToken();
                if (checkString(linha)) {
                    linha = linha.trim();
                    // se a linha comeca com '//', entao essa ser ignorada
                    if (!linha.startsWith("//")) {
                        buffer.append(linha);
                    }
                    // coloca um espaco entre cada "linha", para que possa ser
                    // quebrada pelo parser
                    buffer.append(" ");

                }
            }
            retorno = buffer.toString().trim();
        }
        return retorno;
    }


    /**
     * Transforma a String passada como parmetro em um array de Strings. O
     * critrio usado para a separao em outras Strings so os caracteres de
     * quebra de linha : <code>\n</code> e <code>\r</code>.
     *
     * @param block  String correspondente ao bloco que ser quebrado em linhas
     *
     * @return Um array de Strings, correspondente s linhas do bloco.
     */
    public static String[] getLines(String block) {
        String[] retorno = null;
        Vector tmpRetorno = new Vector();
        if (checkString(block)) {
            StringTokenizer tokenizer = new StringTokenizer(block, "\n\r\t");
            String linha = "";
            while (tokenizer.hasMoreTokens()) {
                linha = tokenizer.nextToken();
                if (linha != null) {
                    linha = linha.trim();
                }
                tmpRetorno.addElement(linha);
            }
            retorno = (String[]) tmpRetorno.toArray();
        }
        return retorno;
    }


    private static boolean compatibleIdentifier(INode node) {
        return (node instanceof JIdentifier);
    }

    private static boolean compatibleName(INode node) {
        return (node instanceof JName); /* verificar o tamanho do name*/
    }

    private static boolean compatibleType(INode node) {
        return ((node instanceof JType) &&
                ((node.isVariable()) ||
                ((!node.isVariable()) && ((JType) node).isDimensionless())));

    }

    private static boolean compatibleVariableDeclarator(INode node) {
        boolean retorno = false;
        if (node instanceof JVariableDeclarator) {
            JVariableDeclarator varDec = (JVariableDeclarator) node;
            if (node.isVariable() ||
                    ((!node.isVariable()) &&
                    varDec.getInitializer() == null && varDec.getDimensions() == 0)) {
                retorno = true;
            }
        }
        return retorno;
    }

    /**
     * Verifica a compatibilidade entre os dois ns. Os nicos tipos de ns
     * que podem ser compatveis entre si so:
     *
     * A) JIdentifier, JName, JType e JVariableDeclarator
     *
     * B) JName e JNameList
     *
     * C) Parameter e JParameterList
     *
     *
     * Por enquanto s ser tratado o caso A
     */
    public static boolean compatibleTypes(INode node1, INode node2) {
        boolean retorno = false;
        // verifica se node1  compatvel com JIdentifier, JName,
        // JType ou JVariableDeclarator

        retorno = (compatibleIdentifier(node1) || compatibleName(node1) ||
                compatibleType(node1) || compatibleVariableDeclarator(node1));

        retorno = retorno && (compatibleIdentifier(node2) || compatibleName(node2) ||
                compatibleType(node2) || compatibleVariableDeclarator(node2));


        return retorno;
    }


    public static boolean compatibleVariableTypes(int type1, int type2) {
        final int compatibleTypes = JaTSNode.IDENTIFIER + JaTSNode.NAME +
                JaTSNode.TYPE + JaTSNode.VARIABLE_DECLARATOR;
        return (((type1 & compatibleTypes) > 0) &&
                ((type2 & compatibleTypes) > 0));

    }

    /**
     * Verifica se uma dada varivel pertence  lista de chaves de um mapeamento
     * para isso verifica apenas o nome da varivel. Seu tipo no  levado em
     * considerao. Isso deve-se, basicamente ao fato de haverem tipos
     * compatveis, como JType, JName e JIdentifier.
     *
     * @param  map Mapeamento (var-valor) no qual deseja-se verificar a
     *             existencia da varivel
     * @param  var  Varivel cuja existencia ser verificada no ma
     */
    public static boolean containsVar(Map map, INode var) {
        boolean retorno = false;
        if (map != null && (var != null && var.isVariable())) {
            // pega o nome da variavel que estamos procurando
            String name = var.getVariableName();
            String tmpVarName = null;
            // pega o conjunto de variveis e o seu respectivo Iterator,
            //para que se possa percorrer por este conjunto
            Set keys = map.keySet();
            Iterator it = keys.iterator();
            String tmpVar = null;
            while (it.hasNext() && !retorno) {
                tmpVar = (String) it.next();
                tmpVarName = Util.getVariableName(tmpVar);
                // se encontrar uma variavel de mesmo nome, retorna true
                if (name.equals(tmpVarName)) {
                    retorno = true;
                }
            }
        }
        return retorno;
    }


    /**
     * Retorna apenas o nome da varivel, dado seu nome completo, na forma:
     * <code>variableName:variableType</code>. <br>
     *
     * Ex: Dada a String <code> #BasicType:JIdentifier </code>, a chamada: <br>
     *     <code>Util.getVariableName("#BasicType:JIdentifier")</code> <br>
     *     retornaria: <code>#BasicType</code>
     *
     * @param  fullName  Nome da varivel no formato: <code>variableName:
     * 					variableType</code>
     * @return  Retorna apenas o nome da varivel, no caso <code>variableName</code>
     */
    private static String getVariableName(String fullName) {
        String retorno = null;
        if (Util.checkString(fullName) && Util.isAValidVariableName(fullName)) {
            int indexColon = fullName.indexOf(':');
            if (indexColon > 0) {
                retorno = fullName.substring(0, indexColon);
            }
        }
        return retorno;
    }


    /**
     * Verifica se a String  um nome de varivel JaTS vlido. Para isso,
     * verifica se esta String inicia-se com o caracter de varivel, denotado
     * pela contante AbstractNode.VARIABLE_CHAR
     *
     *
     * @param nomeVar String cuja validade como nome de varivel, ser verificada
     *
     * @return true, se  a String formar um nome vlido para uma varivel JaTS
     */
    public static boolean isAValidVariableName(String nomeVar) {
        boolean retorno = false;
        if (Util.checkString(nomeVar)) {
            char initChar = nomeVar.charAt(0);
            if (initChar == AbstractNode.VARIABLECHAR) {
                retorno = true;
            }
        }
        return retorno;
    }


    /**
     * Retorna true se o vetor passado como parametro possuir apenas objetos
     * do tipo <code>String</code>.
     *
     * @param strVector  O Vetor, para o qual se deseja verificar a existencia
     */
    public static boolean isAStringVector(Vector strVector) {
        boolean retorno = true;
        if (strVector != null) {
            Iterator it = strVector.iterator();
            while (it.hasNext() && retorno) {
                retorno = (it.next() instanceof String);
            }
        } else {
            retorno = false;
        }
        return retorno;
    }

    /**
     * Verifica se o array pasado constitui um array vlido, isto , se
     *  diferente de null e se seu tamanho  maior que zero. Opcionalmente,
     * pode-se checar se os elementos do arrya so, todos eles no-nulos
     *
     * @param  array   Array que se deseja verificar se  valido
     * @param  checkInDepth  Checa em profundidade o array
     *
     * @return true, caso o array seja vlido <br>
     *         false, caso contrario
     */
    public static boolean isAValidArray(Object[] array, boolean checkInDepth) {
        // faz a validao inicial
        boolean retorno = (array != null && array.length > 0);
        // se for pra checar em profundidade e tiver passado pela checagem
        //inicial, checa os elementos do array
        if (checkInDepth && retorno) {
            boolean isOk = true;
            for (int i = 0; i < array.length && isOk; i++) {
                if (array[i] == null) {
                    // achou um elemento do array null, pra o for e
                    // retorna false
                    isOk = false;
                }
            }
            retorno = isOk;
        }
        return retorno;

    }


    /**
     * Retorna um conjunto de declaraes de atributos, dado que so passados
     * um conjunto de strings representando estas declaraes.
     */
    public static JFieldDeclarationSet getFieldDeclarationSet(Vector fields) {
        JFieldDeclarationSet retorno = new JFieldDeclarationSet();
        if (isAStringVector(fields)) {
            Iterator it = fields.iterator();
            INode tmpField = null;
            int i = 0;

            while (it.hasNext()) {
                // no preciso mais fazer o instanceof, uma vez, que isso j foi
                // verificado antes.
                try {
                    String str = (String) it.next();
                    tmpField = JatsIO.parseString(JaTSNode.FIELD_DECLARATION, str);
                    retorno.add(tmpField);
                    i++;
                } catch (ParseException e) {
                    throw new IllegalArgumentException("Erro de parsing da " +
                            "String na posio " + i +
                            " do Vector passado como" +
                            " parmetro");
                }
            }
        }
        return retorno;
    }

    /**
     * Retorna um conjunto de declaraes de metodos, dado que so passados
     * um conjunto de strings representando estas declaraes.
     */
    public static JMethodDeclarationSet getMethodDeclarationSet(Vector methods) {
        JMethodDeclarationSet retorno = new JMethodDeclarationSet();
        if (isAStringVector(methods)) {
            Iterator it = methods.iterator();
            INode tmpMethod = null;
            int i = 0;
            while (it.hasNext()) {
                // no preciso mais fazer o instanceof, uma vez que isso j foi
                // verificado antes.
                try {
                    String str = (String) it.next();
                    tmpMethod = JatsIO.parseString(JaTSNode.METHOD_DECLARATION, str);
                    retorno.add(tmpMethod);
                    i++;
                } catch (ParseException e) {
                    throw new IllegalArgumentException("Erro de parsing da " +
                            "String na posio " + i +
                            " do Vector passado como" +
                            " parmetro");
                }
            }
        }
        return retorno;
    }

    /**
     * Retorna um conjunto de declaraes de contrutores, dado que so passados
     * um conjunto de strings representando estas declaraes.
     */
    public static JConstructorDeclarationSet getConstructorDeclarationSet(Vector contructors) {
        JConstructorDeclarationSet retorno = new JConstructorDeclarationSet();
        if (isAStringVector(contructors)) {
            Iterator it = contructors.iterator();
            INode tmpMethod = null;
            int i = 0;
            while (it.hasNext()) {
                // no preciso mais fazer o instanceof, uma vez, que isso j foi
                // verificado antes.
                try {
                    String str = (String) it.next();
                    tmpMethod = JatsIO.parseString(JaTSNode.CONSTRUCTOR_DECLARATION, str);
                    retorno.add(tmpMethod);
                    i++;
                } catch (ParseException e) {
                    throw new IllegalArgumentException("Erro de parsing da " +
                            "String na posio " + i +
                            " do Vector passado como" +
                            " parmetro");
                }
            }
        }
        return retorno;
    }

    /**
     * Retorna um objeto do Tipo DeclarationSet correspondente ao tipo indicado
     * no parmetro. Os tipos passveis de serem indicados e os respectivos
     * tipos retornados, so: <br>
     *
     * <I>INode.JFieldDeclaration</I>  ou  <I>INode.JFieldDeclarationSet</I> ->
     * cin.jats.engine.parser.nodes.JFieldDeclarationSet  <br>
     * <I>INode.JMethodDeclaration</I>  ou <I>INode.JMethodDeclarationSet</I>->
     * cin.jats.engine.parser.nodes.JMethodDeclarationSet  <br>
     * <I>INode.JConstructorDeclaration</I> ou <I>INode.JConstructorDeclarationSet</I> ->
     * cin.jats.engine.parser.nodes.JConstructorDeclarationSet <br>
     * <I>INode.JImportDeclaration</I>  ou <I>INode.JImportDeclarationSet</I>->
     * cin.jats.engine.parser.nodes.JImportDeclarationSet <br>
     */
    public static DeclarationSet getDeclarationSet(int type) {
        DeclarationSet retorno = null;

        switch (type) {
            case JaTSNode.FIELD_DECLARATION:
            case JaTSNode.FIELD_DECLARATION_SET:
                retorno = new JFieldDeclarationSet();
                break;
            case JaTSNode.METHOD_DECLARATION:
            case JaTSNode.METHOD_DECLARATION_SET:
                retorno = new JMethodDeclarationSet();
                break;
            case JaTSNode.CONSTRUCTOR_DECLARATION:
            case JaTSNode.CONSTRUCTOR_DECLARATION_SET:
                retorno = new JConstructorDeclarationSet();
                break;
            case JaTSNode.IMPORT_DECLARATION:
            case JaTSNode.IMPORT_DECLARATION_SET:
                retorno = new JImportDeclarationSet();
                break;
            default:
                throw new IllegalArgumentException("Tipo invlido para um" +
                        " DeclarationSet: " + type);
        }

        return retorno;
    }


    /**
     * Imprime o CompilationUnitSet
     */
    public static void printSet(CompilationUnitSet compSet, Writer writer)
            throws Exception {

        if (compSet != null && writer != null) {
            JatsIO jatsio = JatsIO.getInstance();
            int size = compSet.size();
            ICompilationUnit tmpComp;
            for (int i = 0; i < size; i++) {
                writer.write("CompilationUnit " + i + "\n");
                tmpComp = compSet.getCompilationUnit(i);
                jatsio.prettyPrint(tmpComp, writer);
            }
            writer.flush();
            writer.close();
        } else {
            throw new IllegalArgumentException("Argumento compSet nulo em " +
                    "CompilationUnitSet.printSet");
        }
    }


    /**
     * Gera uma String correspondente ao nome de uma variavel do tipo
     * comentrio. A qual, por default, termina com a STRING "_#COMMENT_VAR"
     * obviamente, ela no  um identificador vlido e, por isso, nao
     * pode j estar definida em um template
     *
     * Exemplo: <br>
     *    <table>
     *    <tr>
     *       <td> ENTRADA </td>
     *       <td> SAIDA  </td>
     *    </tr>
     *    <tr>
     *       <td> #METHOD </td>
     *       <td> #METHOD_#COMMENT_VAR  </td>
     *    </tr>
     *    <tr>
     *       <td> NOME </td>
     *       <td> #NOME_#COMMENT_VAR </td>
     *    </tr>
     *    </table>
     *
     * @param name  Nome para o qual a varivel ser gerada
     */
    public static String generateCommentVariable(JName name) {
        String retorno = COMMENT_VAR;
        if (name != null) {
            String tmpId = null;
            if (name.isVariable()) {
                tmpId = name.getVariableName();
                retorno = tmpId + COMMENT_VAR;
            } else {
                tmpId = name.toString();
                retorno = AbstractNode.VARIABLECHAR + tmpId + COMMENT_VAR;
            }
        }
        return retorno;
    }


    /**
     * Gera uma String correspondente ao nome de uma variavel do tipo
     * comentrio. A qual, por default, termina com a STRING "_#COMMENT_VAR"
     * obviamente, ela no  um identificador vlido e, por isso, nao
     * pode j estar definida em um template
     *
     * Exemplo: <br>
     *    <table>
     *    <tr>
     *       <td> ENTRADA </td>
     *       <td> SAIDA  </td>
     *    </tr>
     *    <tr>
     *       <td> #METHOD </td>
     *       <td> #METHOD_#COMMENT_VAR  </td>
     *    </tr>
     *    <tr>
     *       <td> NOME </td>
     *       <td> #NOME_#COMMENT_VAR </td>
     *    </tr>
     *    </table>
     */ 
    public static String generateCommentVariable(JIdentifier name) {
        String retorno = COMMENT_VAR;
        if (name != null) {
            String tmpId = null;
            if (name.isVariable()) {
                tmpId = name.getVariableName();
                retorno = tmpId + COMMENT_VAR;
            } else {
                tmpId = name.getValue();
                retorno = AbstractNode.VARIABLECHAR + tmpId + COMMENT_VAR;
            }
        }
        return retorno;
    }


    /**
     * Gera uma String correspondente ao nome de uma variavel do tipo
     * comentrio. A qual, por default, termina com a STRING "_#COMMENT_VAR"
     * obviameente, ela no  um identificador vlido e, por isso, nao
     * pode j estar definida em um template
     *
     * Exemplo: <br>
     *    <table>
     *    <tr>
     *       <td> ENTRADA </td>
     *       <td> SAIDA  </td>
     *    </tr>
     *    <tr>
     *       <td> #METHOD </td>
     *       <td> #METHOD_#COMMENT_VAR  </td>
     *    </tr>
     *    <tr>
     *       <td> NOME </td>
     *       <td> #NOME_#COMMENT_VAR </td>
     *    </tr>
     *    </table>
     *
     * @param jid Identificador, para o qual a varivel ser gerada
     */
    public static String generateTypeCommentVariable(JIdentifier jid) {
        String retorno = COMMENT_VAR;
        if (jid != null) {
            String tmpId = null;
            if (jid.isVariable()) {
                tmpId = jid.getVariableName();
                retorno = tmpId + TYPE_COMMENT_VAR;
            } else {
                tmpId = jid.getValue();
                retorno = AbstractNode.VARIABLECHAR + tmpId + TYPE_COMMENT_VAR;
            }
        }
        return retorno;
    }

    /**
     * Realiza o parsing de uma String correspondente a um numero integral
     * (short, int, long) em Java e retorna um objeto da classe Wrapper
     * correspondente(no caso, Integer ou Long).
     *
     * @param litNumber  String representando o nmero  a ser "parsed"
     *
     * @return  Objeto da classe Wrapper correspondente  String passada.
     */
    public static Number parseIntegralNumber(JLiteral litNumber) {
        Number retorno = null;
        String tmpNumber = "";
        String number = "";
        if (litNumber != null) {
            number = litNumber.getValue();
            if (Util.checkString(number)) {
                if (number.endsWith("l") || number.endsWith("L")) {
                    tmpNumber = number.substring(0, number.length() - 1);
                    retorno = Long.decode(tmpNumber);
                } else if (litNumber.getLiteralType() == JLiteral.LONG_LITERAL) {
                    retorno = Long.decode(number);
                } else {
                    retorno = Integer.decode(number);
                }
            }
        }
        return retorno;
    }

}
