/*
 * JaTS Project - Java Transformation System
 * http://www.cin.ufpe.br/~jats
 *  
 * Copyright  2005  Informatics Center - Federal University of Pernambuco
 * 
 * JaTS is a free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 */
package cin.jats.engine.util;

import java.util.Iterator;
import java.util.Map;
import java.util.Set;


/**
 * Esta classe define mtodos para realizar substituio de variveis
 * dentro de uma String. Diferente do mecanismo do Replacement Visitor
 * aqui as variveis so String e os valores tambm
 * 
 * @author Adeline
 * @version 1.0
 * @since JDK 1.3
 */
public class VariableReplacer {


    /**
     * Substitui uma lista de chaves por uma lista de palavras num String. As listas precisam
     * ter o mesmo tamanho.
     *
     * @param   map      Um map contendo as tags e seus respectivos valores.
     * @param    string         String onde sero substitudas as chaves.
     *
     * @return             Um String com as chaves substitudas.
     */
    public static String substituirTagsString(Map map, String string) {
        String tmp = null;

        if (Util.checkString(string)) {

            Set chaves = map.keySet();
            Iterator iteratorChaves = chaves.iterator();
            String chave = null;
            String tmpChave = null;
            // elimina as linhas de comentrio - esse mtodo dever ser chamado
            // para eliminar as linhas de comentrio. No entanto,  preciso ver, como
            // mant-las no texto integralmente. por isso, ainda no est sendo usada
            // talvez seja melhor manipular um vector com as linhas da String
            // tmp = Util.preprocessBlock(string);

            tmp = string;
            int indice;

            // percorre o string tantas vezes quanto for necessrio para cada chave
            while (iteratorChaves.hasNext()) {
                tmpChave = (String) iteratorChaves.next();
                int indexColon = tmpChave.indexOf(':');

                if (indexColon > 0) {
                    chave = tmpChave.substring(0, indexColon);
                }

                //verifica se a chave est no string
                //se no h mais palavras com a chave no String, vai para a prxima chave
                indice = tmp.indexOf(tmpChave);
                if (indice >= 0) {

                    while (indice >= 0) {
                        //substitui a chave pelo seu map no String
                        String valor = (String) map.get(tmpChave);
                        tmp = replaceVariable(tmp, indice, valor, tmpChave);
                        indice = tmp.indexOf(tmpChave, indice);
                    }

                    //quando eh que cai nesse caso???
                } else if (chave != null) {
                    // verifica pela chave sem a informao de tipo
                    indice = tmp.indexOf(chave);
                    while (indice >= 0) {
                        //substitui a chave pelo seu map no String
                        // pega o valor mapeado a tmpChave
                        String valor = (String) map.get(tmpChave);
                        tmp = replaceVariable(tmp, indice, valor, chave);
                        indice = tmp.indexOf(chave, indice);
                    }
                }
            }
        } else {
            throw new IllegalArgumentException("Argumentos nulos");
        }

        return tmp;
    }


    /**
     * Substitui uma lista de chaves por uma lista de palavras num String. As listas precisam
     * ter o mesmo tamanho.
     *
     * @param   map      Um map contendo as tags e seus respectivos valores.
     * @param   string   String onde sero substitudas as chaves.
     *
     * @return           Um String com as chaves substitudas.
     */
    public static String substituirTagsString(ExpressionsMap map, String string) {
        String tmp = null;

        if (Util.checkString(string)) {

            Iterator iteratorChaves = map.getKeys();
            String chave = null;
            String tmpChave = null;
            // elimina as linhas de comentrio - esse mtodo dever ser chamado
            // para eliminar as linhas de comentrio. No entanto,  preciso ver,
            // como mant-las no texto integralmente. por isso, ainda no est
            // sendo usada talvez seja melhor manipular um vector com as linhas
            // da String
            // tmp = Util.preprocessBlock(string);

            tmp = string;
            int indice;

            // percorre o string tantas vezes quanto for necessrio para cada chave
            while (iteratorChaves.hasNext()) {

                tmpChave = (String) iteratorChaves.next();
                int indexColon = tmpChave.indexOf(':');
                if (indexColon > 0) {
                    chave = tmpChave.substring(0, indexColon);
                }
                //verifica se a chave est no string
                //se no h mais palavras com a chave no String, vai para a
                //prxima chave
                indice = tmp.indexOf(tmpChave);
                if (indice > 0) {
                    while (indice >= 0) {
                        //substitui a chave pelo seu map no String
                        String valor = (String) map.get(tmpChave);
                        tmp = replaceVariable(tmp, indice, valor, tmpChave);
                        indice = tmp.indexOf(tmpChave, indice);
                    }
                } else if (chave != null) {
                    // verifica pela chave sem a informao de tipo
                    indice = tmp.indexOf(chave);
                    while (indice >= 0) {
                        //substitui a chave pelo seu map no String
                        // pega o valor mapeado a tmpChave
                        String valor = (String) map.get(tmpChave);
                        tmp = replaceVariable(tmp, indice, valor, chave);
                        indice = tmp.indexOf(chave, indice);
                    }
                }
            }
        } else {
            throw new IllegalArgumentException("Argumentos nulos");
        }
        return tmp;
    }


    private static String replaceVariable(String tmp, int indice, String valor,
                                          String variable) {
        StringBuffer tmpBuffer;
        tmpBuffer = new StringBuffer();
        tmpBuffer.append(tmp.substring(0, indice));
        tmpBuffer.append(valor);
        tmpBuffer.append(tmp.substring(indice + variable.length(), tmp.length()));
        tmp = tmpBuffer.toString();
        return tmp;
    }


}
