/*
 * JaTS Project - Java Transformation System
 * http://www.cin.ufpe.br/~jats
 *  
 * Copyright  2005  Informatics Center - Federal University of Pernambuco
 * 
 * JaTS is a free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 */
package cin.jats.engine.visitors;

import cin.jats.engine.parser.nodes.*;
 
import cin.jats.engine.parser.nodes.exceptions.ExecutionException;
import cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException;
import cin.jats.engine.parser.nodes.metajats.CodeAssignStatement;
import cin.jats.engine.parser.nodes.metajats.ParsingStatement;
import cin.jats.engine.parser.nodes.metajats.RSetEntryStatement;
import cin.jats.engine.parser.nodes.metajats.TransfDeclaration;

/**
 * Esta classe d uma implementao default para todos os mtodos visit da
 * interface IVisitor - simplemente retorna o n. A idia  perimitir que a
 * criao de classes do pacote cin.jats.engine.parser.nodes seja feita de forma
 * mais fcil, uma vez que os mtodos visit estaro todos implementados aqui.
 *
 * @author Adeline de Sousa
 * @version 12/11/2002
 */
public abstract class AbstractVisitor implements IVisitor {


    /**
	 * @see JAllocationExpression
	 */
    public Object visit(JAllocationExpression node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {
        return node;
    }

    /**
	 * @see JAnonymousClass
	 */
    public Object visit(JAnonymousClass node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {
        return node;
    }

    /**
	 * @see JArrayAccess
	 */
    public Object visit(JArrayAccess node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {
        return node;
    }

    /**
	 * @see JArrayDimsAndInits
	 */
    public Object visit(JArrayDimsAndInits node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {
        return node;
    }

    /**
	 * @see JArrayInitializer
	 */
    public Object visit(JArrayInitializer node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {
        return node;
    }

    /**
	 * @see Attribute
	 */
//    public Object visit(Attribute node, Object data)
//            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
//            ExecutionException {
//        return node;
//    }

//    /**
//	 * @see AttributeArgument
//	 */
//    public Object visit(AttributeArgument node, Object data)
//            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
//            ExecutionException {
//        return node;
//    }
//
//    /**
//	 * @see AttributeArgumentList
//	 */
//    public Object visit(AttributeArgumentList node, Object data)
//            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
//            ExecutionException {
//        return node;
//    }

    /**
	 * @see AttributeSection
	 */
//    public Object visit(AttributeSection node, Object data)
//            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
//            ExecutionException {
//        return node;
//    }


    /**
	 * @see JBinaryExpression
	 */
    public Object visit(JBinaryExpression node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {
        return node;
    }

    /**
	 * @see JBlock
	 */
    public Object visit(JBlock node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {
        return node;
    }

    /**
	 * @see CastExpression
	 */
    public Object visit(CastExpression node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {
        return node;
    }

    /**
	 * @see JCatchClause
	 */
    public Object visit(JCatchClause node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {
        return node;
    }

    /**
	 * @see TypeBody
	 */
    public Object visit(TypeBody node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {
        return node;
    }
    

    /**
	 * @see JClassDeclaration
	 */
    public Object visit(JClassDeclaration node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {
        return node;
    }

    /**
	 * @see JCompilationUnit
	 */
    public Object visit(JCompilationUnit node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {
        return node;
    }

    /**
	 * @see JConditionalDeclaration
	 */
    public Object visit(JConditionalDeclaration node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {
        return node;
    }

    /**
	 * @see JConstructorDeclaration
	 */
    public Object visit(JConstructorDeclaration node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {
        return node;
    }

    /**
	 * @see JConstructorDeclarationSet
	 */
    public Object visit(JConstructorDeclarationSet node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {
        return node;
    }

    /**
	 * @see ClassBodyConstructorDeclarations
	 */
    public Object visit(ClassBodyConstructorDeclarations node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {
        return node;
    }

    /**
	 * @see JExecutableDeclaration
	 */
    public Object visit(JExecutableDeclaration node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {
        return node;
    }

    /**
	 * @see JExpressionList
	 */
    public Object visit(JExpressionList node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {
        return node;
    }

    /**
	 * @see JFieldAccess
	 */
    public Object visit(JFieldAccess node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {
        return node;
    }

    /**
	 * @see JFieldDeclaration
	 */
    public Object visit(JFieldDeclaration node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {
        return node;
    }

    /**
	 * @see JFieldDeclarationSet
	 */
    public Object visit(JFieldDeclarationSet node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {
        return node;
    }

    /**
	 * @see JForStatement
	 */
    public Object visit(JForStatement node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {
        return node;
    }

    /**
	 * @see BodyFieldDeclarations
	 */
    public Object visit(BodyFieldDeclarations node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {
        return node;
    }

    /**
	 * @see ClassBodyInitializers
	 */
    public Object visit(ClassBodyInitializers node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {
        return node;
    }

    /**
	 * @see JFormalParameter
	 */
    public Object visit(JFormalParameter node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {
        return node;
    }

    /**
	 * @see JIdentifier
	 */
    public Object visit(JIdentifier node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {
        return node;
    }

    /**
	 * @see JIfStatement
	 */
    public Object visit(JIfStatement node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {
        return node;
    }

    /**
	 * @see JImportDeclaration
	 */
    public Object visit(JImportDeclaration node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {
        return node;
    }

    /**
	 * @see JImportDeclarationSet
	 */
    public Object visit(JImportDeclarationSet node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {
        return node;
    }

    /**
	 * @see ImportDeclarations
	 */
    public Object visit(ImportDeclarations node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {
        return node;
    }

    /**
	 * @see JInitializer
	 */
    public Object visit(JInitializer node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {
        return node;
    }

    /**
	 * @see JInitializerSet
	 */
    public Object visit(JInitializerSet node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {
        return node;
    }

    /**
	 * @see JInterfaceDeclaration
	 */
    public Object visit(JInterfaceDeclaration node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {
        return node;
    }

    /**
	 * @see InstanceOfExpression
	 */
    public Object visit(InstanceOfExpression node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {
        return node;
    }

    /**
	 * @see IterativeDeclaration
	 */
    public Object visit(IterativeDeclaration node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {
        return node;
    }

    /**
	 * @see JLiteral
	 */
    public Object visit(JLiteral node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {
        return node;
    }

    /**
	 * @see JLocalVariableDeclaration
	 */
    public Object visit(JLocalVariableDeclaration node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {

        return node;
    }


    /**
	 * @see JMethodDeclaration
	 */
    public Object visit(JMethodDeclaration node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {
        return node;
    }

    /**
	 * @see JMethodDeclarationSet
	 */
    public Object visit(JMethodDeclarationSet node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {
        return node;
    }

    /**
	 * @see BodyMethodDeclarations
	 */
    public Object visit(BodyMethodDeclarations node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {
        return node;
    }

    /**
	 * @see BodyTypeDeclarations
	 */
    public Object visit(BodyTypeDeclarations node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {
        return node;
    }

	
    /**
	 * @see JMethodInvocation
	 */
    public Object visit(JMethodInvocation node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {
        return node;
    }

    /**
	 * @see JModifierList
	 */
    public Object visit(JModifierList node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {
        return node;
    }

    /**
	 * @see JName
	 */
    public Object visit(JName node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {
        return node;
    }

    /**
	 * @see JNodeString
	 */
    public Object visit(JNodeString node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {
        return node;
    }

    /**
	 * @see JNameList
	 */
    public Object visit(JNameList node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {
        return node;
    }

    /**
	 * @see OtherDeclarationsSet
	 */
    public Object visit(OtherDeclarationsSet node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {
        return node;
    }

    /**
	 * @see JPackageDeclaration
	 */
    public Object visit(JPackageDeclaration node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {
        return node;
    }

    /**
	 * @see JParameterList
	 */
    public Object visit(JParameterList node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {
        return node;
    }

    /**
	 * @see JPreconditionDeclaration
	 */
    public Object visit(JPreconditionDeclaration node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {
        return node;
    }


    /**
	 * @see JResultStatement
	 */
    public Object visit(JResultStatement node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {
        return node;
    }

    /**
	 * @see JSwitchStatement
	 */
    public Object visit(JSwitchStatement node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {
        return node;
    }

    /**
	 * @see JSwitchLabel
	 */
    public Object visit(JSwitchLabel node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {
        return node;
    }

    /**
	 * @see JSwitchPair
	 */
    public Object visit(JSwitchPair node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {
        return node;
    }

    /**
	 * @see JTernaryExpression
	 */
    public Object visit(JTernaryExpression node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {

        return node;
    }

    /**
	 * @see JTryStatement
	 */
    public Object visit(JTryStatement node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {

        return node;
    }

    /**
	 * @see JType
	 */
    public Object visit(JType node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {
        return node;
    }

    /**
	 * @see TypeDeclarationSet
	 */
    public Object visit(TypeDeclarationSet node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {
        return node;
    }

    /**
	 * @see JUnaryExpression
	 */
    public Object visit(JUnaryExpression node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {
        return node;
    }


    /**
	 * @see JVariableDeclarator
	 */
    public Object visit(JVariableDeclarator node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {
        return node;
    }

    /**
	 * @see JWhileStatement
	 */
    public Object visit(JWhileStatement node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {
        return node;
    }

    public Object visit(WrapperNode node, Object data) throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {
        return node;
    }

    public Object visit(JSynchronizedStatement node, Object data) throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, ExecutionException {
        return node;
    }

    /**
	 * @see JStatementExpression
	 */
    public Object visit(JStatementExpression node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {
        return node;
    }

    
    
//    public Object visit(EnumDeclaration node, Object data)
//			throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
//			ExecutionException
//	{
//    	return node;
//	}
//
//    public Object visit(EnumMember node, Object data)
//			throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
//			ExecutionException
//	{
//    	return node;
//	}


    
    /**
	 * @see MapTable
	 */
    public Object visit(MapTable node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {
        return node;
    }


    /**
	 * @see OrDeclaration
	 */
    public Object visit(OrDeclaration node, Object data) {
        return node;
    }

    public Object visit(ExploreDeclaration node, Object data)
			throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException,
			IllegalArgumentException, ExecutionException {
		return node;
	}

	public Object visit(WhereClause node, Object data)
			throws IllegalArgumentException, InconsistentNodeException,
			ExecutionException {
		return node;
	}

	public Object visit(MicroTransformationNode node, Object data)
			throws IllegalArgumentException, InconsistentNodeException,
			ExecutionException {
		return node;
	}

	public Object visit(Pattern node, Object data)
			throws IllegalArgumentException, InconsistentNodeException,
			ExecutionException {
		return node;
	}

	public Object visit(PrefixedIdentifier node, Object data)
			throws IllegalArgumentException, InconsistentNodeException,
			ExecutionException {
		return node;
	}

	public Object visit(ContextDeclaration node, Object data)
			throws IllegalArgumentException, InconsistentNodeException,
			ExecutionException {
	    return node;
	}

	public Object visit(JStatementList node, Object data)
			throws IllegalArgumentException, InconsistentNodeException,
			ExecutionException {
		return node;
	}
	
    public Object visit(CodeAssignStatement node, Object data)
			throws IllegalArgumentException, InconsistentNodeException,
			ExecutionException {
		return node;
	}
    
	public Object visit(ParsingStatement node, Object data)
			throws IllegalArgumentException, InconsistentNodeException,
			ExecutionException {
		return node;
	}
	
	public Object visit(RSetEntryStatement node, Object data)
			throws IllegalArgumentException, InconsistentNodeException,
			ExecutionException {
		return node;
	}
	
	public Object visit(TransfDeclaration node, Object data)
			throws IllegalArgumentException, InconsistentNodeException,
			ExecutionException {
		return node;
	}
	
}
