/*
 * JaTS Project - Java Transformation System
 * http://www.cin.ufpe.br/~jats
 *  
 * Copyright  2005  Informatics Center - Federal University of Pernambuco
 * 
 * JaTS is a free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 */
package cin.jats.engine.visitors;


import cin.jats.engine.parser.nodes.*;

import cin.jats.engine.parser.nodes.exceptions.*;
import cin.jats.engine.transformation.MicroTransformation;
import cin.jats.engine.util.NodeList;
import cin.jats.engine.util.NodeStack;


/**
 * Classe que implementa a interface <code>IVisitor</code> por meio de um
 * visitor que realiza a clonagem de uma rvore sinttica.
 *
 * @see INode
 * @see IVisitor
 *
 * @author Fernando Castor
 * @author Adeline Sousa
 * @author Gustavo Santos
 *
 * @version 13/10/2005
 */
public class CloningVisitor extends AbstractVisitor {


    // Pilha responsvel por guardar o estado atual da clonagem.
    protected NodeStack state;


    /**
     * Construtor vazio da classe.
     */
    public CloningVisitor() {
        state = new NodeStack();
    }
	

//    public  Object visit( EnumMember node,  Object data) throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException{
//        if (node == null) {
//            throw new IllegalArgumentException("node is null");
//        }
//        EnumMember resultNode = new EnumMember();
//        if (node.isASourceNode()) {
//            if (node.getAttributes() != null ) {
//            	if (!(this.state.top() instanceof AttributeSectionList)) {
//            		throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
//            	}
//            	resultNode.setAttributes((AttributeSectionList)this.state.pop());
//            }
//            
//        	if (node.getIdentifier() == null || !(this.state.top() instanceof JIdentifier)) {
//                throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
//            }
//            resultNode.setIdentifier((JIdentifier)this.state.pop());
//            
//            if (node.getExpression() != null) {
//            	if (!(this.state.top() instanceof JExpression)) {
//            		throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
//            	}
//                resultNode.setExpression((JExpression)this.state.pop());
//            }
//        }
//        this.doCommonCloningWork(node, resultNode);
//        return resultNode;
//    }

//    public  Object visit( EnumDeclaration node,  Object data) throws IllegalArgumentException, cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException{
//        if (node == null) {
//            throw new IllegalArgumentException("node is null");
//        }
//        EnumDeclaration resultNode = new EnumDeclaration();
//        if (node.isASourceNode()) {
//            if (node.getAttributes() != null) {
//            	if(!(this.state.top() instanceof AttributeSectionList)) {
//            		throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
//            	}
//            	resultNode.setAttributes((AttributeSectionList)this.state.pop());
//            }
//            
//        	if (node.getModifiers() == null || !(this.state.top() instanceof JModifierList)) {
//                throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
//            }
//            resultNode.setModifiers((JModifierList)this.state.pop());
//            if (node.getName() == null || !(this.state.top() instanceof JIdentifier)) {
//                throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
//            }
//            resultNode.setName((JIdentifier)this.state.pop());
//            
//            if (node.getType() != null) {
//            	if (!(this.state.top() instanceof JType)) {
//            		throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
//            	}
//                resultNode.setType((JType)this.state.pop());
//            }
//            
//            EnumMember tmp = null;
//			int i = 0;
//			int size = node.size();
//
//			while (i < size) {
//				if (this.state.top() instanceof EnumMember) {
//					tmp = (EnumMember) this.state.pop();
//					resultNode.getEnumMembers().addElement(tmp);
//				}
//				i++;
//			}
//        }
//        this.doCommonCloningWork(node, resultNode);
//        return resultNode;
//    }


//    public  Object visit( AttributeSectionList node,  Object data) throws IllegalArgumentException, cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException{
//        if (node == null) {
//            throw new IllegalArgumentException("node is null");
//        }
//        AttributeSectionList resultNode = new AttributeSectionList();
//        if (node.isASourceNode()) {
//            int max = node.size();
//            for(int i = 0; i < max; i++ ){
//                resultNode.addAttributeSection((AttributeSection)this.state.pop());
//            }
//        }
//        this.doCommonCloningWork(node, resultNode);
//        return resultNode;
//    }
//
//    public  Object visit( AttributeSection node,  Object data) throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException{
//        if (node == null) {
//            throw new IllegalArgumentException("node is null");
//        }
//        AttributeSection resultNode = new AttributeSection();
//        if (node.isASourceNode()) {
//            if (node.getAttributeTarget() == null || !(this.state.top() instanceof JIdentifier)) {
//                throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
//            }
//            resultNode.setAttributeTarget((JIdentifier)this.state.pop());
//            if (node.getAttributes() == null || !(this.state.top() instanceof AttributeList)) {
//                throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
//            }
//            resultNode.setAttributes((AttributeList)this.state.pop());
//        }
//        this.doCommonCloningWork(node, resultNode);
//        return resultNode;
//    }
//
//    public  Object visit( AttributeList node,  Object data) throws IllegalArgumentException, cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException{
//        if (node == null) {
//            throw new IllegalArgumentException("node is null");
//        }
//        AttributeList resultNode = new AttributeList();
//        if (node.isASourceNode()) {
//            int max = node.size();
//            for(int i = 0; i < max; i++ ){
//                resultNode.addAttribute((Attribute)this.state.pop());
//            }
//        }
//        this.doCommonCloningWork(node, resultNode);
//        return resultNode;
//    }

//    public  Object visit( AttributeArgumentList node,  Object data) throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException{
//        if (node == null) {
//            throw new IllegalArgumentException("node is null");
//        }
//        
//        AttributeArgumentList resultNode = new AttributeArgumentList();
//        if (node.isASourceNode()) {
//            int max = node.size();
//            for(int i = 0; i < max; i++ ){
//                resultNode.addArgument((AttributeArgument)this.state.pop());
//            }
//        }
//        
//        this.doCommonCloningWork(node, resultNode);
//        return resultNode;
//    }
//
//    public  Object visit( AttributeArgument node,  Object data) throws IllegalArgumentException, cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException{
//        if (node == null) {
//            throw new IllegalArgumentException("node is null");
//        }
//        AttributeArgument resultNode = new AttributeArgument();
//        if (node.isASourceNode()) {
//            if (node.getIdentifier() != null) {
//            	if(!(this.state.top() instanceof JIdentifier)) {
//            		throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
//            	}
//                resultNode.setIdentifier((JIdentifier)this.state.pop());
//            }
//
//            if (node.getExpression() == null || !(this.state.top() instanceof JExpression)) {
//                throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
//            }
//            resultNode.setExpression((JExpression)this.state.pop());
//        }
//        this.doCommonCloningWork(node, resultNode);
//        return resultNode;
//    }

//    public  Object visit( Attribute node,  Object data) throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException{
//        if (node == null) {
//            throw new IllegalArgumentException("node is null");
//        }
//        Attribute resultNode = new Attribute();
//        if (node.isASourceNode()) {
//            if (node.getName() == null || !(this.state.top() instanceof JName)) {
//                throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
//            }
//            resultNode.setName((JName)this.state.pop());
//            
//            if (node.getArgumentList() != null) {
//            	if (!(this.state.top() instanceof AttributeArgumentList)) {
//            		throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
//            	}
//                resultNode.setArgumentList((AttributeArgumentList)this.state.pop());
//            }
//        }
//        this.doCommonCloningWork(node, resultNode);
//        return resultNode;
//    }

    /**
     *  Mtodo auxiliar para evitar repetio de cdigo na clonagem de
     * declaraes  condicionais.
     */
    public void cloneConditionalDeclaration(JConditionalDeclaration node,
                                            JConditionalDeclaration resultNode)
            throws IllegalArgumentException, cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException {

        if (node == null) {
            throw new IllegalArgumentException();
        }

        // Clona a lista de declaraes que  processada caso a condio avalie
        // para false.
        resultNode.setFalseDeclarationSet(
                this.cloneFalseDeclarationSetOfConditionalDeclaration(node, resultNode));

        // Clona a lista de declaraes que  processada caso a condio avalie
        // para true.
        resultNode.setTrueDeclarationSet(
                this.cloneTrueDeclarationSetOfConditionalDeclaration(node, resultNode));

        // Clona a condio da declarao condicional.
        resultNode.setCondition(
                this.cloneConditionConditionalDeclaration(node));

        this.doCommonCloningWork(node, resultNode);
    }

    /**
     * Mtodo auxiliar para a clonagem de declaraes condicionais
     */
    private JExpression cloneConditionConditionalDeclaration(
            JConditionalDeclaration node) throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException {

        JExpression resultNode = null;

        if (node.getCondition() == null
                || !(this.state.top() instanceof JExpression)) {
            throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
        }

        resultNode = (JExpression) this.state.pop();
        return resultNode;
    }

    /**
     *  Mtodo auxiliar na clonagem de declaraes iterativas.
     */
    private NodeList cloneDeclarationSetOfIterativeDeclaration(
            IterativeDeclaration node, IterativeDeclaration resultNode) {

        NodeList decSet = new NodeList();

        int i = node.getDeclarationSet().size() - 1;
        while (i >= 0) {
            INode dec = this.state.pop();
            decSet.addElement(dec);
            if (dec instanceof JConditionalDeclaration) {
                ((JConditionalDeclaration) dec).setParentNode(resultNode);
            }
            i--;
        }

        return decSet;
    }

    /**
     * Mtodo auxiliar para realizar a clonagem de declaraes executveis.
     */
    private INode cloneExecutableDeclaration(INode node, INode resultNode)
            throws IllegalArgumentException, cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException {

        if (node == null) {
            throw new IllegalArgumentException();
        }

        INode retorno = null;

        INode topoPilha = this.state.top();

        if (!((topoPilha instanceof JExecutableDeclaration)
                || (topoPilha instanceof NestableExecutableDeclaration))
                && node.isExecutable()) {
            throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
        } else if (((topoPilha instanceof JExecutableDeclaration)
                || (topoPilha instanceof NestableExecutableDeclaration))
                && (node.isExecutable())) {
            retorno = this.state.pop();
        }

        if (retorno instanceof NestableExecutableDeclaration) {
            ((NestableExecutableDeclaration) retorno).setParentNode(resultNode);
        }

        return retorno;
    }

    /**
     * Mtodo auxiliar na clonagem de declaraes condicionais.
     */
    private NodeList cloneFalseDeclarationSetOfConditionalDeclaration(
            JConditionalDeclaration node, JConditionalDeclaration resultNode) {

        NodeList decSet = new NodeList();
        NodeList fDecSet = node.getFalseDeclarationSet();

        if (fDecSet != null) {
            int i = fDecSet.size() - 1;
            while (i >= 0) {
                INode dec = this.state.pop();
                decSet.addElement(dec);
                i--;
            }
            setParentNode(decSet, resultNode);
        }
        return decSet;
    }

    /**
     * Mtodo auxiliar na clonagem de declaraes iterativas.
     */
    private INode cloneIterationSetOfIterativeDeclaration(
            IterativeDeclaration node) throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException {

        INode iterSet = null;

        if (node.getIterationSet() == null) {
            throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
        }
        iterSet = (INode) this.state.pop();

        return iterSet;
    }

    /**
     *  Mtodo auxiliar para evitar repetio de cdigo na clonagem de declaraes
     *  iterativas.
     */
    private void cloneIterativeDeclaration(IterativeDeclaration node,
                                           IterativeDeclaration resultNode)
            throws IllegalArgumentException, cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException {

        if (node == null) {
            throw new IllegalArgumentException();
        }

        // Clona o iteration set do n.
        resultNode.setIterationSet(
                this.cloneIterationSetOfIterativeDeclaration(node));

        // Clona o declaration set do n.
        resultNode.setDeclarationSet(
                this.cloneDeclarationSetOfIterativeDeclaration(node, resultNode));

        // Clona o placeholder do n.
        resultNode.setPlaceholder(
                this.clonePlaceholderOfIterativeDeclaration(node));

        setParentNode(resultNode.getDeclarationSet(), resultNode);


        resultNode.setParentNode(node.getParentNode());

        this.doCommonCloningWork(node, resultNode);
    }

    /**
     * Mtodo auxiliar que coloca <code>parent</code> como o n pai de todas
     * as instncias da <code>NestableExecutableDeclaration</code>.
     */
    private static void setParentNode(NodeList elements, INode parent) {
        int i = 0;
        int max = elements.size();

        while (i < max) {
            INode tempNode = elements.elementAt(i);

            if (tempNode instanceof NestableExecutableDeclaration) {
                ((NestableExecutableDeclaration) tempNode).
                        setParentNode(parent);
            }
            i++;
        }
    }


    /**
     * Mtodo auxiliar para a clonagem de declaraes condicionais
     */
    private INode cloneParentNodeConditionalDeclaration(
            JConditionalDeclaration node) throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException {

        INode resultNode = null;

        if (node.getParentNode() == null) {
            throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
        }

        resultNode = this.state.pop();
        return resultNode;
    }

    /**
     * Mtodo auxiliar na clonagem de declaraes iterativas.
     */
    private INode clonePlaceholderOfIterativeDeclaration(
            IterativeDeclaration node) throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException {

        INode ph = null;

        if (node.getPlaceholder() == null) {
            throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
        }
        ph = (INode) this.state.pop();

        return ph;
    }

    /**
     * Mtodo auxiliar na clonagem de declaraes condicionais.
     */
    private NodeList cloneTrueDeclarationSetOfConditionalDeclaration(
            JConditionalDeclaration node, JConditionalDeclaration resultNode) {

        NodeList decSet = new NodeList();
        NodeList tDecSet = node.getTrueDeclarationSet();

        if (tDecSet != null) {
            int i = tDecSet.size() - 1;
            while (i >= 0) {
                INode dec = this.state.pop();
                decSet.addElement(dec);
                i--;
            }
            setParentNode(decSet, resultNode);
        }

        return decSet;
    }

    private JStatementList cloneStatementList(JStatementList node,
			JStatementList resultNode) throws IllegalArgumentException,
			InconsistentNodeException {

		if (node == null || resultNode == null) {
			throw new IllegalArgumentException();
		}

		if (node.isASourceNode()) {

			for (int i = 0; i < node.size(); i++) {
				resultNode.add(this.state.pop());

				// seta o pai das declaracoes iterativas e condicionais
				if (resultNode.elementAt(i) instanceof NestableExecutableDeclaration) {
					((NestableExecutableDeclaration) resultNode.elementAt(i))
							.setParentNode(resultNode);
				}
			}
		}
		
		if (node.getPosition() != null) {
		    resultNode.setPosition(node.getPosition());
		}

		return resultNode;
	}
    
    
    /**
	 * Mtodo auxiliar para evitar repetio de cdigo. Realiza algumas tarefas
	 * comuns  clonagem de todos os tipos de n, exceto s declaraes
	 * executveis e iterativas e s pr-condies.
	 */
    protected void doCommonCloningWork(INode node, AbstractNode resultNode)
            throws IllegalArgumentException, cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException {

        if (node == null || resultNode == null) {
            throw new IllegalArgumentException();
        }
//        resultNode.setIndentLevel(node.getIndentLevel());
        resultNode.cloneVariable(node);
        resultNode.setExecutableDeclaration(this.cloneExecutableDeclaration(node, resultNode));
        resultNode.setLine(((JaTSNode)node).getLine());
        AbstractNode newNode = (AbstractNode) node;
        if (newNode.isOrDeclaration()) {
            resultNode.setOrDeclaration(newNode.getOrDeclaration());
        }
        this.state.push(resultNode);
    }

    /**
     * Retorna a raiz da rvore clonada.
     *
     * @return Um objeto do tipo <code>INode</code> correspondendo  raiz
     * da rvore clonada.
     */
    public INode getClone() {
        return this.state.pop();
    }

    /**
     * Mtodos visit para objetos do tipo <code>JAllocationExpression</code>.
     *
     * @param node O n a ser clonado.
     * @param data Um objeto que pode conter informao adicional necessria ao
     * visitor.
     *
     * @return Um objeto qualquer.
     *
     * @exception java.lang.IllegalArgumentException Levantada quando o
     * parmetro <code>node</code> for <code>null</code>.
     * @exception java.lang.IllegalArgumentException Levantada quando um n
     * apresenta uma estrutura interna invlida.
     */
    public Object visit(JAllocationExpression node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException {
        if (node == null) {
            throw new IllegalArgumentException();
        }

        JAllocationExpression resultNode = new JAllocationExpression();

        if (node.getAllocationType() == JAllocationExpression.OBJECTALLOCATION) {

            //se tiver o corpo
            if (node.getInternalClassBody() != null) {

                if (!(this.state.top() instanceof TypeBody)) {
                    throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
                }
                TypeBody body = (TypeBody) this.state.pop();
                resultNode.setInternalClassBody(body);

            }
            if (node.getArguments() == null
                    || !(this.state.top() instanceof JExpressionList)) {
                throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
            }
            JExpressionList args = (JExpressionList) this.state.pop();
            resultNode.setArguments(args);

        } else if (node.getAllocationType() ==
                JAllocationExpression.PRIMITIVETYPEARRAYALLOCATION
                || node.getAllocationType() ==
                JAllocationExpression.OBJECTARRAYALLOCATION) {

            if (node.getArrayDimsAndInits() == null
                    || !(this.state.top() instanceof JArrayDimsAndInits)) {
                throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
            }

            JArrayDimsAndInits adi = (JArrayDimsAndInits) this.state.pop();
            resultNode.setArrayDimsAndInits(adi);
        }


        if (node.getTypeName() == null
                || !(this.state.top() instanceof JName)) {
            throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
        }
        resultNode.setName((JName) this.state.pop());


        if (node.getReferencedObject() != null) {
            System.out.println("JAllocationExpression" + this.state.top());
            resultNode.setReferencedObject(this.state.pop());
        }

        resultNode.setAllocationType(node.getAllocationType());
        this.doCommonCloningWork(node, resultNode);
        return resultNode;
    }

    /**
     * @see JAnonymousClass
     */
    public Object visit(JAnonymousClass node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {

        JAnonymousClass resultNode = new JAnonymousClass();
        if (node.isASourceNode()) {

            if ((node.getName() == null) || !(this.state.top() instanceof JName)) {
                throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
            }
            JName name = (JName) this.state.pop();
            resultNode.setName(name);

            if (node.getArguments() != null &&
                    !(this.state.top() instanceof JExpressionList)) {
                throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
            }
            JExpressionList args = (JExpressionList) this.state.pop();
            resultNode.setArguments(args);

            if (node.getBody() == null ||
                    !(this.state.top() instanceof TypeBody)) {
                throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
            }

            TypeBody classBody = (TypeBody) this.state.pop();
            resultNode.setBody(classBody);
        }

        this.doCommonCloningWork(node, resultNode);
        return resultNode;
    }

    /**
     * Revisado em 01/07/2004
     * @see JArrayAccess
     */
    public Object visit(JArrayAccess node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {
        if (node == null) {
            throw new IllegalArgumentException();
        }

        JArrayAccess resultNode = new JArrayAccess();

        if (node.isASourceNode()) {
            INode arrayName = null;
            if (node.getArrayName() == null) {
                throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
            }
            arrayName = (INode) this.state.pop();

            if (!(arrayName != null)) {
                throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
            }
            resultNode.setArrayName(arrayName);

            if (node.getIndexers() != null) {
                if (!(this.state.top() instanceof JExpressionList)) {
                    throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
                }
                resultNode.setIndexers((JExpressionList) this.state.pop());

            }

        }

        this.doCommonCloningWork(node, resultNode);
        return resultNode;
    }

    /**
     * Mtodos visit para objetos do tipo <code>JArrayDimsAndInits</code>.
     * @param node O n a ser clonado.
     * @param data Um objeto que pode conter informao adicional necessria ao
     * visitor.
     *
     * @return Um objeto qualquer.
     *
     * @exception java.lang.IllegalArgumentException Levantada quando o
     * parmetro <code>node</code> for <code>null</code>.
     * @exception java.lang.IllegalArgumentException Levantada quando um n
     * apresenta uma estrutura interna invlida.
     */
    public Object visit(JArrayDimsAndInits node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException {
        if (node == null) {
            throw new IllegalArgumentException();
        }

        JArrayDimsAndInits resultNode = new JArrayDimsAndInits();
        NodeList dimSizes = new NodeList();

        int i = 0;

        resultNode.setDimensions(node.getDimensions());

        if (node.getArrayInitializer() != null) {
            INode tempNode = this.state.pop();
            if (!(tempNode instanceof JArrayInitializer))
                throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);

            resultNode.setArrayInitializer((JArrayInitializer) tempNode);
        }

        if (node.getNumberOfDimensionSizesSpecified() > 0) {
            i = node.getNumberOfDimensionSizesSpecified();
            while (i > 0) {
                // Os dimension sizes so armazenados de trs pra frente, ou
                // seja, para um array declarado int[3][4][1+x], os tamanhos
                // sero armazenados na lista como < "1+x", "4", "3">.
                dimSizes.addElement(this.state.pop());
                i--;
            }
            int j = 1, max = dimSizes.size();
            while (j <= max) {
                INode tempNode = dimSizes.elementAt(max - j);
                if (!(tempNode instanceof JExpression))
                    throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);

                resultNode.setSizeOfDimension((JExpression) tempNode, j);
                j++;
            }
        }

        this.doCommonCloningWork(node, resultNode);

        return resultNode;
    }

    /**
     * Mtodos visit para objetos do tipo <code>JArrayInitializer</code>.
     *
     * @param node O n a ser clonado.
     * @param data Um objeto que pode conter informao adicional necessria ao
     * visitor.
     *
     * @return Um objeto qualquer.
     *
     * @exception java.lang.IllegalArgumentException Levantada quando o
     * parmetro <code>node</code> for <code>null</code>.
     * @exception java.lang.IllegalArgumentException Levantada quando um n
     * apresenta uma estrutura interna invlida.
     */
    public Object visit(JArrayInitializer node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException {
        if (node == null) {
            throw new IllegalArgumentException();
        }

        JArrayInitializer resultNode = new JArrayInitializer();
        int i = node.size();
        NodeList inits = new NodeList();

        while (i > 0) {
            inits.addElement(this.state.pop());
            i--;
        }

        int j = 0, max = inits.size();
        while (j < max) {
            INode tempNode = inits.elementAt(max - (j + 1));

            if (!(tempNode instanceof JExpression
                    || tempNode instanceof JArrayInitializer))
                throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
            resultNode.addInitializer(tempNode);
            j++;
        }

        this.doCommonCloningWork(node, resultNode);

        return resultNode;
    }

    /**
     * Mtodos visit para objetos do tipo <code>JBinaryExpression</code>.
     *
     * @param node O n a ser clonado.
     * @param data Um objeto que pode conter informao adicional necessria ao
     * visitor.
     *
     * @return Um objeto qualquer.
     *
     * @exception java.lang.IllegalArgumentException Levantada quando o
     * parmetro <code>node</code> for <code>null</code>.
     * @exception java.lang.IllegalArgumentException Levantada quando um n
     * apresenta uma estrutura interna invlida.
     */
    public Object visit(JBinaryExpression node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException {
        if (node == null) {
            throw new IllegalArgumentException();
        }

        //elemento gambiarroso para permitir a criacao de
        //objetos AssignmentExp e AssignmentStm sem metodos
        //especificos no visitor
        JBinaryExpression resultNode = null;
        try {
			resultNode = (JBinaryExpression) node.getClass().newInstance();
		} catch (InstantiationException e) {
		} catch (IllegalAccessException e) {
		}

        if (node.getRightHandOperand() == null
                || !(this.state.top() instanceof JExpression))
            throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
        JExpression rightHand = (JExpression) this.state.pop();

        if (node.getLeftHandOperand() == null
                || !(this.state.top() instanceof JExpression))
            throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
        JExpression leftHand = (JExpression) this.state.pop();

        resultNode.setLeftHandOperand(leftHand);
        resultNode.setRightHandOperand(rightHand);

        resultNode.setOperator(node.getOperator());
        this.doCommonCloningWork(node, resultNode);

        return resultNode;
    }

    /**
     * Mtodos visit para objetos do tipo <code>JBlock</code>.
     *
     * @param node O n a ser clonado.
     * @param data Um objeto que pode conter informao adicional necessria ao
     * visitor.
     *
     * @return Um objeto qualquer.
     *
     * @exception java.lang.IllegalArgumentException Levantada quando o
     * parmetro <code>node</code> for <code>null</code>.
     * @exception java.lang.IllegalArgumentException Levantada quando um n
     * apresenta uma estrutura interna invlida.
     */
    public Object visit(JBlock node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException {

        if (node == null) {
            throw new IllegalArgumentException();
        }
        
        JBlock resultNode = new JBlock();

    	resultNode.setStatements((JStatementList)this.state.pop());
        
        this.doCommonCloningWork(node, resultNode);

        return resultNode;
    }

    
    /**
     * Mtodos visit para objetos do tipo <code>CastExpression</code>.
     *
     * @param node O n a ser clonado.
     * @param data Um objeto que pode conter informao adicional necessria ao
     * visitor.
     *
     * @return Um objeto qualquer.
     *
     * @exception java.lang.IllegalArgumentException Levantada quando o
     * parmetro <code>node</code> for <code>null</code>.
     * @exception java.lang.IllegalArgumentException Levantada quando um n
     * apresenta uma estrutura interna invlida.
     */
    public Object visit(CastExpression node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException {
        if (node == null) {
            throw new IllegalArgumentException();
        }

        CastExpression resultNode = new CastExpression();

        // clona o tipo
        if (node.getType() == null
                || !(this.state.top() instanceof JType)) {
            throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
        }
        JType type = (JType) this.state.pop();

        // clona o operando
        if (node.getOperand() == null
                || !(this.state.top() instanceof JExpression)) {
            throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
        }
        JExpression operand = (JExpression) this.state.pop();

        // seta o n resultado
        resultNode.setType(type);
        resultNode.setOperand(operand);

        this.doCommonCloningWork(node, resultNode);
        return resultNode;
    }

    /**
     * Mtodos visit para objetos do tipo <code>TypeBody</code>.
     *
     * @param node O n a ser clonado.
     * @param data Um objeto que pode conter informao adicional necessria ao<br>
     * visitor.
     *
     * @return Um objeto qualquer.
     *
     * @exception java.lang.IllegalArgumentException Levantada quando o
     * parmetro <code>node</code> for <code>null</code>.
     * @exception java.lang.IllegalArgumentException Levantada quando um n
     * apresenta uma estrutura interna invlida.
     */
    public Object visit(TypeBody node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException {
        if (node == null) {

            throw new IllegalArgumentException("Corpo da classe nulo");

        }
        TypeBody resultNode = new TypeBody();
        BodyFieldDeclarations fields = new BodyFieldDeclarations();
        ClassBodyConstructorDeclarations constructors =
                new ClassBodyConstructorDeclarations();
        OtherDeclarationsSet idecs = new OtherDeclarationsSet();
        BodyMethodDeclarations methods = new BodyMethodDeclarations();
        ClassBodyInitializers inits = new ClassBodyInitializers();
        BodyTypeDeclarations types = new BodyTypeDeclarations();

        if (node.getFieldDeclarationSet() == null ||
                node.getOtherDeclarationsSet() == null ||
                node.getMethodDeclarationSet() == null ||
                node.getConstructorDeclarationSet() == null) {
            throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
        }

        // Conjunto de declaraes de construtores
        if (!(this.state.top() instanceof ClassBodyConstructorDeclarations)) {
            throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
        }
        constructors = (ClassBodyConstructorDeclarations) this.state.pop();
        resultNode.setConstructorDeclarationSet(constructors);


        // Conjunto de declaraes de mtodos
        if (!(this.state.top() instanceof BodyMethodDeclarations)) {
            throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
        }

        methods = (BodyMethodDeclarations) this.state.pop();
        resultNode.setMethodDeclarationSet(methods);

        // Conjunto de outras declaraes.
        if (!(this.state.top() instanceof OtherDeclarationsSet)) {
            System.out.println("this.state.top() = " + this.state.top().getClass().getName());
            throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
        }
        idecs = (OtherDeclarationsSet) this.state.pop();
        resultNode.setOtherDeclarationsSet(idecs);

        // Conjunto de declaraes de atributo
        if (!(this.state.top() instanceof BodyFieldDeclarations)) {
            throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
        }
        fields = (BodyFieldDeclarations) this.state.pop();
        resultNode.setFieldDeclarationSet(fields);

        // Conjunto de inicializadores
        if (!(this.state.top() instanceof ClassBodyInitializers)) {
            throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
        }
        inits = (ClassBodyInitializers) this.state.pop();
        resultNode.setInitializerSet(inits);

        // Conjunto de tipos aninhados
        if (!(this.state.top() instanceof BodyTypeDeclarations)) {
            throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
        }
        types = (BodyTypeDeclarations) this.state.pop();
        resultNode.setTypeDeclarationSet(types);


        this.doCommonCloningWork(node, resultNode);
        return resultNode;
    }


    /**
     * Mtodos visit para objetos do tipo <code>JCatchClause</code>.
     *
     * @param node O n a ser clonado.
     * @param data Um objeto que pode conter informao adicional necessria ao
     * visitor.
     *
     * @return Um objeto qualquer.
     *
     * @exception java.lang.IllegalArgumentException Levantada quando o
     * parmetro <code>node</code> for <code>null</code>.
     * @exception java.lang.IllegalArgumentException Levantada quando um n
     * apresenta uma estrutura interna invlida.
     */
    public Object visit(JCatchClause node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {

        if (node == null) {
            throw new IllegalArgumentException();
        }

        JCatchClause resultNode = new JCatchClause();
        if (node.isASourceNode()) {

            // clona o comentrio
            if (node.getComment() != null) {
                resultNode.setComment(node.getComment());
            }
            // clona o parametro
            if (node.getParameter() != null) {
                if (!(this.state.top() instanceof JFormalParameter)) {
                    throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
                }
                resultNode.setParameter((JFormalParameter) this.state.pop());
            }

            // clona o bloco
            if (node.getBlock() != null) {
                if (!(this.state.top() instanceof JBlock)) {
                    throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
                }
                resultNode.setBlock((JBlock) this.state.pop());
            }
        }
        this.doCommonCloningWork(node, resultNode);
        return resultNode;
    }

    /**
     * Mtodos visit para objetos do tipo <code>JClassDeclaration</code>.
     *
     * @param node O n a ser clonado.
     * @param data Um objeto que pode conter informao adicional necessria ao<br>
     * visitor.
     *
     * @return Um objeto qualquer.
     *
     * @exception java.lang.IllegalArgumentException Levantada quando o
     * parmetro <code>node</code> for <code>null</code>.
     * @exception java.lang.IllegalArgumentException Levantada quando um n
     * apresenta uma estrutura interna invlida.
     */
    public Object visit(JClassDeclaration node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException {

        if (node == null) {
            throw new IllegalArgumentException("Declaracao de classe nula");
        }

        JClassDeclaration resultNode = new JClassDeclaration();

        if (node.isASourceNode()) {
            if (node.getModifiers() == null ||
                    !(this.state.top() instanceof JModifierList)) {
                throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
            }

            if (node.hasComment()) {
                resultNode.setComment(node.getComment());
            }


            JModifierList mods = (JModifierList) this.state.pop();
            resultNode.setModifiers(mods);
            JIdentifier name = new JIdentifier();

            if ((node.getName() == null) || !(this.state.top() instanceof JIdentifier)) {
                throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
            }

            name = (JIdentifier) this.state.pop();

            resultNode.setName(name);

            if (node.getSuperClass() != null && (this.state.top() instanceof JName)) {

                JName superClass = new JName();
                superClass = (JName) this.state.pop();
                resultNode.setSuperClass(superClass);
            }

            if (node.getInterfaces() != null) {

                if (!(this.state.top() instanceof JNameList)) {
                    throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
                }

                JNameList interfaces = (JNameList) this.state.pop();
                resultNode.setInterfaces(interfaces);
            }

            if (node.getBody() == null ||
                    !(this.state.top() instanceof TypeBody)) {
                throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
            }

            TypeBody classBody = (TypeBody) this.state.pop();
            resultNode.setBody(classBody);
        }

        this.doCommonCloningWork(node, resultNode);

        return resultNode;
    }

    /**
     * Mtodos visit para objetos do tipo <code>IType</code>.
     *
     * @param node O n a ser clonado.
     * @param data Um objeto que pode conter informao adicional necessria ao<br>
     * visitor.
     *
     * @return Um objeto qualquer.
     *
     * @exception java.lang.IllegalArgumentException Levantada quando o
     * parmetro <code>node</code> for <code>null</code>.
     * @exception java.lang.IllegalArgumentException Levantada quando um n
     * apresenta uma estrutura interna invlida.
     */
    public Object visit(JCompilationUnit node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException
	{

        if (node == null) {
            throw new IllegalArgumentException("IType nulo");
        }

        JCompilationUnit resultNode = new JCompilationUnit();

        if ((node.getTypeDeclarationList()) == null) {
            throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
        }

        // Pr-condio
        if (node.getPrecondition() != null) {
            INode tempPre = this.state.pop();
            if (!(tempPre instanceof JPreconditionDeclaration)) {
                throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
            }
            JPreconditionDeclaration pre = (JPreconditionDeclaration) tempPre;
            resultNode.setPrecondition(pre);
        }

        // Pacote
        if (node.getPackage() != null) {
            INode tempPack = this.state.pop();
            if (!(tempPack instanceof JPackageDeclaration)) {
                throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
            }
            JPackageDeclaration packageName = (JPackageDeclaration) tempPack;
            resultNode.setPackage(packageName);
        }
        // Imports
        if (node.getImports() != null) {
            INode tempImport = this.state.pop();
            if (!(tempImport instanceof ImportDeclarations)) {
                throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
            }
            ImportDeclarations imports = (ImportDeclarations) tempImport;
            resultNode.setImports(imports);
        }

        // Declaraes de tipo
        NodeList types = new NodeList();
        JTypeDeclaration t;
        int i = 0;
        int size = node.getTypeDeclarationList().size();
        while (i < size) {
            INode tmp2 = this.state.top();
            if (tmp2 instanceof JTypeDeclaration) {
                t = (JTypeDeclaration) this.state.pop();
                types.addElement(t);
            }
            i++;
        }

        resultNode.setTypeDeclarationList(types);

        this.doCommonCloningWork(node, resultNode);

        return resultNode;
    }

    /**
     * Mtodos visit para objetos do tipo <code>JConditionalDeclaration</code>.*
     * @param node O n a ser clonado.
     * @param data Um objeto que pode conter informao adicional necessria ao<br>
     * visitor.
     *
     * @return Um objeto qualquer.
     *
     * @exception java.lang.IllegalArgumentException Levantada quando o
     * parmetro <code>node</code> for <code>null</code>.
     * @exception java.lang.IllegalArgumentException Levantada quando um n
     * apresenta uma estrutura interna invlida.
     */
    public Object visit(JConditionalDeclaration node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException {

        if (node == null) {
            throw new IllegalArgumentException();
        }

        JConditionalDeclaration resultNode = new JConditionalDeclaration();
        this.cloneConditionalDeclaration(node, resultNode);

        return resultNode;
    }


    /**
     * Mtodos visit para objetos do tipo <code>JConstructorDeclaration</code>.
     * @param node O n a ser clonado.
     * @param data Um objeto que pode conter informao adicional necessria ao
     *        visitor.
     * @return Um objeto qualquer.
     * @exception java.lang.IllegalArgumentException Levantada quando o
     *            parmetro <code>node</code> for <code>null</code>.
     * @exception java.lang.IllegalArgumentException Levantada quando um n
     *            apresenta uma estrutura interna invlida.
     */
    public Object visit(JConstructorDeclaration node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException {

        if (node == null) {
            throw new IllegalArgumentException("Declaracao de construtor nula");
        }

        JConstructorDeclaration resultNode = new JConstructorDeclaration();
        JModifierList modifier = new JModifierList(JModifierList.CONSTRUCTOR_DECLARATION);
        JIdentifier id = new JIdentifier();
        JParameterList params = new JParameterList();
        JMethodInvocation invocation = null;
//        JExpressionList args = null;
        JNameList excs = null;
        JBlock body = null;

        if (node.hasComment()) {
            resultNode.setComment(node.getComment());
        }

        if (node.isASourceNode()) {
            if (node.getName() == null) {
                throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
            }

            // clona o modificador
            if (!(this.state.top() instanceof JModifierList)) {
                throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
            }
            modifier = (JModifierList) this.state.pop();

            resultNode.setModifiers(modifier);

            // clona o identificador
            if (!(this.state.top() instanceof JIdentifier)) {
                throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
            }
            id = (JIdentifier) this.state.pop();
            resultNode.setName(id);

            // clona os parametros do construtor
            if (!(this.state.top() instanceof JParameterList)) {
                throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
            }
            params = (JParameterList) this.state.pop();
            resultNode.setParameters(params);

            // clona a chamada ao super-construtor
            if (node.getConstructorInvocation() != null) {
            	if (!(this.state.top() instanceof JMethodInvocation)) {
            		throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
            	}
            	
            	invocation = (JMethodInvocation) this.state.pop();
            	resultNode.setConstructorInvocation(invocation);
            }
            
            
            // clona a lista de excecoes
            if (!(this.state.top() instanceof JNameList)) {
                throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
            }
            excs = (JNameList) this.state.pop();
            resultNode.setExceptions(excs);

            // clona o corpo od construtor
            if (!(this.state.top() instanceof JBlock)) {
                throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
            }
            body = (JBlock) this.state.pop();
            resultNode.setBody(body);
        }

        // nivel de indentacao
//        if (node.getIndentLevel() > 0) {
//            resultNode.setIndentLevel(node.getIndentLevel());
//        } else {
//            resultNode.setIndentLevel(1);
//        }
        this.doCommonCloningWork(node, resultNode);
        return resultNode;
    }


    /**
     * Mtodos visit para objetos do tipo <code>JConstructorDeclarationSet</code>.
     * @param node O n a ser clonado.
     * @param data Um objeto que pode conter informao adicional necessria ao<br>
     *        visitor.
     * @return Um objeto qualquer.
     * @exception java.lang.IllegalArgumentException Levantada quando o
     *            parmetro <code>node</code> for <code>null</code>.
     * @exception java.lang.IllegalArgumentException Levantada quando um n
     *            apresenta uma estrutura interna invlida.
     */
    public Object visit(JConstructorDeclarationSet node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException {

        if (node == null) {
            throw new IllegalArgumentException();
        }

        JConstructorDeclarationSet resultNode = new JConstructorDeclarationSet();
        JConstructorDeclaration tmp = new JConstructorDeclaration();
        int i = 0;
        int size = node.size();

        while (i < size) {
            if (this.state.top() instanceof JConstructorDeclaration) {
                tmp = (JConstructorDeclaration) this.state.pop();
                resultNode.add(tmp);
            }
            i++;
        }
        this.doCommonCloningWork(node, resultNode);
        return resultNode;
    }

    /**
     * Mtodos visit para objetos do tipo
     * <code>ClassBodyConstructorDeclarations</code>.
     * @param node O n a ser impresso.
     * @param data Um objeto que pode conter informao adicional necessria
     * ao visitor.
     *
     * @return Um objeto qualquer.
     *
     * @exception java.lang.IllegalArgumentException Levantada quando o
     * parmetro <code>node</code> for <code>null</code>.
     * @exception java.lang.IllegalArgumentException Levantada quando um n
     * apresenta uma estrutura interna invlida.
     */
    public Object visit(ClassBodyConstructorDeclarations node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException {

        if (node == null) {
            throw new IllegalArgumentException("JConstructorDeclaration nula");
        }

        ClassBodyConstructorDeclarations resultNode =
                new ClassBodyConstructorDeclarations();

        // Clonagem das declaraes de construtor
        if (node.getConstructorsSet() == null) {
            throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
        }

        JConstructorDeclarationSet constructorsSet =
                new JConstructorDeclarationSet();

        if (!(this.state.top() instanceof JConstructorDeclarationSet)) {
            throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
        }
        constructorsSet = (JConstructorDeclarationSet) this.state.pop();

        resultNode.setConstructorsSet(constructorsSet);

        // Clonagem das declaraes do tipo ConstructorVarDeclaration
        if (node.getConstructorVarSet() == null) {
            throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
        }

        NodeList constructorVarSet = new NodeList();
        int numConstructorVars = node.getConstructorVarSet().size();
        int j = 0;

        while (j < numConstructorVars) {
            if (!(this.state.top() instanceof JConstructorDeclaration)) {
                throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
            }
            JConstructorDeclaration tempConstructorVar =
                    (JConstructorDeclaration) this.state.pop();
            constructorVarSet.addElement(tempConstructorVar);
            j++;
        }

        resultNode.setConstructorVarSet(constructorVarSet);

        // Clonagem das declaraes do tipo ConstructorsVarDeclaration
        if (node.getConstructorsVarSet() == null) {
            throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
        }

        NodeList constructorsVarSet = new NodeList();
        int numConstructorsVars = node.getConstructorsVarSet().size();
        int k = 0;

        while (k < numConstructorsVars) {
            if (!(this.state.top() instanceof JConstructorDeclarationSet)) {
                throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
            }
            JConstructorDeclarationSet tempConstructorsVar =
                    (JConstructorDeclarationSet) this.state.pop();
            constructorsVarSet.addElement(tempConstructorsVar);
            k++;
        }

        resultNode.setConstructorsVarSet(constructorsVarSet);

        this.doCommonCloningWork(node, resultNode);
        return resultNode;
    }


    /**
     * Mtodos visit para objetos do tipo <code>JExecutableDeclaration</code>.
     *
     * @param node O n a ser clonado.
     * @param data Um objeto que pode conter informao adicional necessria
     * ao visitor.
     *
     * @return Um objeto qualquer.
     *
     * @exception java.lang.IllegalArgumentException Levantada quando o
     * parmetro <code>node</code> for <code>null</code>.
     * @exception java.lang.IllegalArgumentException Levantada quando um n
     * apresenta uma estrutura interna invlida.
     */
    public Object visit(JExecutableDeclaration node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException {

        if (node == null) {
            throw new IllegalArgumentException();
        }

        JExecutableDeclaration resultNode = null;
        NodeList nodes = new NodeList();
        INode resNode = null;

        int i = 0;
        int max = node.size();

        while (i < max) {
            if (!(this.state.top() instanceof JExpression)) {
                throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
            }
            JExpression exp = (JExpression) this.state.pop();
            nodes.addElement(exp);
            i++;
        }

        if (node.getResultNode() != null) {
            resNode = (INode) this.state.pop();
        }

        // Dois tipos de declarao executvel so possveis.
        // Information-Extracting Declarations ...
        if (resNode == null && nodes.size() == 1) {
            resultNode = new JExecutableDeclaration((JExpression) nodes.elementAt(0));
        }
        // ... e Information-Modifying Declarations.
        else if (resNode != null && node.getReferencedVariable() != null) {
            resultNode = new JExecutableDeclaration(node.getReferencedVariable(),
                    resNode, nodes);
        } else {
            throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
        }

        this.doCommonCloningWork(node, resultNode);

        return resultNode;
    }

    /**
     * Revisado por Adeline, em 01/07//2004
     *
     * Mtodos visit para objetos do tipo <code>JExpressionList</code>.
     * @param node O n a ser clonado.
     * @param data Um objeto que pode conter informao adicional necessria ao
     * visitor.
     *
     * @return Um objeto qualquer.
     *
     * @exception java.lang.IllegalArgumentException Levantada quando o
     * parmetro <code>node</code> for <code>null</code>.
     * @exception java.lang.IllegalArgumentException Levantada quando um n
     * apresenta uma estrutura interna invlida.
     */
    public Object visit(JExpressionList node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException {
        if (node == null) {
            throw new IllegalArgumentException();
        }
        JExpressionList resultNode = new JExpressionList();
        if (node.isASourceNode()) {
            int max = node.size();
            for (int i = 0; i < max; i++) {
                INode tempNode = this.state.pop();
                if (!(tempNode instanceof JExpression)) {
                    throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
                }
                resultNode.addExpression((JExpression) tempNode);
            }

        }
        resultNode.setType(node.getType());
        this.doCommonCloningWork(node, resultNode);

        return resultNode;
    }

    /**
     * Mtodos visit para objetos do tipo <code>JFieldAccess</code>.
     *
     * @param node O n a ser clonado.
     * @param data Um objeto que pode conter informao adicional necessria ao
     * visitor.
     *
     * @return Um objeto qualquer.
     *
     * @exception java.lang.IllegalArgumentException Levantada quando o
     * parmetro <code>node</code> for <code>null</code>.
     * @exception java.lang.IllegalArgumentException Levantada quando um n
     * apresenta uma estrutura interna invlida.
     */
    public Object visit(JFieldAccess node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException {
        if (node == null) {
            throw new IllegalArgumentException();
        }

        JFieldAccess resultNode = new JFieldAccess();

        if (node.isASourceNode()) {

            // clona o comentrio
            if (node.getComment() != null) {
                resultNode.setComment(node.getComment());
            }
            INode fieldName = null;
            if (node.getFieldName() == null) {
                throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
            }
            fieldName = this.state.pop();
            if (!(fieldName instanceof JIdentifier))
                throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);

            resultNode.setFieldName((JIdentifier) fieldName);

            INode refObj = null;
            if (node.getReferencedObject() != null) {
                refObj = this.state.pop();
                resultNode.setReferencedObject(refObj);
            }
        }

        this.doCommonCloningWork(node, resultNode);
        return resultNode;
    }

    /**
     * Mtodos visit para objetos do tipo <code>JFieldDeclaration</code>.
     *
     * @param node O n a ser impresso.
     * @param data Um objeto que pode conter informao adicional necessria ao<br>
     * visitor.
     *
     * @return Um objeto qualquer.
     *
     * @exception java.lang.IllegalArgumentException Levantada quando o
     * parmetro <code>node</code> for <code>null</code>.
     * @exception java.lang.IllegalArgumentException Levantada quando um n
     * apresenta uma estrutura interna invlida.
     */
    public Object visit(JFieldDeclaration node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException {

        if (node == null) {
            throw new IllegalArgumentException("JFieldDeclaration nulo");
        }

        JFieldDeclaration resultNode = new JFieldDeclaration();

        if (node.hasComment()) {
            resultNode.setComment(node.getComment());
        }

        // Se for executvel, a declarao de atributo no tem modificadores,
        // tipo ou declaradores.
        if (node.isASourceNode()) {
            // Clonagem dos modificadores
            if (node.getModifiers() == null ||
                    !(this.state.top() instanceof JModifierList)) {
                throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
            }
            resultNode.setModifiers((JModifierList) this.state.pop());

            // Clonagem do tipo do atributo
            if (node.getType() == null ||
                    !(this.state.top() instanceof JType)) {
                throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
            }
            JType type = (JType) this.state.pop();
            resultNode.setType(type);

            // Clonagem dos nomes dos atributos e suas respectivas inicializaes
            if (node.getVariables() == null) {
                throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
            }

            JVariableDeclarator temp = new JVariableDeclarator();
            JVariableDeclaratorSet varDec = new JVariableDeclaratorSet();
            int vd = node.size();
            int i = 0;

            while (i < vd) {
                if (!(this.state.top() instanceof JVariableDeclarator)) {
                    throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
                }
                temp = (JVariableDeclarator) this.state.pop();
                varDec.add(temp);

                i++;
            }

            resultNode.setVariables(varDec);
        }

        this.doCommonCloningWork(node, resultNode);

        return resultNode;
    }

    /**
     * Mtodos visit para objetos do tipo <code>JFieldDeclarationSet</code>.
     *
     * @param node O n a ser clonado.
     * @param data Um objeto que pode conter informao adicional necessria ao<br>
     * visitor.
     *
     * @return Um objeto qualquer.
     *
     * @exception java.lang.IllegalArgumentException Levantada quando o
     * parmetro <code>node</code> for <code>null</code>.
     * @exception java.lang.IllegalArgumentException Levantada quando um n
     * apresenta uma estrutura interna invlida.
     */
    public Object visit(JFieldDeclarationSet node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException {

        if (node == null) {
            throw new IllegalArgumentException();
        }

        JFieldDeclarationSet resultNode = new JFieldDeclarationSet();
        JVariableDeclaration tmp = new JFieldDeclaration();
        int i = 0;
        int size = node.size();

        while (i < size) {
            if (this.state.top() instanceof JVariableDeclaration) {
                tmp = (JVariableDeclaration) this.state.pop();
                resultNode.add(tmp);
            }
            i++;
        }

        this.doCommonCloningWork(node, resultNode);

        return resultNode;
    }

    /**
     * Mtodos visit para objetos do tipo <code>JForStatement</code>.
     *
     * @param node O n a ser clonado.
     * @param data Um objeto que pode conter informao adicional necessria ao<br>
     * visitor.
     *
     * @return Um objeto qualquer.
     *
     * @exception java.lang.IllegalArgumentException Levantada quando o
     * parmetro <code>node</code> for <code>null</code>.
     * @exception java.lang.IllegalArgumentException Levantada quando um n
     * apresenta uma estrutura interna invlida.
     */
    public Object visit(JForStatement node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {

        JForStatement resultNode = new JForStatement();
        if (node == null) {
            throw new IllegalArgumentException();
        }

        if (node.isASourceNode()) {

            // todos os filhos de um for podem ser nulos, por isso  preciso checar
            // se cada um deles  diferente de null, antes de dar um pop
            // inicializao
            if (node.getForInit() != null) {
                if (!(this.state.top() != null)) {
                    throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
                }
                resultNode.setForInit(this.state.pop());
            }

            // condio de parada
            if (node.getExpression() != null) {
                if (!(this.state.top() instanceof JExpression)) {
                    throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
                }
                resultNode.setExpression((JExpression) this.state.pop());
            }

            // atualizao
            if (node.getForUpdate() != null) {
                if (!(this.state.top() instanceof JExpressionList)) {
                    throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
                }
                resultNode.setForUpdate((JExpressionList) this.state.pop());
            }
            // clona o statement
            if (node.getStatement() != null) {
                if (!(this.state.top() != null)) {
                    throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
                }
                resultNode.setStatement(this.state.pop());
            }
        }

        this.doCommonCloningWork(node, resultNode);
        return resultNode;
    }

    /**
     * Mtodos visit para objetos do tipo <code>BodyFieldDeclarations</code>.
     *
     * @param node O n a ser impresso.
     * @param data Um objeto que pode conter informao adicional necessria
     * ao visitor.
     *
     * @return Um objeto qualquer.
     *
     * @exception java.lang.IllegalArgumentException Levantada quando o
     * parmetro <code>node</code> for <code>null</code>.
     * @exception java.lang.IllegalArgumentException Levantada quando um n
     * apresenta uma estrutura interna invlida.
     */
    public Object visit(BodyFieldDeclarations node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException {

        if (node == null) {
            throw new IllegalArgumentException("JFieldDeclaration nulo");
        }

        BodyFieldDeclarations resultNode = new BodyFieldDeclarations();

        // Clonagem das declaraes de atributo
        if (node.getFieldsSet() == null) {
            throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
        }

        JFieldDeclarationSet fieldsSet = new JFieldDeclarationSet();
        if (!(this.state.top() instanceof JFieldDeclarationSet)) {
            throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
        }
        fieldsSet = (JFieldDeclarationSet) this.state.pop();
        resultNode.setFieldsSet(fieldsSet);

        // Clonagem das declaraes do tipo FieldVarDeclaration
        if (node.getFieldVarSet() == null) {
            throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
        }
        NodeList attributeSet = new NodeList();
        int numFieldVars = node.getFieldVarSet().size();
        int j = 0;
        while (j < numFieldVars) {
            if (!(this.state.top() instanceof JVariableDeclaration)) {
                throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
            }
            JVariableDeclaration tempFieldVar = (JVariableDeclaration) this.state.pop();
            attributeSet.addElement(tempFieldVar);
            j++;
        }
        resultNode.setFieldVarSet(attributeSet);

        // Clonagem das declaraes do tipo FieldsVarDeclaration
        if (node.getFieldsVarSet() == null) {
            throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
        }
        NodeList attributesSet = new NodeList();
        int numFieldsVars = node.getFieldsVarSet().size();
        int k = 0;
        while (k < numFieldsVars) {
            if (!(this.state.top() instanceof JFieldDeclarationSet)) {
                throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
            }
            JFieldDeclarationSet tempFieldsVar =
                    (JFieldDeclarationSet) this.state.pop();
            attributesSet.addElement(tempFieldsVar);
            k++;
        }

        resultNode.setFieldsVarSet(attributesSet);
        this.doCommonCloningWork(node, resultNode);
        return resultNode;
    }


    /**
     * Mtodos visit para objetos do tipo <code>ClassBodyInitializers</code>.
     *
     * @param node O n a ser impresso.
     * @param data Um objeto que pode conter informao adicional necessria
     * ao visitor.
     *
     * @return Um objeto qualquer.
     *
     * @exception java.lang.IllegalArgumentException Levantada quando o
     * parmetro <code>node</code> for <code>null</code>.
     * @exception java.lang.IllegalArgumentException Levantada quando um n
     * apresenta uma estrutura interna invlida.
     */
    public Object visit(ClassBodyInitializers node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException {

        if (node == null) {
            throw new IllegalArgumentException("Argumento nulo");
        }

        ClassBodyInitializers resultNode = new ClassBodyInitializers();

        // Clonagem das declaraes de atributo
        if (node.getInitializersSet() == null) {
            throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
        }

        JInitializerSet initsSet = new JInitializerSet();
        if (!(this.state.top() instanceof JInitializerSet)) {
            throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
        }
        initsSet = (JInitializerSet) this.state.pop();
        resultNode.setInitializersSet(initsSet);

        // Clonagem das declaraes do tipo InitVar
        if (node.getInitializerVarSet() == null) {
            throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
        }
        NodeList initVarSet = new NodeList();
        int numFieldVars = node.getInitializerVarSet().size();
        int j = 0;
        while (j < numFieldVars) {
            if (!(this.state.top() instanceof JInitializer)) {
                throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
            }
            JInitializer tempFieldVar = (JInitializer) this.state.pop();
            initVarSet.addElement(tempFieldVar);
            j++;
        }
        resultNode.setInitializerVarSet(initVarSet);

        // Clonagem das declaraes do tipo InitializerSet
        if (node.getInitializerVarSet() == null) {

            throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
        }
        NodeList attributesSet = new NodeList();
        int numFieldsVars = node.getInitializersVarSet().size();
        int k = 0;
        while (k < numFieldsVars) {
            if (!(this.state.top() instanceof JInitializerSet)) {
                throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
            }
            JInitializerSet tempFieldsVar =
                    (JInitializerSet) this.state.pop();
            attributesSet.addElement(tempFieldsVar);
            k++;
        }

        resultNode.setInitializersVarSet(attributesSet);
        this.doCommonCloningWork(node, resultNode);
        return resultNode;
    }


    /**
     * Mtodos visit para objetos do tipo <code>JFormalParameter</code>.
     *
     * @param node O n a ser clonado.
     * @param data Um objeto que pode conter informao adicional necessria ao
     * visitor.
     *
     * @return Um objeto qualquer.
     *
     * @exception java.lang.IllegalArgumentException Levantada quando o
     * parmetro <code>node</code> for <code>null</code>.
     * @exception java.lang.IllegalArgumentException Levantada quando um n
     * apresenta uma estrutura interna invlida.
     */
    public Object visit(JFormalParameter node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException {
        if (node == null) {
            throw new IllegalArgumentException("Parametro formal nulo");
        }

        JFormalParameter resultNode = new JFormalParameter();
        JModifierList mod =
                new JModifierList(JModifierList.PARAMETER_DECLARATION);
        JIdentifier id = null;
        JType t = null;

        if (node.isASourceNode()) {

            // clona o modificador
            if (node.getModifiers() != null) {
                if (!(this.state.top() instanceof JModifierList)) {
                    throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
                }
                mod = (JModifierList) this.state.pop();
            }
            resultNode.setModifiers(mod);

            // clona o tipo
            if (!(this.state.top() instanceof JType)) {
                throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
            }
            t = (JType) this.state.pop();
            resultNode.setType(t);

            // clona o identificador
            if ( node.getIdentifier() != null &&
                    !(this.state.top() instanceof JIdentifier)) {
                throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
            }
            id = (JIdentifier) this.state.pop();
            resultNode.setIdentifier(id);
        }

        this.doCommonCloningWork(node, resultNode);

        return resultNode;
    }

    /**
     * Mtodos visit para objetos do tipo <code>JIdentifier</code>.
     *
     * @param node O n a ser clonado.
     * @param data Um objeto que pode conter informao adicional necessria ao
     * visitor.
     *
     * @return Um objeto qualquer.
     *
     * @exception java.lang.IllegalArgumentException Levantada quando o
     * parmetro <code>node</code> for <code>null</code>.
     * @exception java.lang.IllegalArgumentException Levantada quando um n
     * apresenta uma estrutura interna invlida.
     */
    public Object visit(JIdentifier node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException {
        if (node == null) {
            throw new IllegalArgumentException();
        }

        JIdentifier resultNode = new JIdentifier();

        if (!node.isExecutable()) {
            if (node.getValue() == null) {
                throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
            }
            resultNode.setValue(node.getValue());
        }

        this.doCommonCloningWork(node, resultNode);

        return resultNode;
    }

    /**
     * Mtodos visit para objetos do tipo <code>JIfStatement</code>.
     *
     * @param node O n a ser clonado.
     * @param data Um objeto que pode conter informao adicional necessria ao
     * visitor.
     *
     * @return Um objeto qualquer.
     *
     * @exception java.lang.IllegalArgumentException Levantada quando o
     * parmetro <code>node</code> for <code>null</code>.
     * @exception java.lang.IllegalArgumentException Levantada quando um n
     * apresenta uma estrutura interna invlida.
     */
    public Object visit(JIfStatement node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {
        if (node == null) {
            throw new IllegalArgumentException("JIfStatement nulo");
        }

        JIfStatement resultNode = new JIfStatement();

        if (node.isASourceNode()) {

            // clona a expresso
            if (node.getExpression() != null) {
                if (!(this.state.top() instanceof JExpression)) {
                    throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
                }
                resultNode.setExpression((JExpression) this.state.pop());
            }

            // clona o comentrio do if
            if (node.getIfComment() != null) {
                resultNode.setIfComment(node.getIfComment());

            }
            // clona o statement qd a condio do if  true
            if (node.getIfStatement() != null) {
                if ((this.state.top() instanceof JStatement)) {
                    resultNode.setIfStatement(this.state.pop());
                }
            }

            // clona o comentrio do else
            if (node.getElseComment() != null) {
                resultNode.setElseComment(node.getElseComment());

            }
            // clona o statement qd a condio do if  false
            if (node.getElseStatement() != null) {
                if ((this.state.top() instanceof JStatement)) {
                    resultNode.setElseStatement(this.state.pop());
                }
            }
        }

        this.doCommonCloningWork(node, resultNode);
        return resultNode;
    }

    /**
     * Mtodos visit para objetos do tipo <code>JImportDeclaration</code>.
     *
     * @param node O n a ser clonado.
     * @param data Um objeto que pode conter informao adicional necessria ao
     * visitor.
     *
     * @return Um objeto qualquer.
     *
     * @exception java.lang.IllegalArgumentException Levantada quando o
     * parmetro <code>node</code> for <code>null</code>.
     * @exception java.lang.IllegalArgumentException Levantada quando um n
     * apresenta uma estrutura interna invlida.
     */
    public Object visit(JImportDeclaration node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException {
        if (node == null) {
            throw new IllegalArgumentException();
        }

        JImportDeclaration resultNode = new JImportDeclaration();

        if (node.isASourceNode()) {
            if (node.getPackageName() == null) {
                throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
            }
            if (!(this.state.top() instanceof JName)) {
                throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
            }
            JName packageName = (JName) this.state.pop();
            resultNode.setPackageName(packageName);

            if (node.getClassName() != null) {
                if (!(this.state.top() instanceof JIdentifier)) {
                    throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
                }
                JIdentifier className = (JIdentifier) this.state.pop();
                resultNode.setClassName(className);
            }
            resultNode.setImportAllClasses(node.importAllClasses());
        }

        this.doCommonCloningWork(node, resultNode);

        return resultNode;
    }

    /**
     * Mtodos visit para objetos do tipo <code>JImportDeclarationSet</code>.
     * @param node O n a ser clonado.
     * @param data Um objeto que pode conter informao adicional necessria
     * ao visitor.
     * @return Um objeto qualquer.
     * @exception java.lang.IllegalArgumentException Levantada quando o
     *            parmetro <code>node</code> for <code>null</code>.
     * @exception java.lang.IllegalArgumentException Levantada quando um n
     *            apresenta uma estrutura interna invlida.
     */
    public Object visit(JImportDeclarationSet node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException {

        if (node == null) {
            throw new IllegalArgumentException();
        }

        JImportDeclarationSet resultNode = new JImportDeclarationSet();
        JImportDeclaration tmp = new JImportDeclaration();
        int i = 0;
        int size = node.size();

        while (i < size) {
            if (this.state.top() instanceof JImportDeclaration) {
                tmp = (JImportDeclaration) this.state.pop();
                resultNode.add(tmp);
            }
            i++;
        }

        this.doCommonCloningWork(node, resultNode);
        return resultNode;
    }

    /**
     * Mtodos visit para objetos do tipo <code>JInitializer</code>.
     *
     * @param node O n a ser clonado.
     * @param data Um objeto que pode conter informao adicional necessria ao
     * visitor.
     *
     * @return Um objeto qualquer.
     *
     * @exception java.lang.IllegalArgumentException Levantada quando o
     * parmetro <code>node</code> for <code>null</code>.
     * @exception java.lang.IllegalArgumentException Levantada quando um n
     * apresenta uma estrutura interna invlida.
     */
    public Object visit(JInitializer node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {

        if (node == null) {
            throw new IllegalArgumentException();
        }

        JInitializer resultNode = new JInitializer();
        if (node.isASourceNode()) {
            resultNode.setStatic(node.isStatic());

            // clona o bloco
            if (node.getBlock() != null) {
                if (!(this.state.top() instanceof JBlock)) {
                    throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
                }
                resultNode.setBlock((JBlock) this.state.pop());
            }
        }
        this.doCommonCloningWork(node, resultNode);
        return resultNode;
    }

    /**
     * Mtodos visit para objetos do tipo <code>JInitializerSet</code>.
     *
     * @param node O n a ser clonado.
     * @param data Um objeto que pode conter informao adicional necessria ao
     * visitor.
     *
     * @return Um objeto qualquer.
     *
     * @exception java.lang.IllegalArgumentException Levantada quando o
     * parmetro <code>node</code> for <code>null</code>.
     * @exception java.lang.IllegalArgumentException Levantada quando um n
     * apresenta uma estrutura interna invlida.
     */
    public Object visit(JInitializerSet node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {

        if (node == null) {
            throw new IllegalArgumentException();
        }
        JInitializerSet resultNode = new JInitializerSet();
        if (node.getDeclarationSet() != null) {
            int size = node.getDeclarationSet().size();
            for (int i = 0; i < size; i++) {
                if (!(this.state.top() instanceof JInitializer)) {
                    throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
                }
                resultNode.add(this.state.pop());
            }
        }
        this.doCommonCloningWork(node, resultNode);
        return resultNode;
    }

    /**
     * Mtodos visit para objetos do tipo <code>JInterfaceDeclaration</code>.
     *
     * @param node O n a ser clonado.
     * @param data Um objeto que pode conter informao adicional necessria
     * ao visitor.
     *
     * @return Um objeto qualquer.
     *
     * @exception java.lang.IllegalArgumentException Levantada quando o
     * parmetro <code>node</code> for <code>null</code>.
     * @exception java.lang.IllegalArgumentException Levantada quando um n
     * apresenta uma estrutura interna invlida.
     */
    public Object visit(JInterfaceDeclaration node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException {

        // ordem de escrita no accept:
        // corpo
        // interfaces
        // name
        // modifiers
        if (node == null) {
            throw new IllegalArgumentException("Declaracao de classe nula");
        }
        JInterfaceDeclaration resultNode = new JInterfaceDeclaration();
        if (node.getModifiers() == null ||
                !(this.state.top() instanceof JModifierList)) {
            throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
        }
        if (node.hasComment()) {
            resultNode.setComment(node.getComment());
        }

        JModifierList mods = (JModifierList) this.state.pop();
        resultNode.setModifiers(mods);
        JIdentifier name = new JIdentifier();

        if ((node.getName() == null) || !(this.state.top() instanceof JIdentifier)) {
            throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
        }
        name = (JIdentifier) this.state.pop();
        resultNode.setName(name);
        if (node.getInterfaces() != null) {
            if (!(this.state.top() instanceof JNameList)) {
                throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
            }
            JNameList interfaces = (JNameList) this.state.pop();
            resultNode.setInterfaces(interfaces);
        }
        if (node.getBody() != null) {
            if (!(this.state.top() instanceof TypeBody)) {
                throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
            }
            TypeBody body = (TypeBody) this.state.pop();
            resultNode.setBody(body);

        }

        this.doCommonCloningWork(node, resultNode);
        return resultNode;
    }

    /**
     * Mtodos visit para objetos do tipo <code>JImportDeclarationSet</code>.
     *
     * @param node O n a ser impresso.
     * @param data Um objeto que pode conter informao adicional necessria
     * ao visitor.
     *
     * @return Um objeto qualquer.
     *
     * @exception java.lang.IllegalArgumentException Levantada quando o
     * parmetro <code>node</code> for <code>null</code>.
     * @exception java.lang.IllegalArgumentException Levantada quando um n
     * apresenta uma estrutura interna invlida.
     */
    public Object visit(ImportDeclarations node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException {

        if (node == null) {
            throw new IllegalArgumentException("JImportDeclaration nula");
        }

        ImportDeclarations resultNode = new ImportDeclarations();

        // Clonagem das declaraes de importao (tipo: import java.io.File;)
        if (node.getImportsSet() == null) {
            throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
        }

        JImportDeclarationSet importsSet = new JImportDeclarationSet();

        if (!(this.state.top() instanceof JImportDeclarationSet)) {
            throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
        }
        importsSet = (JImportDeclarationSet) this.state.pop();

        resultNode.setImportsSet(importsSet);

        // Clonagem das declaraes do tipo ImportVarDeclaration
        if (node.getImportVarSet() == null) {
            throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
        }

        NodeList importVarSet = new NodeList();
        int numImportVars = node.getImportVarSet().size();
        int j = 0;

        while (j < numImportVars) {
            if (!(this.state.top() instanceof JImportDeclaration)) {
                throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
            }
            JImportDeclaration tempImportVar = (JImportDeclaration) this.state.pop();
            importVarSet.addElement(tempImportVar);
            j++;
        }

        resultNode.setImportVarSet(importVarSet);

        // Clonagem das declaraes do tipo ImportsVarDeclaration
        if (node.getImportsVarSet() == null) {
            throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
        }

        NodeList importsVarSet = new NodeList();
        int numImportsVars = node.getImportsVarSet().size();
        int k = 0;

        while (k < numImportsVars) {
            if (!(this.state.top() instanceof JImportDeclarationSet)) {
                throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
            }
            JImportDeclarationSet tempImportsVar = (JImportDeclarationSet) this.state.pop();
            importsVarSet.addElement(tempImportsVar);
            k++;
        }

        resultNode.setImportsVarSet(importsVarSet);
        this.doCommonCloningWork(node, resultNode);

        return resultNode;
    }

    /**
     * Mtodos visit para objetos do tipo <code>InstanceOfExpression</code>.
     *
     * @param node O n a ser clonado.
     * @param data Um objeto que pode conter informao adicional necessria ao
     * visitor.
     *
     * @return Um objeto qualquer.
     *
     * @exception java.lang.IllegalArgumentException Levantada quando o
     * parmetro <code>node</code> for <code>null</code>.
     * @exception java.lang.IllegalArgumentException Levantada quando um n
     * apresenta uma estrutura interna invlida.
     */
    public Object visit(InstanceOfExpression node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException {

        if (node == null) {
            throw new IllegalArgumentException();
        }

        InstanceOfExpression resultNode = new InstanceOfExpression();

        if (node.getType() == null
                || !(this.state.top() instanceof JType))
            throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
        JType type = (JType) this.state.pop();

        if (node.getOperand() == null
                || !(this.state.top() != null))
            throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
        INode operand = this.state.pop();

        resultNode.setOperand(operand);
        resultNode.setType(type);
        this.doCommonCloningWork(node, resultNode);
        return resultNode;
    }


    /**
     * Mtodos visit para objetos do tipo <code>IterativeDeclaration</code>.
     *
     * @param node O n a ser clonado.
     * @param data Um objeto que pode conter informao adicional necessria ao
     * visitor.
     *
     * @return Um objeto qualquer.
     *
     * @exception java.lang.IllegalArgumentException Levantada quando o
     * parmetro <code>node</code> for <code>null</code>.
     * @exception java.lang.IllegalArgumentException Levantada quando um n
     * apresenta uma estrutura interna invlida.
     */
    public Object visit(IterativeDeclaration node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException {

        IterativeDeclaration resultNode = new IterativeDeclaration();

        this.cloneIterativeDeclaration(node, resultNode);

        return resultNode;
    }

    /**
     * Mtodos visit para objetos do tipo <code>JLiteral</code>.
     *
     * @param node O n a ser clonado.
     * @param data Um objeto que pode conter informao adicional necessria ao
     * visitor.
     *
     * @return Um objeto qualquer.
     *
     * @exception java.lang.IllegalArgumentException Levantada quando o
     * parmetro <code>node</code> for <code>null</code>.
     * @exception java.lang.IllegalArgumentException Levantada quando um n
     * apresenta uma estrutura interna invlida.
     */
    public Object visit(JLiteral node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException {
        if (node == null) {
            throw new IllegalArgumentException();
        }

        JLiteral resultNode = new JLiteral();

        if (!node.isVariable() && node.getValue() == null) {
            throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
        } else if (node.getLiteralValue() != null) {
            resultNode.setLiteralValue((node.getValue()));
            resultNode.setLiteralType(node.getLiteralType());
        }

        this.doCommonCloningWork(node, resultNode);
        return resultNode;
    }

    public Object visit(JLocalVariableDeclaration node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {

        if (node == null) {
            throw new IllegalArgumentException("JLocalVariableDeclaration nulo");
        }

        JLocalVariableDeclaration resultNode = new JLocalVariableDeclaration();

        // Se for executvel, a declarao de atributo no tem modificadores,
        // tipo ou declaradores.
        if (node.isASourceNode()) {

            // clona o comentrio
            if (node.getComment() != null) {
                resultNode.setComment(node.getComment());
            }

            // Clonagem dos modificadores
            if (node.getModifiers() == null ||
                    !(this.state.top() instanceof JModifierList)) {
                throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
            }
            resultNode.setModifiers((JModifierList) this.state.pop());

            // Clonagem do tipo do atributo
            if (node.getType() == null ||
                    !(this.state.top() instanceof JType)) {
                throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
            }
            JType type = (JType) this.state.pop();
            resultNode.setType(type);

            // Clonagem dos nomes dos atributos e suas respectivas inicializaes
            if (node.getVariables() == null) {
                throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
            }

            JVariableDeclarator temp = new JVariableDeclarator();
            JVariableDeclaratorSet varDec = new JVariableDeclaratorSet();
            int vd = node.size();
            int i = 0;

            while (i < vd) {
                if (!(this.state.top() instanceof JVariableDeclarator)) {
                    throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
                }
                temp = (JVariableDeclarator) this.state.pop();
                varDec.add(temp);
                i++;
            }
            resultNode.setVariables(varDec);
        }

        this.doCommonCloningWork(node, resultNode);

        return resultNode;
    }

    /**
     * Mtodos visit para objetos do tipo <code>MapTable</code>.
     * @param node O n onde a substituio ocorrer.
     * @param data Um objeto que pode conter informao adicional necessria ao
     *        visitor.
     * @return Um objeto qualquer.
     * @exception java.lang.IllegalArgumentException Levantada quando o
     *            parmetro <code>node</code> for <code>null</code>.
     * @exception java.lang.IllegalArgumentException Levantada quando um n
     *            apresenta uma estrutura interna invlida.
     */
    public Object visit(MapTable node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {

        if (node == null) {
            throw new IllegalArgumentException();
        }

        MapTable resultNode = new MapTable();

        if (!node.isExecutable()) {
            if (node.getMap() == null) {
                throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
            }
            resultNode.setMap(node.getMap());
        }

        this.doCommonCloningWork(node, resultNode);

        return resultNode;
    }

    /**
     * Mtodos visit para objetos do tipo <code>JMethodDeclaration</code>.
     * @param node O n a ser clonado.
     * @param data Um objeto que pode conter informao adicional necessria ao
     *        visitor.
     * @return Um objeto qualquer.
     * @exception java.lang.IllegalArgumentException Levantada quando o
     *            parmetro <code>node</code> for <code>null</code>.
     * @exception java.lang.IllegalArgumentException Levantada quando um n
     *            apresenta uma estrutura interna invlida.
     */
    public Object visit(JMethodDeclaration node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException {

        if (node == null) {
            throw new IllegalArgumentException("Declaracao de metodo nula");
        }

        JMethodDeclaration resultNode = new JMethodDeclaration();
        JModifierList modifiers = new JModifierList(JModifierList.METHOD_DECLARATION);
        JType resType = null;
        JName name = new JName();
        JParameterList parameters = new JParameterList();
        JNameList excs = null;
        JBlock mBody = null;
        if (node.isASourceNode()) {
            if (node.getName() == null) {
                throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
            }
            if (node.getResultType() == null) {
                throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
            }
            if (node.hasComment()) {
                resultNode.setComment(node.getComment());
            }
            // clona os modificadores
            if (!(this.state.top() instanceof JModifierList)) {
                throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
            }
            modifiers = (JModifierList) this.state.pop();
            resultNode.setModifiers(modifiers);

            // clona o tipo de retorno
            if (!(this.state.top() instanceof JType)) {
                throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
            }
            resType = (JType) this.state.pop();
            resultNode.setResultType(resType);

            // clona o identificador
            if (!(this.state.top() instanceof JName)) {
                throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
            }
            name = (JName) this.state.pop();
            resultNode.setName(name);

            // clona a lista de parametros
            if (!(this.state.top() instanceof JParameterList)) {
                throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
            }
            parameters = (JParameterList) this.state.pop();
            resultNode.setParameters(parameters);

            // clona a dimensao
            resultNode.setDimensions(node.getDimensions());


            // clona a lista de excecoes
            if ((node.getExceptions() != null) && !(this.state.top() instanceof JNameList)) {
                throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
            }
            excs = (JNameList) this.state.pop();
            resultNode.setExceptions(excs);

            // clona o corpo do mtodo
            if ((node.hasBody()) && !(this.state.top() instanceof JBlock)) {
                throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
            } else if (node.hasBody()) {
                mBody = (JBlock) this.state.pop();
                resultNode.setBody(mBody);
            }
        }
        // seta o nivel de indentao do mtodo. Se por um acaso o atual for
        // zero seta para um.
//        if (node.getIndentLevel() > 0) {
//            resultNode.setIndentLevel(node.getIndentLevel());
//        } else {
//            resultNode.setIndentLevel(1);
//
//        }


        this.doCommonCloningWork(node, resultNode);
        return resultNode;
    }


    /**
     * Mtodos visit para objetos do tipo <code>JMethodDeclarationSet</code>.
     * @param node O n a ser clonado.
     * @param data Um objeto que pode conter informao adicional necessria ao<br>
     *        visitor.
     * @return Um objeto qualquer.
     * @exception java.lang.IllegalArgumentException Levantada quando o
     *            parmetro <code>node</code> for <code>null</code>.
     * @exception java.lang.IllegalArgumentException Levantada quando um n
     *            apresenta uma estrutura interna invlida.
     */
    public Object visit(JMethodDeclarationSet node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException {

        if (node == null) {
            throw new IllegalArgumentException();
        }
        JMethodDeclarationSet resultNode = new JMethodDeclarationSet();
        JMethodDeclaration tmp = new JMethodDeclaration();
        int i = 0;
        int size = node.size();
        while (i < size) {
            if (this.state.top() instanceof JMethodDeclaration) {
                tmp = (JMethodDeclaration) this.state.pop();
                resultNode.add(tmp);
            }
            i++;
        }
        this.doCommonCloningWork(node, resultNode);
        return resultNode;
    }


    /**
     * Mtodos visit para objetos do tipo <code>BodyMethodDeclarations</code>.
     *
     * @param node O n a ser impresso.
     * @param data Um objeto que pode conter informao adicional necessria
     * ao visitor.
     *
     * @return Um objeto qualquer.
     *
     * @exception java.lang.IllegalArgumentException Levantada quando o
     * parmetro <code>node</code> for <code>null</code>.
     * @exception java.lang.IllegalArgumentException Levantada quando um n
     * apresenta uma estrutura interna invlida.
     */
    public Object visit(BodyMethodDeclarations node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException
	{

        if (node == null) {
            throw new IllegalArgumentException("JMethodDeclaration nula");
        }

        BodyMethodDeclarations resultNode = new BodyMethodDeclarations();

        // Clonagem das declaraes de metodo (tipo: private #rt #a (#fp) {})
        if (node.getMethodsSet() == null) {
            throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
        }

        JMethodDeclarationSet methodsSet = new JMethodDeclarationSet();
        if (!(this.state.top() instanceof JMethodDeclarationSet)) {
            throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
        }
        methodsSet = (JMethodDeclarationSet) this.state.pop();

        resultNode.setMethodsSet(methodsSet);

        // Clonagem das declaraes do tipo MethodVarDeclaration
        if (node.getMethodVarSet() == null) {
            throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
        }

        NodeList methodVarSet = new NodeList();
        int numMethodVars = node.getMethodVarSet().size();
        int j = 0;

        while (j < numMethodVars) {
            if (!(this.state.top() instanceof JMethodDeclaration)) {
                throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
            }
            JMethodDeclaration tempMethodVar =
                    (JMethodDeclaration) this.state.pop();
            methodVarSet.addElement(tempMethodVar);
            j++;
        }

        resultNode.setMethodVarSet(methodVarSet);

        // Clonagem das declaraes do tipo MethodsVarDeclaration
        if (node.getMethodsVarSet() == null) {
            throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
        }

        NodeList methodsVarSet = new NodeList();
        int numMethodsVars = node.getMethodsVarSet().size();
        int k = 0;

        while (k < numMethodsVars) {
            if (!(this.state.top() instanceof JMethodDeclarationSet)) {
                throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
            }
            JMethodDeclarationSet tempMethodsVar = (JMethodDeclarationSet) this.state.pop();
            methodsVarSet.addElement(tempMethodsVar);
            k++;
        }

        resultNode.setMethodsVarSet(methodsVarSet);
        this.doCommonCloningWork(node, resultNode);
        return resultNode;
    }


    /**
     * Mtodos visit para objetos do tipo <code>BodyMethodDeclarations</code>.
     *
     * @param node O n a ser impresso.
     * @param data Um objeto que pode conter informao adicional necessria
     * ao visitor.
     *
     * @return Um objeto qualquer.
     *
     * @exception java.lang.IllegalArgumentException Levantada quando o
     * parmetro <code>node</code> for <code>null</code>.
     * @exception java.lang.IllegalArgumentException Levantada quando um n
     * apresenta uma estrutura interna invlida.
     */
    public Object visit(BodyTypeDeclarations node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException {

        if (node == null) {
            throw new IllegalArgumentException("BodyTypeDeclarations nula");
        }

        BodyTypeDeclarations resultNode = new BodyTypeDeclarations();

        // Clonagem das declaraes de tipos completas
        if (node.getTypesSet() == null) {
            throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
        }

        TypeDeclarationSet typesSet = new TypeDeclarationSet();
        if (!(this.state.top() instanceof TypeDeclarationSet)) {
            throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
        }
        typesSet = (TypeDeclarationSet) this.state.pop();

        resultNode.setTypesSet(typesSet);

        // Clonagem das declaraes do tipo #type:ClassDeclaration
        if (node.getTypeVarSet() == null) {
            throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
        }

        NodeList typeVarSet = new NodeList();
        int numTypeVars = node.getTypeVarSet().size();
        int j = 0;

        while (j < numTypeVars) {
            if (!(this.state.top() instanceof JTypeDeclaration)) {
                throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
            }
            JTypeDeclaration tempTypeVar =
                    (JTypeDeclaration) this.state.pop();
            typeVarSet.addElement(tempTypeVar);
            j++;
        }

        resultNode.setTypeVarSet(typeVarSet);

        // Clonagem das declaraes do tipo MethodsVarDeclaration
        if (node.getTypesVarSet() == null) {
            throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
        }

        NodeList typesVarSet = new NodeList();
        int numTypesVars = node.getTypesVarSet().size();
        int k = 0;

        while (k < numTypesVars) {
            if (!(this.state.top() instanceof TypeDeclarationSet)) {
                throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
            }
            TypeDeclarationSet tempTypesVar =
                    (TypeDeclarationSet) this.state.pop();
            typesVarSet.addElement(tempTypesVar);
            k++;
        }

        resultNode.setTypesVarSet(typesVarSet);
        this.doCommonCloningWork(node, resultNode);
        return resultNode;
    }

    /**
     * Mtodos visit para objetos do tipo <code>JMethodInvocation</code>.
     *
     * @param node O n a ser clonado.
     * @param data Um objeto que pode conter informao adicional necessria ao
     * visitor.
     *
     * @return Um objeto qualquer.
     *
     * @exception java.lang.IllegalArgumentException Levantada quando o
     * parmetro <code>node</code> for <code>null</code>.
     * @exception java.lang.IllegalArgumentException Levantada quando um n
     * apresenta uma estrutura interna invlida.
     */
    public Object visit(JMethodInvocation node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException {
        if (node == null) {
            throw new IllegalArgumentException();
        }

        JMethodInvocation resultNode = new JMethodInvocation();

        if (node.getArgumentList() == null
                || !(this.state.top() instanceof JExpressionList)) {
            throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
        }
        JExpressionList args = (JExpressionList) this.state.pop();

        INode methodName = null;
        if (node.getMethodName() == null) {
            throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
        }
        methodName = this.state.pop();

        // clona o comentrio
        if (node.getComment() != null) {
            resultNode.setComment(node.getComment());
        }

        INode refObj = null;
        if (node.getReferencedObject() != null) {
            refObj = this.state.pop();
            resultNode.setReferencedObject(refObj);
        }

        resultNode.setMethodName(JName.unwrapName(methodName));
        resultNode.setArgumentList(args);

        this.doCommonCloningWork(node, resultNode);
        return resultNode;
    }

    /**
     * Mtodos visit para objetos do tipo <code>JModifierList</code>.
     *
     * @param node O n a ser clonado.
     * @param data Um objeto que pode conter informao adicional necessria ao
     * visitor.
     *
     * @return Um objeto qualquer.
     *
     * @exception java.lang.IllegalArgumentException Levantada quando o
     * parmetro <code>node</code> for <code>null</code>.
     * @exception java.lang.IllegalArgumentException Levantada quando um n
     * apresenta uma estrutura interna invlida.
     */
    public Object visit(JModifierList node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException {
        if (node == null) {
            throw new IllegalArgumentException();
        }

        JModifierList resultNode = new JModifierList();

        resultNode.setPossibleModifiers(node.getPossibleModifiers());
        resultNode.addModifier(node.getModifiers());

        this.doCommonCloningWork(node, resultNode);

        return resultNode;
    }

    /**
     * Mtodos visit para objetos do tipo <code>JName</code>.
     *
     * @param node O n a ser clonado.
     * @param data Um objeto que pode conter informao adicional necessria ao
     * visitor.
     *
     * @return Um objeto qualquer.
     *
     * @exception java.lang.IllegalArgumentException Levantada quando o
     * parmetro <code>node</code> for <code>null</code>.
     * @exception java.lang.IllegalArgumentException Levantada quando um n
     * apresenta uma estrutura interna invlida.
     */
    public Object visit(JName node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException {
        if (node == null) {
            throw new IllegalArgumentException();
        }

        JName resultNode = new JName();

        // desempilha o que veio  e adiciona em um name temporrio
        // pois a ordem est invertida
        int i = node.size() - 1;
        NodeList names = new NodeList();

        while (i >= 0) {
            names.addElement(this.state.pop());
            i--;
        }

        // pega o name temporrio e inverte seus elementos, para que o name
        // retornado seja igual ao name original
        int j = 0, max = names.size();
        while (j < max) {
            INode tempNode = names.elementAt(max - (j + 1));
            if (!(tempNode instanceof JIdentifier))
                throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);

            resultNode.addIdentifier((JIdentifier) tempNode);
            j++;
        }

        this.doCommonCloningWork(node, resultNode);

        return resultNode;
    }

    /**
     * Mtodos visit para objetos do tipo <code>JNodeString</code>.
     *
     * @param node O n a ser clonado.
     * @param data Um objeto que pode conter informao adicional necessria ao
     * visitor.
     *
     * @return Um objeto qualquer.
     *
     * @exception java.lang.IllegalArgumentException Levantada quando o
     * parmetro <code>node</code> for <code>null</code>.
     * @exception java.lang.IllegalArgumentException Levantada quando um n
     * apresenta uma estrutura interna invlida.
     */
    public Object visit(JNodeString node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException {
        if (node == null) {
            throw new IllegalArgumentException();
        }

        JNodeString resultNode = new JNodeString(node.getNodeValue());

        this.doCommonCloningWork(node, resultNode);
        return resultNode;
    }

    /**
     * Mtodos visit para objetos do tipo <code>JNameList</code>.
     *
     * @param node O n a ser clonado.
     * @param data Um objeto que pode conter informao adicional necessria ao
     * visitor.
     *
     * @return Um objeto qualquer.
     *
     * @exception java.lang.IllegalArgumentException Levantada quando o
     * parmetro <code>node</code> for <code>null</code>.
     * @exception java.lang.IllegalArgumentException Levantada quando um n
     * apresenta uma estrutura interna invlida.
     */
    public Object visit(JNameList node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException {

        if (node == null) {
            throw new IllegalArgumentException();
        }
        JNameList resultNode = new JNameList();
        if (node.isASourceNode()) {
            int i = 0;
            int size = node.size();
            while (i < size) {
                if (this.state.top() instanceof JName) {
                    resultNode.add(this.state.pop());
                }
                i++;
            }
        }
        this.doCommonCloningWork(node, resultNode);
        return resultNode;
    }

    /**
     * Mtodos visit para objetos do tipo <code>OtherDeclarationsSet</code>.
     *
     * @param node O n a ser clonado.
     * @param data Um objeto que pode conter informao adicional necessria ao
     * visitor.
     *
     * @return Um objeto qualquer.
     *
     * @exception java.lang.IllegalArgumentException Levantada quando o
     * parmetro <code>node</code> for <code>null</code>.
     * @exception java.lang.IllegalArgumentException Levantada quando um n
     * apresenta uma estrutura interna invlida.
     */
    public Object visit(OtherDeclarationsSet node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException {

        if (node == null) {
            throw new IllegalArgumentException();
        }

        OtherDeclarationsSet resultNode = new OtherDeclarationsSet();

        // Clona as "outras" declaraes.
        int max3 = node.numberOfConditionalDeclarations();
        int k = 0;

        while (k < max3) {
            INode tempOtherDec = this.state.pop();
            resultNode.addConditionalDeclaration(tempOtherDec);
            if (tempOtherDec instanceof NestableExecutableDeclaration) {
                ((NestableExecutableDeclaration) tempOtherDec).setParentNode(resultNode);
            }
            k++;
        }

        // Clona as declaraes resultantes.
        int max = node.numberOfResultDeclarations();
        int i = 0;

        while (i < max) {
            INode tempResDec = this.state.pop();
            resultNode.add(tempResDec);
            i++;
        }

        // Clona as declaraes iterativas.
        int max2 = node.numberOfIterativeDeclarations();
        int j = 0;

        while (j < max2) {
            if (!(this.state.top() instanceof IterativeDeclaration)) {
                throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
            }
            IterativeDeclaration tempIterDec =
                    (IterativeDeclaration) this.state.pop();
            resultNode.addIterativeDeclaration(tempIterDec);
            tempIterDec.setParentNode(resultNode);
            j++;
        }

        this.doCommonCloningWork(node, resultNode);

        return resultNode;
    }


    /**
     * Mtodos visit para objetos do tipo <code>JPackageDeclaration</code>.
     *
     * @param node O n a ser clonado.
     * @param data Um objeto que pode conter informao adicional necessria ao<br>
     * visitor.
     *
     * @return Um objeto qualquer.
     *
     * @exception java.lang.IllegalArgumentException Levantada quando o
     * parmetro <code>node</code> for <code>null</code>.
     * @exception java.lang.IllegalArgumentException Levantada quando um n
     * apresenta uma estrutura interna invlida.
     */
    public Object visit(JPackageDeclaration node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException {

        if (node == null) {
            throw new IllegalArgumentException();
        }
        JPackageDeclaration resultNode = new JPackageDeclaration();
        if (node.isASourceNode()) {
            resultNode = new JPackageDeclaration();
            JName tmp = new JName();
            if (this.state.top() instanceof JName) {
                tmp = (JName) this.state.pop();
                resultNode.setName(tmp);
            }

        }
        this.doCommonCloningWork(node, resultNode);
        return resultNode;
    }

    /**
     * Mtodos visit para objetos do tipo <code>JParameterList</code>.
     *
     * @param node O n a ser clonado.
     * @param data Um objeto que pode conter informao adicional necessria ao<br>
     * visitor.
     *
     * @return Um objeto qualquer.
     *
     * @exception java.lang.IllegalArgumentException Levantada quando o
     * parmetro <code>node</code> for <code>null</code>.
     * @exception java.lang.IllegalArgumentException Levantada quando um n
     * apresenta uma estrutura interna invlida.
     */
    public Object visit(JParameterList node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException {

        if (node == null) {
            throw new IllegalArgumentException();
        }
        JParameterList resultNode = new JParameterList();
        JFormalParameter tmp = new JFormalParameter();
        if (node.isASourceNode()) {
            int i = 0;
            int size = node.size();
            while (i < size) {
                if (this.state.top() instanceof JFormalParameter) {
                    tmp = (JFormalParameter) this.state.pop();
                    resultNode.add(tmp);
                }
                i++;
            }

        }
        this.doCommonCloningWork(node, resultNode);
        return resultNode;
    }

    /**
     * Mtodos visit para objetos do tipo <code>JPreconditionDeclaration</code>.
     *
     * @param node O n a ser impresso.
     * @param data Um objeto que pode conter informao adicional necessria ao<br>
     * visitor.
     *
     * @return Um objeto qualquer.
     *
     * @exception java.lang.IllegalArgumentException Levantada quando o
     * parmetro <code>node</code> for <code>null</code>.
     * @exception java.lang.IllegalArgumentException Levantada quando um n
     * apresenta uma estrutura interna invlida.
     */
    public Object visit(JPreconditionDeclaration node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException {

        if (node == null) {
            throw new IllegalArgumentException();
        }
        JPreconditionDeclaration resultNode = new JPreconditionDeclaration();
//        if (!(this.state.top() instanceof JExecutableDeclaration)) {
//            throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException("Literal nulo");
//        }
        
//        if (!(this.state.top() instanceof JLiteral)) {
//            throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException("Literal nulo");
//        }
//        JLiteral l = (JLiteral) this.state.pop();
//        JExecutableDeclaration exec = (JExecutableDeclaration) this.state.pop();
//        resultNode.setExecutableDeclaration(exec);

        this.doCommonCloningWork(node, resultNode);
        return resultNode;
    }

    /**
     * Mtodos visit para objetos do tipo <code>JResultStatement</code>.
     *
     * @param node O n a ser impresso.
     * @param data Um objeto que pode conter informao adicional necessria ao<br>
     * visitor.
     *
     * @return Um objeto qualquer.
     *
     * @exception java.lang.IllegalArgumentException Levantada quando o
     * parmetro <code>node</code> for <code>null</code>.
     * @exception java.lang.IllegalArgumentException Levantada quando um n
     * apresenta uma estrutura interna invlida.
     */
    public Object visit(JResultStatement node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {

        if (node == null) {
            throw new IllegalArgumentException();
        }

        JResultStatement resultNode = new JResultStatement();
        if (node.isASourceNode()) {

            // clona o comentrio
            if (node.getComment() != null) {
                resultNode.setComment(node.getComment());
            }
            // clona o tipo
            resultNode.setType(node.getType());
            // clona a expresso, se houver
            if (node.getExpression() != null) {
                if (!(this.state.top() instanceof JExpression)) {
                    throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
                }
                resultNode.setExpression((JExpression) this.state.pop());
            }
        }
        this.doCommonCloningWork(node, resultNode);
        return resultNode;

    }


    /**
     * Mtodos visit para objetos do tipo <code>JStatementExpression</code>.
     *
     * @param node O n a ser clonado.
     * @param data Um objeto que pode conter informao adicional necessria ao
     * visitor.
     *
     * @return Um objeto qualquer.
     *
     * @exception java.lang.IllegalArgumentException Levantada quando o
     * parmetro <code>node</code> for <code>null</code>.
     * @exception java.lang.IllegalArgumentException Levantada quando um n
     * apresenta uma estrutura interna invlida.
     */
    public  Object visit( JStatementExpression node,  Object data) throws IllegalArgumentException, cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException{
        if (node == null) {
            throw new IllegalArgumentException("node is null");
        }
        JStatementExpression resultNode = new JStatementExpression();
        if (node.isASourceNode()) {
            if (node.getExpression() == null
                    && !(this.state.top() instanceof JExpression)) {
                throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
            }
            resultNode.setExpression((JExpression)this.state.pop());
        }
        this.doCommonCloningWork(node, resultNode);
        return resultNode;
    }

    /**
     * Mtodos visit para objetos do tipo <code>JSwitchStatement</code>.
     *
     * @param node O n a ser clonado.
     * @param data Um objeto que pode conter informao adicional necessria ao
     * visitor.
     *
     * @return Um objeto qualquer.
     *
     * @exception java.lang.IllegalArgumentException Levantada quando o
     * parmetro <code>node</code> for <code>null</code>.
     * @exception java.lang.IllegalArgumentException Levantada quando um n
     * apresenta uma estrutura interna invlida.
     */
    public Object visit(JSwitchStatement node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {

        if (node == null) {
            throw new IllegalArgumentException();
        }

        JSwitchStatement resultNode = new JSwitchStatement();
        if (!node.isExecutable() && !node.isVariable()) {
            // clona a expresso do switch
            if (node.getExpression() != null) {
                if (!(this.state.top() instanceof JExpression)) {
                    throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
                }
                resultNode.setExpression((JExpression) this.state.pop());
            }
            // clona os cases
            if (node.getCases() != null) {
                int max = node.getCases().size();
                NodeList cases = new NodeList();
                for (int i = 0; i < max; i++) {
                    if (!(this.state.top() instanceof JSwitchPair)) {
                        throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
                    }
                    cases.addElement(this.state.pop());
                }
                resultNode.setCases(cases);
            }
        }
        this.doCommonCloningWork(node, resultNode);
        return resultNode;
    }

    /**
     * Mtodos visit para objetos do tipo <code>JSwitchLabel</code>.
     *
     * @param node O n a ser clonado.
     * @param data Um objeto que pode conter informao adicional necessria ao
     * visitor.
     *
     * @return Um objeto qualquer.
     *
     * @exception java.lang.IllegalArgumentException Levantada quando o
     * parmetro <code>node</code> for <code>null</code>.
     * @exception java.lang.IllegalArgumentException Levantada quando um n
     * apresenta uma estrutura interna invlida.
     */
    public Object visit(JSwitchLabel node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {

        if (node == null) {
            throw new IllegalArgumentException();
        }
        JSwitchLabel resultNode = new JSwitchLabel();
        if (node.isASourceNode()) {
            // clona o tipo
            resultNode.setType(node.getType());

            // clona a expresso, se houver
            if (node.getExpression() != null) {
                if (!(this.state.top() instanceof JExpression)) {
                    throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
                }
                resultNode.setExpression((JExpression) this.state.pop());
            }
        }
        this.doCommonCloningWork(node, resultNode);
        return resultNode;
    }

    /**
     * Mtodos visit para objetos do tipo <code>JSwitchPair</code>.
     *
     * @param node O n a ser clonado.
     * @param data Um objeto que pode conter informao adicional necessria ao
     * visitor.
     *
     * @return Um objeto qualquer.
     *
     * @exception java.lang.IllegalArgumentException Levantada quando o
     * parmetro <code>node</code> for <code>null</code>.
     * @exception java.lang.IllegalArgumentException Levantada quando um n
     * apresenta uma estrutura interna invlida.
     */
    public Object visit(JSwitchPair node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {

        if (node == null) {
            throw new IllegalArgumentException();
        }
        JSwitchPair resultNode = new JSwitchPair();
        if (node.isASourceNode()) {
            // clona o label
            if (node.getLabel() != null) {
                if (!(this.state.top() instanceof JSwitchLabel)) {
                    throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
                }
                resultNode.setLabel((JSwitchLabel) this.state.pop());
            }

            // clona a lista de statements associados
            if (node.getStatements() != null) {
                int max = node.getStatements().size();
                NodeList statements = new NodeList();

                for (int i = 0; i < max; i++) {
                    if (!(this.state.top() instanceof JStatement)) {
                        throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
                    }
                    statements.addElement(this.state.pop());
                }
                resultNode.setStatements(statements);
            }

        }
        this.doCommonCloningWork(node, resultNode);
        return resultNode;
    }

    /**
     * Mtodos visit para objetos do tipo <code>JTernaryExpression</code>.
     *
     * @param node O n a ser clonado.
     * @param data Um objeto que pode conter informao adicional necessria ao
     * visitor.
     *
     * @return Um objeto qualquer.
     *
     * @exception java.lang.IllegalArgumentException Levantada quando o
     * parmetro <code>node</code> for <code>null</code>.
     * @exception java.lang.IllegalArgumentException Levantada quando um n
     * apresenta uma estrutura interna invlida.
     */
    public Object visit(JTernaryExpression node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException {
        if (node == null) {
            throw new IllegalArgumentException();
        }

        JTernaryExpression resultNode = new JTernaryExpression();

        if (node.getThirdOperand() == null
                || !(this.state.top() instanceof JExpression))
            throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
        JExpression third = (JExpression) this.state.pop();

        if (node.getSecondOperand() == null
                || !(this.state.top() instanceof JExpression))
            throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
        JExpression second = (JExpression) this.state.pop();

        if (node.getFirstOperand() == null
                || !(this.state.top() instanceof JExpression))
            throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
        JExpression first = (JExpression) this.state.pop();

        resultNode.setThirdOperand(third);
        resultNode.setSecondOperand(second);
        resultNode.setFirstOperand(first);

        resultNode.setOperator(node.getOperator());
        this.doCommonCloningWork(node, resultNode);

        return resultNode;
    }

    /**
     * Mtodos visit para objetos do tipo <code>JTryStatement</code>.
     *
     * @param node O n a ser clonado.
     * @param data Um objeto que pode conter informao adicional necessria ao
     * visitor.
     *
     * @return Um objeto qualquer.
     *
     * @exception java.lang.IllegalArgumentException Levantada quando o
     * parmetro <code>node</code> for <code>null</code>.
     * @exception java.lang.IllegalArgumentException Levantada quando um n
     * apresenta uma estrutura interna invlida.
     */
    public Object visit(JTryStatement node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {
        if (node == null) {
            throw new IllegalArgumentException("JTryStatement nulo");
        }

        JTryStatement resultNode = new JTryStatement();

        if (node.isASourceNode()) {
            // clona o bloco try
            if (node.getTryBlock() != null) {
                if (!(this.state.top() instanceof JBlock)) {
                    throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
                }
                resultNode.setTryBlock((JBlock) this.state.pop());
            }

            // clona o statement
            if (node.getCatchlist() != null) {
                int max = node.getCatchlist().size();
                NodeList catchList = new NodeList();
                for (int i = 0; i < max; i++) {
                    if (!(this.state.top() instanceof JCatchClause)) {
                        throw new IllegalArgumentException("");
                    }
                    catchList.addElement(this.state.pop());
                }
                resultNode.setCatchlist(catchList);
            }

            // clona o bloco finally, se houver
            if (node.getFinallyBlock() != null) {
                if (!(this.state.top() instanceof JBlock)) {
                    throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
                }
                resultNode.setFinallyBlock((JBlock) this.state.pop());
            }
        }

        this.doCommonCloningWork(node, resultNode);

        return resultNode;
    }


    /**
     * Mtodos visit para objetos do tipo <code>JType</code>.
     *
     * @param node O n a ser clonado.
     * @param data Um objeto que pode conter informao adicional necessria ao
     * visitor.
     *
     * @return Um objeto qualquer.
     *
     * @exception java.lang.IllegalArgumentException Levantada quando o
     * parmetro <code>node</code> for <code>null</code>.
     * @exception java.lang.IllegalArgumentException Levantada quando um n
     * apresenta uma estrutura interna invlida.
     */
    public Object visit(JType node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException {

        if (node == null) {
            throw new IllegalArgumentException();
        }
        JType resultNode = new JType();
        if (node.isASourceNode()) {
            if ((node.getTypeName() == null
                    || !(this.state.top() instanceof JName)))
                throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);

            JName typeName = (JName) this.state.pop();
            resultNode.setName(typeName);
        }

        resultNode.setNumberOfDimensions(node.getNumberOfDimensions());
        this.doCommonCloningWork(node, resultNode);
        return resultNode;
    }

    /**
     * Mtodos visit para objetos do tipo <code>JMethodDeclarationSet</code>.
     * @param node O n a ser clonado.
     * @param data Um objeto que pode conter informao adicional necessria ao<br>
     *        visitor.
     * @return Um objeto qualquer.
     * @exception java.lang.IllegalArgumentException Levantada quando o
     *            parmetro <code>node</code> for <code>null</code>.
     * @exception java.lang.IllegalArgumentException Levantada quando um n
     *            apresenta uma estrutura interna invlida.
     */
    public Object visit(TypeDeclarationSet node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException {

        if (node == null) {
            throw new IllegalArgumentException();
        }
        TypeDeclarationSet resultNode = new TypeDeclarationSet();
        JTypeDeclaration tmp = null;
        int i = 0;
        int size = node.size();
        while (i < size) {
            if (this.state.top() instanceof JTypeDeclaration) {
                tmp = (JTypeDeclaration) this.state.pop();
                resultNode.add(tmp);
            }
            i++;
        }
        this.doCommonCloningWork(node, resultNode);
        return resultNode;
    }


    /**
     * Mtodos visit para objetos do tipo <code>JUnaryExpression</code>.
     *
     * @param node O n a ser clonado.
     * @param data Um objeto que pode conter informao adicional necessria ao
     * visitor.
     *
     * @return Um objeto qualquer.
     *
     * @exception java.lang.IllegalArgumentException Levantada quando o
     * parmetro <code>node</code> for <code>null</code>.
     * @exception java.lang.IllegalArgumentException Levantada quando um n
     * apresenta uma estrutura interna invlida.
     */
    public Object visit(JUnaryExpression node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException {
        if (node == null) {
            throw new IllegalArgumentException();
        }

        JUnaryExpression resultNode = new JUnaryExpression();

        if (node.getOperand() == null
                || !(this.state.top() instanceof JExpression))
            throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
        JExpression operand = (JExpression) this.state.pop();

        resultNode.setOperand(operand);
        resultNode.setOperator(node.getOperator());

        this.doCommonCloningWork(node, resultNode);
        return resultNode;
    }


    /**
     * Mtodos visit para objetos do tipo <code>JVariableDeclarator</code>.
     *
     * @param node O n a ser clonado.
     * @param data Um objeto que pode conter informao adicional necessria
     * ao visitor.
     *
     * @return Um objeto qualquer.
     *
     * @exception java.lang.IllegalArgumentException Levantada quando o
     * parmetro <code>node</code> for <code>null</code>.
     * @exception java.lang.IllegalArgumentException Levantada quando um n
     * apresenta uma estrutura interna invlida.
     */
    public Object visit(JVariableDeclarator node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException {

        if (node == null) {
            throw new IllegalArgumentException("JVariableDeclarator nulo");
        }
        JVariableDeclarator resultNode = new JVariableDeclarator();
        if (node.isASourceNode()) {
            if ((node.getIdentifier() == null ||
                    !(this.state.top() instanceof JIdentifier))) {
                throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
            }
            JIdentifier id = (JIdentifier) this.state.pop();
            resultNode.setIdentifier(id);
        }

        if (node.getInitializer() != null) {
            INode init = this.state.pop();
            resultNode.setInitializer(init);
        }
        resultNode.setDimensions(node.getDimensions());
        this.doCommonCloningWork(node, resultNode);
        return resultNode;
    }

    /**
     * Mtodos visit para objetos do tipo <code>JWhileStatement</code>.
     *
     * @param node O n a ser clonado.
     * @param data Um objeto que pode conter informao adicional necessria
     * ao visitor.
     *
     * @return Um objeto qualquer.
     *
     * @exception java.lang.IllegalArgumentException Levantada quando o
     * parmetro <code>node</code> for <code>null</code>.
     * @exception java.lang.IllegalArgumentException Levantada quando um n
     * apresenta uma estrutura interna invlida.
     */
    public Object visit(JWhileStatement node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {

        if (node == null) {
            throw new IllegalArgumentException("JWhileStatement nulo");
        }
        JWhileStatement resultNode = new JWhileStatement();
        if (node.isASourceNode()) {
            // clona o tipo
            resultNode.setType(node.getType());

            // se tiver comentrio, clona o comentrio
            if (node.getComment() != null) {
                resultNode.setComment(node.getComment());
            }


            // clona a expresso
            if (node.getExpression() != null) {
                if (!(this.state.top() instanceof JExpression)) {
                    throw new cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException(node);
                }
                resultNode.setExpression((JExpression) this.state.pop());
            }
            // clona o statement
            if (node.getStatement() != null) {
                if (!(this.state.top() instanceof JStatement)
                		&& !(this.state.top() instanceof JUnaryExpression)) {
                    throw new InconsistentNodeException(node);
                }
                resultNode.setStatement(this.state.pop());
            }
        }
        this.doCommonCloningWork(node, resultNode);
        return resultNode;

    }


	public Object visit(JStatementList node, Object data)
			throws IllegalArgumentException, InconsistentNodeException,
			ExecutionException {
		JStatementList resultNode = new JStatementList();
		resultNode = cloneStatementList(node, resultNode);
		this.doCommonCloningWork(node, resultNode);
		return resultNode;

	}


	public Object visit(ExploreDeclaration node, Object data) throws IllegalArgumentException, InconsistentNodeException, ExecutionException {
        if (node == null) {
            throw new IllegalArgumentException();
        }
        
		ExploreDeclaration resultNode = new ExploreDeclaration();
        
		NodeList whereClauses = new NodeList();

        int i = node.getWhereClauses().size() - 1;
        while (i >= 0) {
            INode dec = this.state.pop();
            whereClauses.addElement(dec);
            i--;
        }

        resultNode.setWhereClauses(whereClauses);
		resultNode.setBaseNode(this.state.pop());

		this.doCommonCloningWork(node, resultNode);
		
		return resultNode;
	}


	public Object visit(WhereClause node, Object data) throws IllegalArgumentException, InconsistentNodeException, ExecutionException {
        if (node == null) {
            throw new IllegalArgumentException();
        }

		WhereClause resultNode = new WhereClause();

		resultNode.setClauseType(node.getClauseType());		
		
		for (int i = 0; i < node.getTransformationNodes().size(); i++) {
			resultNode.addTransformationNode((MicroTransformationNode)this.state.pop());			
		}
		
		this.doCommonCloningWork(node, resultNode);

		return resultNode;
	}


	public Object visit(MicroTransformationNode node, Object data) throws IllegalArgumentException, InconsistentNodeException, ExecutionException {
        if (node == null) {
            throw new IllegalArgumentException();
        }
		
		MicroTransformationNode resultNode = new MicroTransformationNode();
		CloningVisitor cv = new CloningVisitor();
		INode left = node.getTransformation().getSource();
		INode leftClone = (INode) left.accept(cv, data);
		cv = new CloningVisitor();
		INode right = node.getTransformation().getTarget();
		INode rightClone = (INode) right.accept(cv, data);		
		INode pre = node.getTransformation().getPrecondition();		
		INode preClone = null;
		if (pre != null)
			preClone = (INode) pre.accept(cv, data);
		
		MicroTransformation transformation = new MicroTransformation(leftClone, rightClone);
		transformation.setType(node.getTransformation().getType());
		transformation.setPrecondition((JPreconditionDeclaration) preClone);
		
		resultNode.setTransformation(transformation);

		this.doCommonCloningWork(node, resultNode);
		
		return resultNode;
	}


	public Object visit(Pattern node, Object data) throws IllegalArgumentException, InconsistentNodeException, ExecutionException {
        if (node == null) {
            throw new IllegalArgumentException();
        }
        Pattern resultNode = new Pattern();
        
        resultNode.setBody((JaTSNode) this.state.pop());
        resultNode.setName((JIdentifier) this.state.pop());
        
		this.doCommonCloningWork(node, resultNode);

		return resultNode;
	}


	public Object visit(PrefixedIdentifier node, Object data)
			throws IllegalArgumentException, InconsistentNodeException,
			ExecutionException {

		if (node == null) {
			throw new IllegalArgumentException();
		}

		PrefixedIdentifier resultNode = new PrefixedIdentifier();

		resultNode.setPrefix(node.getPrefix());
		resultNode.setRootId((JIdentifier) this.state.pop());

		this.doCommonCloningWork(node, resultNode);

		return resultNode;
	}
	
    public Object visit(ContextDeclaration node, Object data)
			throws IllegalArgumentException, InconsistentNodeException,
			ExecutionException {

		if (node == null) {
			throw new IllegalArgumentException();
		}

		ContextDeclaration resultNode = new ContextDeclaration();

		resultNode.setCodeOfInterest((JStatementList) this.state.pop());

		resultNode = (ContextDeclaration) cloneStatementList(node, resultNode);

		//resultNode.setContext((JStatementList) this.state.pop());
		//resultNode.setCodePosition(node.getCodePosition());

		this.doCommonCloningWork(node, resultNode);

		return resultNode;
	}
	
}