/*
 * JaTS Project - Java Transformation System
 * http://www.cin.ufpe.br/~jats
 *  
 * Copyright  2005  Informatics Center - Federal University of Pernambuco
 * 
 * JaTS is a free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 */
package cin.jats.engine.visitors;

import cin.jats.engine.parser.nodes.BodyFieldDeclarations;
import cin.jats.engine.parser.nodes.BodyMethodDeclarations;
import cin.jats.engine.parser.nodes.BodyTypeDeclarations;
import cin.jats.engine.parser.nodes.CastExpression;
import cin.jats.engine.parser.nodes.ClassBodyConstructorDeclarations;
import cin.jats.engine.parser.nodes.ClassBodyInitializers;
import cin.jats.engine.parser.nodes.ContextDeclaration;
import cin.jats.engine.parser.nodes.ExploreDeclaration;
import cin.jats.engine.parser.nodes.INode;
import cin.jats.engine.parser.nodes.ImportDeclarations;
import cin.jats.engine.parser.nodes.InstanceOfExpression;
import cin.jats.engine.parser.nodes.IterativeDeclaration;
import cin.jats.engine.parser.nodes.JAllocationExpression;
import cin.jats.engine.parser.nodes.JAnonymousClass;
import cin.jats.engine.parser.nodes.JArrayAccess;
import cin.jats.engine.parser.nodes.JArrayDimsAndInits;
import cin.jats.engine.parser.nodes.JArrayInitializer;
import cin.jats.engine.parser.nodes.JBinaryExpression;
import cin.jats.engine.parser.nodes.JBlock;
import cin.jats.engine.parser.nodes.JCatchClause;
import cin.jats.engine.parser.nodes.JClassDeclaration;
import cin.jats.engine.parser.nodes.JCompilationUnit;
import cin.jats.engine.parser.nodes.JConditionalDeclaration;
import cin.jats.engine.parser.nodes.JConstructorDeclaration;
import cin.jats.engine.parser.nodes.JConstructorDeclarationSet;
import cin.jats.engine.parser.nodes.JExecutableDeclaration;
import cin.jats.engine.parser.nodes.JExpressionList;
import cin.jats.engine.parser.nodes.JFieldAccess;
import cin.jats.engine.parser.nodes.JFieldDeclaration;
import cin.jats.engine.parser.nodes.JFieldDeclarationSet;
import cin.jats.engine.parser.nodes.JForStatement;
import cin.jats.engine.parser.nodes.JFormalParameter;
import cin.jats.engine.parser.nodes.JIdentifier;
import cin.jats.engine.parser.nodes.JIfStatement;
import cin.jats.engine.parser.nodes.JImportDeclaration;
import cin.jats.engine.parser.nodes.JImportDeclarationSet;
import cin.jats.engine.parser.nodes.JInitializer;
import cin.jats.engine.parser.nodes.JInitializerSet;
import cin.jats.engine.parser.nodes.JInterfaceDeclaration;
import cin.jats.engine.parser.nodes.JLiteral;
import cin.jats.engine.parser.nodes.JLocalVariableDeclaration;
import cin.jats.engine.parser.nodes.JMethodDeclaration;
import cin.jats.engine.parser.nodes.JMethodDeclarationSet;
import cin.jats.engine.parser.nodes.JMethodInvocation;
import cin.jats.engine.parser.nodes.JModifierList;
import cin.jats.engine.parser.nodes.JName;
import cin.jats.engine.parser.nodes.JNameList;
import cin.jats.engine.parser.nodes.JNodeString;
import cin.jats.engine.parser.nodes.JPackageDeclaration;
import cin.jats.engine.parser.nodes.JParameterList;
import cin.jats.engine.parser.nodes.JPreconditionDeclaration;
import cin.jats.engine.parser.nodes.JResultStatement;
import cin.jats.engine.parser.nodes.JStatementExpression;
import cin.jats.engine.parser.nodes.JStatementList;
import cin.jats.engine.parser.nodes.JSwitchLabel;
import cin.jats.engine.parser.nodes.JSwitchPair;
import cin.jats.engine.parser.nodes.JSwitchStatement;
import cin.jats.engine.parser.nodes.JSynchronizedStatement;
import cin.jats.engine.parser.nodes.JTernaryExpression;
import cin.jats.engine.parser.nodes.JTryStatement;
import cin.jats.engine.parser.nodes.JType;
import cin.jats.engine.parser.nodes.JUnaryExpression;
import cin.jats.engine.parser.nodes.JVariableDeclarator;
import cin.jats.engine.parser.nodes.JWhileStatement;
import cin.jats.engine.parser.nodes.MapTable;
import cin.jats.engine.parser.nodes.MicroTransformationNode;
import cin.jats.engine.parser.nodes.OtherDeclarationsSet;
import cin.jats.engine.parser.nodes.Pattern;
import cin.jats.engine.parser.nodes.PrefixedIdentifier;
import cin.jats.engine.parser.nodes.TypeBody;
import cin.jats.engine.parser.nodes.TypeDeclarationSet;
import cin.jats.engine.parser.nodes.WhereClause;
import cin.jats.engine.parser.nodes.WrapperNode;
import cin.jats.engine.parser.nodes.exceptions.ExecutionException;
import cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException;
import cin.jats.engine.parser.nodes.exceptions.NodesNotMatchedException;
import cin.jats.engine.parser.nodes.metajats.CodeAssignStatement;
import cin.jats.engine.parser.nodes.metajats.ParsingStatement;
import cin.jats.engine.parser.nodes.metajats.RSetEntryStatement;
import cin.jats.engine.parser.nodes.metajats.TransfDeclaration;
import cin.jats.engine.util.NodeList;
import cin.jats.engine.util.ResultSet;


/**
 * This visitor traverses the AST collecting nodes that match
 * with the "matchingValue" field.
 * All methods are delegators which call the "collect" method.
 * The collected values are inserted in a NodeList object
 * represented by the "collectedValues" field.
 * 
 * @author Gustavo Santos
 * @created 08/12/2005
 * @version 08/12/2005
 * 
 * @see cin.jats.engine.visitors.IVisitor
 *
 */
public class CollectorVisitor extends AbstractVisitor {

	private ResultSet results;
	private NodeList collectedValues;
	private INode matchingValue;

	public CollectorVisitor() {
		results = new ResultSet();
		collectedValues = new NodeList();
	}
	
    /**
	 * @see JAllocationExpression
	 */
    public Object visit(JAllocationExpression node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {
        return collect(node, data);
    }

    /**
	 * @see JAnonymousClass
	 */
    public Object visit(JAnonymousClass node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {
        return collect(node, data);
    }

    /**
	 * @see JArrayAccess
	 */
    public Object visit(JArrayAccess node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {
        return collect(node, data);
    }

    /**
	 * @see JArrayDimsAndInits
	 */
    public Object visit(JArrayDimsAndInits node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {
        return collect(node, data);
    }

    /**
	 * @see JArrayInitializer
	 */
    public Object visit(JArrayInitializer node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {
        return collect(node, data);
    }

    /**
	 * @see Attribute
	 */
//    public Object visit(Attribute node, Object data)
//            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
//            ExecutionException {
//        return collect(node, data);
//    }

//    /**
//	 * @see AttributeArgument
//	 */
//    public Object visit(AttributeArgument node, Object data)
//            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
//            ExecutionException {
//        return collect(node, data);
//    }
//
//    /**
//	 * @see AttributeArgumentList
//	 */
//    public Object visit(AttributeArgumentList node, Object data)
//            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
//            ExecutionException {
//        return collect(node, data);
//    }

    /**
	 * @see AttributeSection
	 */
//    public Object visit(AttributeSection node, Object data)
//            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
//            ExecutionException {
//        return collect(node, data);
//    }


    /**
	 * @see JBinaryExpression
	 */
    public Object visit(JBinaryExpression node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {
        return collect(node, data);
    }

    /**
	 * @see JBlock
	 */
    public Object visit(JBlock node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {
        return collect(node, data);
    }

    /**
	 * @see CastExpression
	 */
    public Object visit(CastExpression node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {
        return collect(node, data);
    }

    /**
	 * @see JCatchClause
	 */
    public Object visit(JCatchClause node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {
        return collect(node, data);
    }

    /**
	 * @see TypeBody
	 */
    public Object visit(TypeBody node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {
        return collect(node, data);
    }
    

    /**
	 * @see JClassDeclaration
	 */
    public Object visit(JClassDeclaration node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {
        return collect(node, data);
    }

    /**
	 * @see JCompilationUnit
	 */
    public Object visit(JCompilationUnit node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {
        return collect(node, data);
    }

    /**
	 * @see JConditionalDeclaration
	 */
    public Object visit(JConditionalDeclaration node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {
        return collect(node, data);
    }

    /**
	 * @see JConstructorDeclaration
	 */
    public Object visit(JConstructorDeclaration node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {
        return collect(node, data);
    }

    /**
	 * @see JConstructorDeclarationSet
	 */
    public Object visit(JConstructorDeclarationSet node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {
        return collect(node, data);
    }

    /**
	 * @see ClassBodyConstructorDeclarations
	 */
    public Object visit(ClassBodyConstructorDeclarations node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {
        return collect(node, data);
    }

    /**
	 * @see JExecutableDeclaration
	 */
    public Object visit(JExecutableDeclaration node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {
        return collect(node, data);
    }

    /**
	 * @see JExpressionList
	 */
    public Object visit(JExpressionList node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {
        return collect(node, data);
    }

    /**
	 * @see JFieldAccess
	 */
    public Object visit(JFieldAccess node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {
        return collect(node, data);
    }

    /**
	 * @see JFieldDeclaration
	 */
    public Object visit(JFieldDeclaration node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {
        return collect(node, data);
    }

    /**
	 * @see JFieldDeclarationSet
	 */
    public Object visit(JFieldDeclarationSet node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {
        return collect(node, data);
    }

    /**
	 * @see JForStatement
	 */
    public Object visit(JForStatement node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {
        return collect(node, data);
    }

    /**
	 * @see BodyFieldDeclarations
	 */
    public Object visit(BodyFieldDeclarations node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {
        return collect(node, data);
    }

    /**
	 * @see ClassBodyInitializers
	 */
    public Object visit(ClassBodyInitializers node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {
        return collect(node, data);
    }

    /**
	 * @see JFormalParameter
	 */
    public Object visit(JFormalParameter node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {
        return collect(node, data);
    }

    /**
	 * @see JIdentifier
	 */
    public Object visit(JIdentifier node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {
        return collect(node, data);
    }

    /**
	 * @see JIfStatement
	 */
    public Object visit(JIfStatement node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {
        return collect(node, data);
    }

    /**
	 * @see JImportDeclaration
	 */
    public Object visit(JImportDeclaration node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {
        return collect(node, data);
    }

    /**
	 * @see JImportDeclarationSet
	 */
    public Object visit(JImportDeclarationSet node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {
        return collect(node, data);
    }

    /**
	 * @see ImportDeclarations
	 */
    public Object visit(ImportDeclarations node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {
        return collect(node, data);
    }

    /**
	 * @see JInitializer
	 */
    public Object visit(JInitializer node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {
        return collect(node, data);
    }

    /**
	 * @see JInitializerSet
	 */
    public Object visit(JInitializerSet node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {
        return collect(node, data);
    }

    /**
	 * @see JInterfaceDeclaration
	 */
    public Object visit(JInterfaceDeclaration node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {
        return collect(node, data);
    }

    /**
	 * @see InstanceOfExpression
	 */
    public Object visit(InstanceOfExpression node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {
        return collect(node, data);
    }

    /**
	 * @see IterativeDeclaration
	 */
    public Object visit(IterativeDeclaration node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {
        return collect(node, data);
    }

    /**
	 * @see JLiteral
	 */
    public Object visit(JLiteral node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {
        return collect(node, data);
    }

    /**
	 * @see JLocalVariableDeclaration
	 */
    public Object visit(JLocalVariableDeclaration node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {

        return collect(node, data);
    }


    /**
	 * @see JMethodDeclaration
	 */
    public Object visit(JMethodDeclaration node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {
        return collect(node, data);
    }

    /**
	 * @see JMethodDeclarationSet
	 */
    public Object visit(JMethodDeclarationSet node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {
        return collect(node, data);
    }

    /**
	 * @see BodyMethodDeclarations
	 */
    public Object visit(BodyMethodDeclarations node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {
        return collect(node, data);
    }

    /**
	 * @see BodyTypeDeclarations
	 */
    public Object visit(BodyTypeDeclarations node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {
        return collect(node, data);
    }

	
    /**
	 * @see JMethodInvocation
	 */
    public Object visit(JMethodInvocation node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {
        return collect(node, data);
    }

    /**
	 * @see JModifierList
	 */
    public Object visit(JModifierList node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {
        return collect(node, data);
    }

    /**
	 * @see JName
	 */
    public Object visit(JName node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {
        return collect(node, data);
    }

    /**
	 * @see JNodeString
	 */
    public Object visit(JNodeString node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {
        return collect(node, data);
    }

    /**
	 * @see JNameList
	 */
    public Object visit(JNameList node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {
        return collect(node, data);
    }

    /**
	 * @see OtherDeclarationsSet
	 */
    public Object visit(OtherDeclarationsSet node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {
        return collect(node, data);
    }

    /**
	 * @see JPackageDeclaration
	 */
    public Object visit(JPackageDeclaration node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {
        return collect(node, data);
    }

    /**
	 * @see JParameterList
	 */
    public Object visit(JParameterList node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {
        return collect(node, data);
    }

    /**
	 * @see JPreconditionDeclaration
	 */
    public Object visit(JPreconditionDeclaration node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {
        return collect(node, data);
    }


    /**
	 * @see JResultStatement
	 */
    public Object visit(JResultStatement node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {
        return collect(node, data);
    }

    /**
	 * @see JSwitchStatement
	 */
    public Object visit(JSwitchStatement node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {
        return collect(node, data);
    }

    /**
	 * @see JSwitchLabel
	 */
    public Object visit(JSwitchLabel node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {
        return collect(node, data);
    }

    /**
	 * @see JSwitchPair
	 */
    public Object visit(JSwitchPair node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {
        return collect(node, data);
    }

    /**
	 * @see JTernaryExpression
	 */
    public Object visit(JTernaryExpression node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {

        return collect(node, data);
    }

    /**
	 * @see JTryStatement
	 */
    public Object visit(JTryStatement node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {

        return collect(node, data);
    }

    /**
	 * @see JType
	 */
    public Object visit(JType node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {
        return collect(node, data);
    }

    /**
	 * @see TypeDeclarationSet
	 */
    public Object visit(TypeDeclarationSet node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {
        return collect(node, data);
    }

    /**
	 * @see JUnaryExpression
	 */
    public Object visit(JUnaryExpression node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {
        return collect(node, data);
    }


    /**
	 * @see JVariableDeclarator
	 */
    public Object visit(JVariableDeclarator node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {
        return collect(node, data);
    }

    /**
	 * @see JWhileStatement
	 */
    public Object visit(JWhileStatement node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {
        return collect(node, data);
    }

    public Object visit(WrapperNode node, Object data) throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {
        return collect(node, data);
    }

    public Object visit(JSynchronizedStatement node, Object data) throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, ExecutionException {
        return collect(node, data);
    }

    /**
	 * @see JStatementExpression
	 */
    public Object visit(JStatementExpression node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {
        return collect(node, data);
    }

    /**
	 * @see MapTable
	 */
    public Object visit(MapTable node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException {
        return collect(node, data);
    }

    public Object visit(ExploreDeclaration node, Object data)
			throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException,
			IllegalArgumentException, ExecutionException {
		return collect(node, data);
	}

	public Object visit(WhereClause node, Object data)
			throws IllegalArgumentException, InconsistentNodeException,
			ExecutionException {
		return collect(node, data);
	}

	public Object visit(MicroTransformationNode node, Object data)
			throws IllegalArgumentException, InconsistentNodeException,
			ExecutionException {
		return collect(node, data);
	}

	public Object visit(Pattern node, Object data)
			throws IllegalArgumentException, InconsistentNodeException,
			ExecutionException {
		return collect(node, data);
	}

	public Object visit(PrefixedIdentifier node, Object data)
			throws IllegalArgumentException, InconsistentNodeException,
			ExecutionException {
		return collect(node, data);
	}

	public Object visit(ContextDeclaration node, Object data)
			throws IllegalArgumentException, InconsistentNodeException,
			ExecutionException {
	    return collect(node, data);
	}

	public Object visit(JStatementList node, Object data)
			throws IllegalArgumentException, InconsistentNodeException,
			ExecutionException {
		return collect(node, data);
	}
	
    public Object visit(CodeAssignStatement node, Object data)
			throws IllegalArgumentException, InconsistentNodeException,
			ExecutionException {
		return collect(node, data);
	}
    
	public Object visit(ParsingStatement node, Object data)
			throws IllegalArgumentException, InconsistentNodeException,
			ExecutionException {
		return collect(node, data);
	}
	
	public Object visit(RSetEntryStatement node, Object data)
			throws IllegalArgumentException, InconsistentNodeException,
			ExecutionException {
		return collect(node, data);
	}
	
	public Object visit(TransfDeclaration node, Object data)
			throws IllegalArgumentException, InconsistentNodeException,
			ExecutionException {
		return collect(node, data);
	}
	
	
	public Object collect(INode node, Object data) throws IllegalArgumentException, InconsistentNodeException {
		if (matchingValue.getClass().isInstance(node)) {
			
			try {
				matchingValue.match(node, results);
				collectedValues.addElement(node);
			} catch (NodesNotMatchedException e) {
			}
		}
		return collectedValues;
	}

	/**
	 * @return Returns the matchingValue.
	 */
	public INode getMatchingValue() {
		return matchingValue;
	}

	/**
	 * @param matchingValue The matchingValue to set.
	 */
	public void setMatchingValue(INode matchingValue) {
		this.matchingValue = matchingValue;
	}

}
