/*
 * JaTS Project - Java Transformation System
 * http://www.cin.ufpe.br/~jats
 *  
 * Copyright  2005  Informatics Center - Federal University of Pernambuco
 * 
 * JaTS is a free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 */
package cin.jats.engine.visitors;

import cin.jats.engine.parser.nodes.*;

import cin.jats.engine.parser.nodes.exceptions.*;
import cin.jats.engine.parser.nodes.metajats.CodeAssignStatement;
import cin.jats.engine.parser.nodes.metajats.ParsingStatement;
import cin.jats.engine.parser.nodes.metajats.RSetEntryStatement;
import cin.jats.engine.parser.nodes.metajats.TransfDeclaration;


/**
 * Interface que define o repertrio de mtodos que todos os visitors devem
 * implementar.
 *
 * @see JaTSNode
 * @author Fernando Castor
 */
public interface IVisitor {


    /**
     * @see JAllocationExpression
     */
    public Object visit(JAllocationExpression node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException;

    /**
     * @see JAnonymousClass
     */
    public Object visit(JAnonymousClass node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException;

    /**
     * @see JArrayAccess
     */
    public Object visit(JArrayAccess node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException;

    /**
     * @see JArrayDimsAndInits
     */
    public Object visit(JArrayDimsAndInits node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException;

    /**
     * @see JArrayInitializer
     */
    public Object visit(JArrayInitializer node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException;

    /**
     * @see Attribute
     */
//    public Object visit(Attribute node, Object data)
//            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
//            ExecutionException;

    /**
         * @see AttributeArgument
         */
//        public Object visit(AttributeArgument node, Object data)
//                throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
//                ExecutionException;
//
//    /**
//         * @see AttributeArgumentList
//         */
//        public Object visit(AttributeArgumentList node, Object data)
//                throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
//                ExecutionException;


    /**
     * @see AttributeSection
     */
//    public Object visit(AttributeSection node, Object data)
//            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
//            ExecutionException;



    /**
     * @see JBinaryExpression
     */
    public Object visit(JBinaryExpression node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException;

    /**
     * @see JBlock
     */
    public Object visit(JBlock node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException;

    /**
     * @see CastExpression
     */
    public Object visit(CastExpression node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException;

    /**
     * @see JCatchClause
     */
    public Object visit(JCatchClause node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException;

    /**
     * @see TypeBody
     */
    public Object visit(TypeBody node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException;
    
    /**
     * @see JClassDeclaration
     */
    public Object visit(JClassDeclaration node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException;

    /**
     * @see JCompilationUnit
     */
    public Object visit(JCompilationUnit node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException;

    /**
     * @see JConditionalDeclaration
     */
    public Object visit(JConditionalDeclaration node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException;

    /**
     * @see JConstructorDeclaration
     */
    public Object visit(JConstructorDeclaration node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException;


    /**
     * @see JConstructorDeclarationSet
     */
    public Object visit(JConstructorDeclarationSet node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException;

    /**
     * @see ClassBodyConstructorDeclarations
     */
    public Object visit(ClassBodyConstructorDeclarations node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException;

    /**
     * @see ClassBodyInitializers
     */
    public Object visit(ClassBodyInitializers node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException;


    /**
     * @see JExecutableDeclaration
     */
    public Object visit(JExecutableDeclaration node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException;

    /**
     * @see JExpressionList
     */
    public Object visit(JExpressionList node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException;

    /**
     * @see JFieldAccess
     */
    public Object visit(JFieldAccess node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException;

    /**
     * @see JFieldDeclaration
     */
    public Object visit(JFieldDeclaration node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException;

    /**
     * @see JFieldDeclarationSet
     */
    public Object visit(JFieldDeclarationSet node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException;

    /**
     * @see JFormalParameter
     */
    public Object visit(JFormalParameter node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException;

    /**
     * @see JForStatement
     */
    public Object visit(JForStatement node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException;


    /**
     * @see BodyFieldDeclarations
     */
    public Object visit(BodyFieldDeclarations node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException;


    /**
     * @see JIdentifier
     */
    public Object visit(JIdentifier node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException;

    /**
     * @see JIfStatement
     */
    public Object visit(JIfStatement node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException;

    /**
     * @see JImportDeclaration
     */
    public Object visit(JImportDeclaration node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException;

    /**
     * @see JImportDeclarationSet
     */
    public Object visit(JImportDeclarationSet node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException;

    /**
     * @see ImportDeclarations
     */
    public Object visit(ImportDeclarations node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException;

    /**
     * @see JInitializer
     */
    public Object visit(JInitializer node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException;

    /**
     * @see JInitializerSet
     */
    public Object visit(JInitializerSet node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException;


    /**
     * @see JInterfaceDeclaration
     */
    public Object visit(JInterfaceDeclaration node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException;


    /**
     * @see InstanceOfExpression
     */
    public Object visit(InstanceOfExpression node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException;

    /**
     * @see IterativeDeclaration
     */
    public Object visit(IterativeDeclaration node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException;

    /**
     * @see JLiteral
     */
    public Object visit(JLiteral node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException;

    /**
     * @see JLocalVariableDeclaration
     */
    public Object visit(JLocalVariableDeclaration node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException;


    /**
     * @see JMethodDeclaration
     */
    public Object visit(JMethodDeclaration node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException;

    /**
     * @see JMethodDeclarationSet
     */
    public Object visit(JMethodDeclarationSet node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException;

    /**
     * @see BodyMethodDeclarations
     */
    public Object visit(BodyMethodDeclarations node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException;

    /**
     * @see BodyTypeDeclarations
     */
    public Object visit(BodyTypeDeclarations node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException;
    
    /**
     * @see JMethodInvocation
     */
    public Object visit(JMethodInvocation node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException;

    /**
     * @see JModifierList
     */
    public Object visit(JModifierList node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException;

    /**
     * @see JName
     */
    public Object visit(JName node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException;

    /**
     * @see JNodeString
     */
    public Object visit(JNodeString node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException;

    /**
     * @see JNameList
     */
    public Object visit(JNameList node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException;

    /**
     * @see OtherDeclarationsSet
     */
    public Object visit(OtherDeclarationsSet node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException;

    /**
     * @see JPackageDeclaration
     */
    public Object visit(JPackageDeclaration node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException;

    /**
     * @see JParameterList
     */
    public Object visit(JParameterList node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException;

    /**
     * @see JPreconditionDeclaration
     */
    public Object visit(JPreconditionDeclaration node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException;

    /**
     * @see JResultStatement
     */
    public Object visit(JResultStatement node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException;

    /**
     * @see JStatementExpression
     */
    public Object visit(JStatementExpression node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException;

    /**
     * @see JSwitchStatement
     */
    public Object visit(JSwitchStatement node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException;

    /**
     * @see JSwitchLabel
     */
    public Object visit(JSwitchLabel node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException;

    /**
     * @see JSwitchPair
     */
    public Object visit(JSwitchPair node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException;

    /**
     * @see JTernaryExpression
     */
    public Object visit(JTernaryExpression node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException;


    /**
     * @see JTryStatement
     */
    public Object visit(JTryStatement node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException;


    /**
     * @see JType
     */
    public Object visit(JType node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException;

    /**
     * @see TypeDeclarationSet
     */
    public Object visit(TypeDeclarationSet node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException;

    /**
     * @see JUnaryExpression
     */
    public Object visit(JUnaryExpression node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException;

    /**
     * @see JVariableDeclarator
     */
    public Object visit(JVariableDeclarator node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException;

    /**
     * @see JWhileStatement
     */
    public Object visit(JWhileStatement node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException;

    public Object visit(WrapperNode node, Object data) throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException;

    public Object visit(JSynchronizedStatement node, Object data) throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, ExecutionException;
    
//    public Object visit(EnumDeclaration node, Object data)
//		throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
//		ExecutionException;
//    
//    public Object visit(EnumMember node, Object data)
//			throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
//			ExecutionException;
    /**
     * @see MapTable
     */
    public Object visit(MapTable node, Object data)
            throws cin.jats.engine.parser.nodes.exceptions.InconsistentNodeException, IllegalArgumentException,
            ExecutionException;

    /**
     * @see OrDeclaration
     */
    public Object visit(OrDeclaration node, Object data);
    
    
    public Object visit(JStatementList node, Object data)
			throws IllegalArgumentException, InconsistentNodeException,
			ExecutionException;

	public Object visit(ExploreDeclaration node, Object data)
			throws IllegalArgumentException, InconsistentNodeException,
			ExecutionException;

	public Object visit(WhereClause node, Object data)
			throws IllegalArgumentException, InconsistentNodeException,
			ExecutionException;

	public Object visit(MicroTransformationNode node, Object data)
			throws IllegalArgumentException, InconsistentNodeException,
			ExecutionException;

    public Object visit(Pattern node, Object data)
			throws IllegalArgumentException, InconsistentNodeException,
			ExecutionException;

    public Object visit(PrefixedIdentifier node, Object data)
			throws IllegalArgumentException, InconsistentNodeException,
			ExecutionException;

    public Object visit(ContextDeclaration node, Object data)
			throws IllegalArgumentException, InconsistentNodeException,
			ExecutionException;
 
    public Object visit(CodeAssignStatement node, Object data)
			throws IllegalArgumentException, InconsistentNodeException,
			ExecutionException;

	public Object visit(ParsingStatement node, Object data)
			throws IllegalArgumentException, InconsistentNodeException,
			ExecutionException;

	public Object visit(RSetEntryStatement node, Object data)
			throws IllegalArgumentException, InconsistentNodeException,
			ExecutionException;

	public Object visit(TransfDeclaration node, Object data)
			throws IllegalArgumentException, InconsistentNodeException,
			ExecutionException;

    
}
