/*
 * JaTS Project - Java Transformation System
 * http://www.cin.ufpe.br/~jats
 *  
 * Copyright  2005  Informatics Center - Federal University of Pernambuco
 * 
 * JaTS is a free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 */
package cin.jats.engine.visitors;


import java.io.Reader;
import java.io.StringReader;
import java.util.ArrayList;
import java.util.List;
import java.util.Vector;

import cin.jats.engine.JatsIO;
import cin.jats.engine.ParserFactory;
import cin.jats.engine.parser.nodes.*;

import cin.jats.engine.parser.nodes.exceptions.*;
import cin.jats.engine.parser.nodes.metajats.ApplyExpression;
import cin.jats.engine.parser.nodes.metajats.CodeAssignStatement;
import cin.jats.engine.parser.nodes.metajats.ParsingStatement;
import cin.jats.engine.parser.nodes.metajats.RSetEntryStatement;
import cin.jats.engine.parser.nodes.metajats.TransfDeclaration;


/**
 * Pre-process MetaJaTS programs converting them into pure
 * java programs.
 * This extends CloningVisitor because all java statement should
 * be keeped unchanged. This way, only meta jats staments are
 * explicitly converted to java code here. 
 *
 * @see INode
 * @see IVisitor
 *
 * @author Gustavo Santos
 * @version 20/12/2005
 */
public class PreProcessorVisitor extends CloningVisitor {

	private JatsIO jatsio;
	
	/**
	 * A set of patterns extracted from the meta program.
	 */
	private List patterns;
	
    /**
     * Construtor vazio da classe.
     */
    public PreProcessorVisitor() {
        jatsio = JatsIO.getInstance();
        patterns = new ArrayList();
    }
	
	/**
	 * @return Returns the patterns.
	 */
	public List getPatterns() {
		return patterns;
	}


	/**
	 * @param patterns The patterns to set.
	 */
	public void setPatterns(Vector patterns) {
		this.patterns = patterns;
	}    

	/**
	 * This method overriden because StamentLists need specific
	 * treatment in meta programs.
	 * 
	 * @param node
	 * @param resultNode
	 * @return
	 * @throws IllegalArgumentException
	 * @throws InconsistentNodeException
	 */
    private JStatementList cloneStatementList(JStatementList node,
			JStatementList resultNode) throws IllegalArgumentException,
			InconsistentNodeException {

		if (node == null || resultNode == null) {
			throw new IllegalArgumentException();
		}

		if (node.isASourceNode()) {

			for (int i = 0; i < node.size(); i++) {
				INode elem = this.state.pop();
				if (elem instanceof JStatementList) {
					for (int j = 0; j < ((JStatementList)elem).size(); j++)
						resultNode.add(((JStatementList)elem).elementAt(j));
				} else {
					resultNode.add(elem);
				}
			}
		}
		
		if (node.getPosition() != null) {
		    resultNode.setPosition(node.getPosition());
		}

		return resultNode;
	}
    
    
    /**
     * See PreProcessorVisitor.cloneStatementList()
     */
	public Object visit(JStatementList node, Object data)
			throws IllegalArgumentException, InconsistentNodeException,
			ExecutionException {
		JStatementList resultNode = new JStatementList();
		resultNode = cloneStatementList(node, resultNode);
		this.doCommonCloningWork(node, resultNode);
		return resultNode;

	}

	
	public Object visit(Pattern node, Object data) throws IllegalArgumentException, InconsistentNodeException, ExecutionException {
		if (node == null) {
            throw new IllegalArgumentException();
        }
		Reader sr = null;
		String s = "";
		JStatementList l = null;
		JStatementList result = new JStatementList();
		
        this.patterns.add(node);
        
        //tira o nome e o corpo da pilha
        this.state.pop();
        this.state.pop();
        
        try {
        	
            //reader = new FileReader("<nome_pat>.jats");
			s = "reader = new FileReader(root+\"";
			s += node.getName().getValue().replaceFirst("#", "") + ".jats\");";
			sr = new StringReader(s);
			jatsio.changeParserType(ParserFactory.JATS_PARSER);
			l = jatsio.parseStatementList(sr);

			result.add(l.getStatements().elementAt(0));

			// Pattern <nome_pat> = jatsio.parsePatternFile(reader);
			s = "Pattern ";
			s += node.getName().getValue().replaceFirst("#", "");
			s += " = jatsio.parsePatternFile(reader);";
			sr = new StringReader(s);
			l = jatsio.parseStatementList(sr);

			result.add(l.getStatements().elementAt(0));

			this.state.push(result);
			
		} catch (Exception e) {
			e.printStackTrace();
			throw new RuntimeException();
		}

		return node;
	}



	public Object visit(CodeAssignStatement node, Object data) throws IllegalArgumentException, InconsistentNodeException, ExecutionException {
		Reader sr = null;
		String s = "";
		JStatementList l = null;
		
		try {

			if (node.getValue() instanceof ApplyExpression) {
				ApplyExpression exp = (ApplyExpression)node.getValue();
				
				//INode result = engine.apply(t, source, res);
				s += "INode "+node.getId()+" = engine.apply(";
				s += exp.getTransfId()+","+exp.getSourceId()+","+exp.getRSetId();
				s += ");";				
			} else {
				
				// reader = new FileReader("<file_path>");
				// jatsio.changeParserType(ParserFactory.JAVA_PARSER);
				// INode <source> = jatsio.parse(source);
				s = "reader = new FileReader(root+";
				s += node.getValue() + ");";
				s += "jatsio.changeParserType(ParserFactory.JAVA_PARSER);";
				s += "INode " + node.getId() + " = jatsio.parse(reader);";				
			}
			
			sr = new StringReader(s);
			jatsio.changeParserType(ParserFactory.JATS_PARSER);
			l = jatsio.parseStatementList(sr);

			this.state.push(l);

		} catch (Exception e) {
			e.printStackTrace();
			throw new RuntimeException();
		}

		return node;
	}

	public Object visit(ParsingStatement node, Object data) throws IllegalArgumentException, InconsistentNodeException, ExecutionException {
		Reader sr = null;
		String s = "";
		JStatementList l = null;
		
		try {
			// String s = <literal>;
			// Reader sr = new StringReader(s);
			// jatsio.changeParserType(ParserFactory.JATS_PARSER);
			// JStatementList l = jatsio.parseStatemetList(sr);
			s = "strTemp = \"" + this.cleanSpecialCharacters(node.getCode()) + "\";";
			s += "reader = new StringReader(strTemp);";
			s += "jatsio.changeParserType(ParserFactory.JATS_PARSER);";
			s += "J"+getCorrespondingNode(node.getType()) + " " + node.getId() + " = jatsio.parse";
			s += node.getType() + "(reader);";
			sr = new StringReader(s);
			jatsio.changeParserType(ParserFactory.JATS_PARSER);
			l = jatsio.parseStatementList(sr);

			this.state.push(l);
		} catch (Exception e) {
			e.printStackTrace();
			throw new RuntimeException();
		}
		
		return node;
	}
	
	/**
	 * Some parser productions do not build nodes with exactly the
	 * same name of itself, so this method "translates" the production
	 * name into the node name.
	 * 
	 * @param production
	 * @return the node type
	 */
	private String getCorrespondingNode(String production) {
		String result = "";
		
		if ("ResultType".equals(production)) {
			result = "Type";
		} else {
			result = production;
		}
		return result;
	}

	private String cleanSpecialCharacters(String str) {
		String result = "";
		
		result = str.replaceAll("\n", "");
		result = result.replaceAll("\t", "");
		result = result.replaceAll("\b", "");
		result = result.replaceAll("\f", "");
		result = result.replaceAll("\r", "");
		result = result.replaceAll("\\\\", "\\\\\\\\");
		result = result.replaceAll("\\\'", "\\\\\\\'");
		result = result.replaceAll("\\\"", "\\\\\\\"");
		
		return result;
		
	}

	public Object visit(RSetEntryStatement node, Object data) throws IllegalArgumentException, InconsistentNodeException, ExecutionException {
		Reader sr = null;
		String s = "", var = "";
		JStatementList l = null;
		
		try {
			//l.setPosition(p);
			//l.setVariableName("#code");
			//res.put(l, l);
			var = node.getValueId();
			if (node.getPositionId() != null){
			    s = var+"."+"setPosition("+node.getPositionId()+");";
			}
			s += node.getRSetId()+"."+"put(\""+node.getJatsVarId()+"\","+var+","+var+".getTypeIdentifier());";
			sr = new StringReader(s);
			jatsio.changeParserType(ParserFactory.JATS_PARSER);
			l = jatsio.parseStatementList(sr);

			this.state.push(l);
		} catch (Exception e) {
			e.printStackTrace();
			throw new RuntimeException();
		}
		
		return node;
	}

	public Object visit(TransfDeclaration node, Object data) throws IllegalArgumentException, InconsistentNodeException, ExecutionException {
		Reader sr = null;
		String s = "";
		JStatementList l = null;
		
		try {
			//MicroTransformation transformation = new MicroTransformation(classDec, newclassDec);
			s += "MicroTransformation "+node.getId()+" = ";
			s += "new MicroTransformation("+node.getSourcePtnId().replaceFirst("#","");
			s += ","+node.getTargetPtnId().replaceFirst("#","")+");";
			sr = new StringReader(s);
			jatsio.changeParserType(ParserFactory.JATS_PARSER);
			l = jatsio.parseStatementList(sr);

			this.state.push(l);
		} catch (Exception e) {
			e.printStackTrace();
			throw new RuntimeException();
		}
		
		return node;
	}

}