package cin.jats.tests;

import java.io.*;
import java.util.*;

import cin.jats.engine.JatsIO;
import cin.jats.engine.ParserFactory;
import cin.jats.engine.JatsEngine;
import cin.jats.engine.parser.nodes.*;
import cin.jats.engine.util.NodeList;
import cin.jats.engine.util.ResultSet;
import cin.jats.tests.gui.JaTSTesterFrame;
import cin.jats.tests.util.UserJaTSVariable;

/**
 * Testing tool for the JaTS engine
 *
 *@author     Clovis Nogueira
 *@author     Gustavo Santos
 *@created    06/06/2002
 *
 *@version 13/12/2005
 */
public class JaTSTester {

  private final static String TEMPLATE_L_ITEM = "templateLeft.";
  private final static String TEMPLATE_R_ITEM = "templateRight.";
  private final static String SOURCE_ITEM = "source.";
  private final static String FILES_DIRECTORY = "filesDirectory";
  private final static String WIZARD_JAR_NAME = "wizardJarName";
  private final static String SOURCE_JAR_NAME = "sourceJarName";
  private final static String APPLY_OUTPUT_DIR = "applyOutputDir";

  private final static String USER_INPUT = "userInput.";
  private final static String JATS_VARIABLE = "jatsVariable.";
  private final static String JATS_TYPE = "jatsType.";
  private final static String JATS_VALUE = "value.";

  private static JaTSTester instance;

  private JatsIO jatsIo;
  private JatsEngine engine;
  private ResultSet resultSet;

  private Vector sourceNames;
  private Vector templateLeftNames;
  private Vector templateRightNames;
  private Vector userVariables;
  private String wizardJarFileName;
  private String sourceJarFileName;
  private String filesDirectory;
  private String outputDirectory;
  private boolean isValidPropertyFile = false;

  /**
   *  Constructor for the JaTSTester object
   */
  public JaTSTester() {
    jatsIo = JatsIO.getInstance();
    engine = JatsEngine.getInstance();
    resultSet = new ResultSet();
    sourceNames = new Vector();
    templateLeftNames = new Vector();
    templateRightNames = new Vector();
    userVariables = new Vector();
    filesDirectory = outputDirectory = "";
//        applyOutpurFile = "output.txt";
  }

  /**
   *  Singleton
   *
   *@return
   */
  public static JaTSTester getInstance() {
    if (instance == null) {
      instance = new JaTSTester();
    }
    return instance;
  }

  /**
   *  A unit test for JUnit
   *
   *@param  templateFile   Description of the Parameter
   *@param  readFromJar    Description of the Parameter
   *@return                Description of the Return Value
   *@exception  Exception  Description of the Exception
   */
  public INode parseTemplateFile(String templateFile,
                                            boolean readFromJar) throws
      Exception {
    Reader in = null;
    try {
      JatsIO io = JatsIO.getInstance();
      if (!readFromJar) {

    	  if (this.filesDirectory.equals("."))
    		this.filesDirectory = JaTSTesterFrame.fileChooser.getSelectedFile().getParentFile().getAbsolutePath();

    	  in = new FileReader(this.filesDirectory + File.separator + templateFile);
      }
//      else {
//        String resourceName = wizardJarFileName + "/" + TEMPLATE_PATH +
//            templateFile;
//        InputStream is = classLoader.getResourceAsStream(resourceName);
//        if (is == null) {
//          throw new FileNotFoundException("Error loading Template file.");
//        }
//        in = new InputStreamReader(is);
//      }
      io.changeParserType(ParserFactory.JATS_PARSER);
      INode node = io.parsePatternFile(in);
      return node;
    }
    finally {
      if (in != null) {
        in.close();
      }
    }
  }

  /**
   *  A unit test for JUnit
   *
   *@param  templateFile   Description of the Parameter
   *@param  readFromJar    Description of the Parameter
   *@return                Description of the Return Value
   *@exception  Exception  Description of the Exception
   */
  public ICompilationUnit parseSourceFile(String templateFile,
                                          boolean readFromJar) throws Exception {
    Reader in = null;
    try {
      JatsIO io = JatsIO.getInstance();
      if (!readFromJar) {
    	  
      	if (this.filesDirectory.equals("."))
    		this.filesDirectory = JaTSTesterFrame.fileChooser.getSelectedFile().getParentFile().getAbsolutePath();
    	  
        in = new FileReader(this.filesDirectory + File.separator + templateFile);
      }
//      else {
//        String resourceName = sourceJarFileName + File.separator + templateFile;
//        InputStream is = classLoader.getResourceAsStream(resourceName);
//        if (is == null) {
//          throw new FileNotFoundException("Error loading Source file.");
//        }
//        in = new InputStreamReader(is);
//      }
      io.changeParserType(ParserFactory.JAVA_PARSER);
      ICompilationUnit node = io.parse(in);
      return node;
    }
    finally {
      if (in != null) {
        in.close();
      }
    }
  }

  /**
   *  A unit test for JUnit
   *
   *@param  readFromSourceJar    Description of the Parameter
   *@param  readFromTemplateJar  Description of the Parameter
   *@exception  Exception        Description of the Exception
   */
  public void testTemplateMatching(boolean readFromSourceJar,
                                   boolean readFromTemplateJar) throws
      Exception {
    if (isValidPropertyFile) {
      resultSet.reset();

      // Creates a set of compilationUnits for the sources
      NodeList sources = generateCompilationUnitSet(sourceNames,
          readFromSourceJar, true);
      // Creates a set of compilationUnits for the templates
      NodeList templates = generateCompilationUnitSet(
          templateLeftNames, readFromTemplateJar, false);

      // Applies the engine matching
      engine.match(templates, sources, resultSet);
    }
    else {
      throw new IllegalArgumentException("Invalid properties file.");
    }
  }

  /**
   *  A unit test for JUnit
   *
   *@param  readFromSourceJar    Description of the Parameter
   *@param  readFromTemplateJar  Description of the Parameter
   *@exception  Exception        Description of the Exception
   */
  public void testEngineApply(boolean readFromSourceJar,
                              boolean readFromTemplateJar) throws Exception {
    if (isValidPropertyFile) {
      resultSet.reset();

      // Creates a set of compilationUnits for the sources
      NodeList sourcesSet = generateCompilationUnitSet(sourceNames,
          readFromSourceJar, true);
      // Creates a set of compilationUnits for the templates
      NodeList templatesLeftSet = generateCompilationUnitSet(
          templateLeftNames, readFromTemplateJar, false);
      // Creates a set of compilationUnits for the sources
      NodeList templatesRightSet = generateCompilationUnitSet(
          templateRightNames, readFromTemplateJar, false);

      // Sets the user variables in the ResultSet
      putUserJaTSVariablesInResultSet();

      // Applies the engine matching
      NodeList results =
          engine.apply(templatesLeftSet, sourcesSet, templatesRightSet,
                       resultSet);

      // reprocessa os templates para imprimir bonitinho.
      //ICompilationUnit[] results = /*this.preprocessTypes(*/ resultCompSet.
      //    toArray(); //);

      FileWriter writer;// = new FileWriter(filesDirectory + "/" + applyOutpurFile);

      if (results != null) {
        int size = results.size();
        INode tmp = null;
        for (int i = 0; i < size; i++) {
          tmp = results.elementAt(i);
          if (tmp != null) {
        	String fileName = "";        	
        	if (tmp instanceof ICompilationUnit)
        		fileName = ""+((ICompilationUnit)tmp).getTypeDeclaration(0).getName();
        	else
        		fileName = ""+tmp.getClass();
        	if (this.outputDirectory.equals("."))
        		this.outputDirectory = JaTSTesterFrame.fileChooser.getSelectedFile().getParentFile().getAbsolutePath();
        	writer = new FileWriter(this.outputDirectory + "/" +
                                    fileName+"_out.java");
            jatsIo.prettyPrint(tmp, writer);
            writer.flush();
            writer.close();
          }
        }
      }
    }
    else {
      throw new IllegalArgumentException("Invalid properties file.");
    }

  }

  /**
   * Pre-processa os tipos antes de envi-los para o IDE. Isto faz com que as
   * classes sejam impressas de forma mais "bonitinha"
   *
   * @param resultedTypes Tipos a serem pre-processados
   */
/*  private ICompilationUnit[] preprocessTypes(JCompilationUnit[] resultedTypes) {
    ICompilationUnit[] retorno = null;
    JatsIO jio = JatsIO.getInstance();
    if (resultedTypes != null) {
      retorno = new ICompilationUnit[resultedTypes.length];

      for (int i = 0; i < retorno.length; i++) {
        try {
          PrePrettyPrinter.preprocess(resultedTypes[i]);
          retorno[i] = jio.parse(new StringReader(resultedTypes[i].toString()));
        }
        catch (ParseException e) {
          e.printStackTrace();
        }
        catch (IllegalArgumentException e) {
          e.printStackTrace();
        }
      }

    }
    return retorno;

  }
*/
  /**
   *  Description of the Method
   *
   *@param  completePath  Description of the Parameter
   */
  public static void createJaTSTesterPropertiesFile(String completePath) {
    Properties prop = new Properties();
    prop.put(FILES_DIRECTORY, "");
    prop.put(APPLY_OUTPUT_DIR, "");
    prop.put(WIZARD_JAR_NAME, "");
    prop.put(SOURCE_JAR_NAME, "");
    FileOutputStream out = null;

    try {
      out = new FileOutputStream(completePath);
      prop.store(out, "teste");
      out.flush();
      out.close();
    }
    catch (Exception ex) {
      ex.printStackTrace();
    }

  }

  /**
   *  Description of the Method
   *
   *@param  out  Description of the Parameter
   */
  public void saveJaTSTesterPropertiesFile(OutputStream out) {
    Properties prop = new Properties();
    prop.put(FILES_DIRECTORY, this.filesDirectory);
    prop.put(APPLY_OUTPUT_DIR, this.outputDirectory);
    prop.put(WIZARD_JAR_NAME, "" + this.wizardJarFileName);
    prop.put(SOURCE_JAR_NAME, "" + this.sourceJarFileName);

    for (int i = 0; i < this.templateLeftNames.size(); i++) {
      prop.put(TEMPLATE_L_ITEM + i, this.templateLeftNames.elementAt(i));
    }
    for (int i = 0; i < this.templateRightNames.size(); i++) {
      prop.put(TEMPLATE_R_ITEM + i, this.templateRightNames.elementAt(i));
    }
    for (int i = 0; i < this.sourceNames.size(); i++) {
      prop.put(SOURCE_ITEM + i, this.sourceNames.elementAt(i));
    }

    for (int i = 0; i < this.userVariables.size(); i++) {
      prop.put(USER_INPUT + JATS_VARIABLE + i,
               ( (UserJaTSVariable)this.userVariables.elementAt(i)).
               getJaTSVariableName());
      prop.put(USER_INPUT + JATS_TYPE + i,
               ( (UserJaTSVariable)this.userVariables.elementAt(i)).
               getJatsTypeString());
      prop.put(USER_INPUT + JATS_VALUE + i,
               ( (UserJaTSVariable)this.userVariables.elementAt(i)).
               getJaTSVariableValue());
    }

    try {
      prop.store(out, "Arquivo de descricao de transformacao");
    }
    catch (Exception e) {
      e.printStackTrace();
    }

  }

  /**
   *  Description of the Method
   *
   *@param  propertiesInpuStream          Description of the Parameter
   *@exception  IllegalArgumentException  Description of the Exception
   *@exception  IOException               Description of the Exception
   */
  public void loadJaTSTesterProperties(InputStream propertiesInpuStream) throws
      IllegalArgumentException, IOException {
    // system properties
    Properties prop = new Properties();
    // user variables properties
    Properties jaTSVarProp = new Properties();

    // Loading the property file name
    prop.load(propertiesInpuStream);

    Vector temLeftKeys = new Vector();
    Vector temRightKeys = new Vector();
    Vector sourceKeys = new Vector();

    Enumeration propertyNames = prop.propertyNames();
    while (propertyNames.hasMoreElements()) {
      String item = (String) propertyNames.nextElement();
      if (item.startsWith(TEMPLATE_L_ITEM)) {
        temLeftKeys.add(item);
      }
      else if (item.startsWith(TEMPLATE_R_ITEM)) {
        temRightKeys.add(item);
      }
      else if (item.startsWith(SOURCE_ITEM)) {
        sourceKeys.add(item);
      }
      else if (item.startsWith(USER_INPUT)) {
        jaTSVarProp.setProperty(item, prop.getProperty(item));
      }
    }

    if (sourceNames.size() != templateLeftNames.size()) {
      isValidPropertyFile = false;
      throw new IllegalArgumentException("Invalid properties file, " +
          "number of Left Templates not the same of Java Source Files.");
    }

    // Sort the items
    Collections.sort(temLeftKeys);
    Collections.sort(temRightKeys);
    Collections.sort(sourceKeys);

    templateLeftNames = new Vector();
    sourceNames = new Vector();
    templateRightNames = new Vector();

    //Fill the vectors with the real names of the items
    for (int i = 0; i < sourceKeys.size(); i++) {
      sourceNames.add(prop.getProperty( (String) sourceKeys.elementAt(i)));
    }
    for (int i = 0; i < temLeftKeys.size(); i++) {
      templateLeftNames.add(prop.getProperty( (String) temLeftKeys.elementAt(i)));
    }
    for (int i = 0; i < temRightKeys.size(); i++) {
      templateRightNames.add(prop.getProperty( (String) temRightKeys.elementAt(
          i)));
    }

    this.filesDirectory = prop.getProperty(FILES_DIRECTORY);
    this.wizardJarFileName = prop.getProperty(WIZARD_JAR_NAME);
    this.sourceJarFileName = prop.getProperty(SOURCE_JAR_NAME);
    this.outputDirectory = prop.getProperty(APPLY_OUTPUT_DIR);

    loadUserJaTSVariables(jaTSVarProp);

    isValidPropertyFile = true;
    propertiesInpuStream.close();
  }

  /**
   *  Description of the Method
   *
   *@param  prop  Description of the Parameter
   */
  private void loadUserJaTSVariables(Properties prop) {
    String variableName = "";
    String value = "";
    String variableType = "";

    int size = prop.size() / 3;
    for (int i = 0; i < size; i++) {
      variableName = prop.getProperty(USER_INPUT + JATS_VARIABLE + i);
      if (variableName != null) {
        value = prop.getProperty(USER_INPUT + JATS_VALUE + i);
        variableType = prop.getProperty(USER_INPUT + JATS_TYPE + i);
        UserJaTSVariable u = new UserJaTSVariable(variableType,
                                                  variableName, value);
        userVariables.add(u);
      }
    }
  }

  /**
   *  Description of the Method
   */
  private void putUserJaTSVariablesInResultSet() {
    Enumeration variables = userVariables.elements();
    UserJaTSVariable userVar;
    while (variables.hasMoreElements()) {
      userVar = (UserJaTSVariable) variables.nextElement();
      switch (userVar.getJaTSType()) {
        case JLiteral.STRING_LITERAL:
          JLiteral literal = new JLiteral();
          literal.setVariableName(userVar.getJaTSVariableName());
          literal.setLiteralType(JLiteral.STRING_LITERAL);

          JLiteral litValue = new JLiteral();
          litValue.setLiteralType(JLiteral.STRING_LITERAL);
          litValue.setLiteralValue(userVar.getJaTSVariableValue());
          resultSet.put(literal, litValue);
          break;
        case JLiteral.BOOLEAN_LITERAL:
          JLiteral litBoolean = new JLiteral();
          litBoolean.setVariableName(userVar.getJaTSVariableName());
          litBoolean.setLiteralType(JLiteral.BOOLEAN_LITERAL);
          resultSet.put(litBoolean, userVar.getJaTSVariableValue());
          break;
        case JLiteral.INTEGER_LITERAL:
          JLiteral litInteger = new JLiteral();
          litInteger.setVariableName(userVar.getJaTSVariableName());
          litInteger.setLiteralType(JLiteral.INTEGER_LITERAL);
          resultSet.put(litInteger, userVar.getJaTSVariableValue());
          break;
        case JaTSNode.NAME:
          resultSet.put(userVar.getJaTSVariableName(),
                        userVar.getJaTSVariableValue(), JaTSNode.NAME);
          break;
        case JaTSNode.IDENTIFIER:
          resultSet.put(userVar.getJaTSVariableName(),
                        userVar.getJaTSVariableValue(), JaTSNode.IDENTIFIER);
          break;

        default:
          resultSet.put(userVar.getJaTSVariableName(),
                        userVar.getJaTSVariableValue(), userVar.getJaTSType());
      }
    }
  }

  private NodeList generateCompilationUnitSet(Vector unitNames,
      boolean readFromJar, boolean isSourceSet) throws Exception {
    // Creates a set of compilationUnits
    NodeList set = new NodeList();
    for (int i = 0; i < unitNames.size(); i++) {
      // Parses each source item
      INode compUnit;
      if (isSourceSet) {
        compUnit = parseSourceFile( (String) unitNames.elementAt(i),
                                   readFromJar);
      }
      else {
        compUnit = parseTemplateFile( (String) unitNames.elementAt(i),
                                     readFromJar);
      }
      // add the compilationUnit generated to the set
      set.addElement(compUnit);
    }
    return set;
  }

  /**
   *  Description of the Method
   */
  public void resetResultSet() {
    resultSet.reset();
  }

  /**
   *  Description of the Method
   *
   *@return                                Description of the Return Value
   *@exception  VariableNotFoundException  Description of the Exception
   */
  public String resultSetDebugString() throws Exception {
    Iterator it = resultSet.getVariables();
    int i = 0;
    StringBuffer strBuffer = new StringBuffer();
    while (it.hasNext()) {
      cin.jats.engine.parser.nodes.JaTSNode chave =
          (cin.jats.engine.parser.nodes.JaTSNode) it.next();

      strBuffer.append(i + " - " + chave + " ==>");
      strBuffer.append(resultSet.get(chave) + "\n\n");
      i++;
    }
    return strBuffer.toString();
  }

  /**
   *  Description of the Method
   *
   *@param  fileName  Description of the Parameter
   *@param  content   Description of the Parameter
   */
  public void writeStringToFile(String fileName, String content) {
    try {
      PrintWriter w = new PrintWriter(new FileOutputStream(fileName));
      w.write(content);
      w.close();
    }
    catch (FileNotFoundException e) {
      e.printStackTrace();
    }
  }

  /**
   *  Gets the applyOutpurFile attribute of the JaTSTester object
   *
   *@return    The applyOutpurFile value
   */
  public String getOutputDirectory() {
    return this.outputDirectory;
  }

  public void setOutputDirectory(String outputDirectory) {
    this.outputDirectory = outputDirectory;
  }


  /**
   *  Gets the fileDirectory attribute of the JaTSTester object
   *
   *@return    The fileDirectory value
   */
  public String getFilesDirectory() {
    return this.filesDirectory;
  }

  /**
   *  Gets the sourceJarFileName attribute of the JaTSTester object
   *
   *@return    The sourceJarFileName value
   */
  public String getSourceJarFileName() {
    return sourceJarFileName;
  }

  /**
   *  Gets the sourceNames attribute of the JaTSTester object
   *
   *@return    The sourceNames value
   */
  public Vector getSourceNames() {
    return sourceNames;
  }

  /**
   *  Gets the templateLeftNames attribute of the JaTSTester object
   *
   *@return    The templateLeftNames value
   */
  public Vector getTemplateLeftNames() {
    return templateLeftNames;
  }

  /**
   *  Gets the templateRightNames attribute of the JaTSTester object
   *
   *@return    The templateRightNames value
   */
  public Vector getTemplateRightNames() {
    return templateRightNames;
  }

  /**
   *  Gets the wizardJarFileName attribute of the JaTSTester object
   *
   *@return    The wizardJarFileName value
   */
  public String getWizardJarFileName() {
    return wizardJarFileName;
  }

  /**
   *  Gets the isValidPropertyFile attribute of the JaTSTester object
   *
   *@return    The isValidPropertyFile value
   */
  public boolean isIsValidPropertyFile() {
    return isValidPropertyFile;
  }

  /**
   *  Sets the templateLeftNames attribute of the JaTSTester object
   *
   *@param  templateLeftNames  The new templateLeftNames value
   */
  public void setTemplateLeftNames(Vector templateLeftNames) {
    this.templateLeftNames = templateLeftNames;
  }

  /**
   *  Sets the templateRightNames attribute of the JaTSTester object
   *
   *@param  templateRightNames  The new templateRightNames value
   */
  public void setTemplateRightNames(Vector templateRightNames) {
    this.templateRightNames = templateRightNames;
  }

  /**
   *  Sets the sourceNames attribute of the JaTSTester object
   *
   *@param  sourceNames  The new sourceNames value
   */
  public void setSourceNames(Vector sourceNames) {
    this.sourceNames = sourceNames;
  }

  /**
   *  Sets the userVariables attribute of the JaTSTester object
   *
   *@param  userVariables  The new userVariables value
   */
  public void setUserVariables(Vector userVariables) {
    this.userVariables = userVariables;
  }

  public Vector getUserVariables() {
    return userVariables;
  }

  public void setFilesDirectory(String filesDirectory) {
    this.filesDirectory = filesDirectory;
  }

  public void setWizardJarFileName(String wizardJarFileName) {
    this.wizardJarFileName = wizardJarFileName;
  }

  public void setSourceJarFileName(String sourceJarFileName) {
    this.sourceJarFileName = sourceJarFileName;
  }
}
