/*
 * JaTS Project - Java Transformation System
 * http://www.cin.ufpe.br/~jats
 *  
 * Copyright  2005  Informatics Center - Federal University of Pernambuco
 * 
 * JaTS is a free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 */
package cin.jats.tests.util;

import cin.jats.engine.parser.nodes.JaTSNode;
import cin.jats.engine.parser.nodes.JLiteral;
import java.util.Collection;
import java.util.Vector;

/**
 * Some JaTS variables can recieve values directly from the
 * user. This class represent such variables and is part of
 * JaTSTester enviroment.
 * A variable representation includes its type, value and
 * user friendly name.
 * 
 * @author     Gustavo Santos
 * @created    7 de Junho de 2002
 * @version    02/12/2005
 */
public class UserJaTSVariable {

    public final static String NAME = "NAME";
    public final static String STRING_LITERAL = "STRING_LITERAL";
    public final static String BOOLEAN_LITERAL = "BOOLEAN_LITERAL";
	public final static String INTEGER_LITERAL = "INTEGER_LITERAL";
    public final static String IDENTIFIER = "IDENTIFIER";
    public final static String PACKAGE_DECLARATION = "PACKAGE_DECLARATION";
    public final static String METHOD_DECLARATION = "METHOD_DECLARATION";
    public final static String FIELD_DECLARATION = "FIELD_DECLARATION";
    public final static String PARAMETER_LIST = "PARAMETER_LIST";
    public final static String STATEMENT_LIST = "STATEMENT_LIST";

    //HA UM PROBLEMA EM HAVER UMA UNICA REPRESENTACAO PARA O TIPO JATS. NOS
    //CASOS DE LITERAIS, OS TIPOS DOS LITERAIS (BOOLEAN, STRING, INT) SAO
    //SETADOS AQUI. POREM AS REPRESENTACOES SE INTERSECTAM (I.E. true = "string")
    private int jaTSType;
    private String jaTSVariableName;
    private String jaTSVariableValue;


    /**
     *  Constructor for the UserJaTSVariable object
     *
     * @param  jatsStringType        The JaTS type name
     * @param  newJaTSVariableName   The name to be displayed
     * @param  newJaTSVariableValue  The variable value
     */
    public UserJaTSVariable(String jatsStringType, String newJaTSVariableName,
            String newJaTSVariableValue) {

        if (jatsStringType != null) {
            if (jatsStringType.equals(NAME)) {
                jaTSType = JaTSNode.NAME;
            } else if (jatsStringType.equals(STRING_LITERAL)) {
                jaTSType = JLiteral.STRING_LITERAL;
            } else if (jatsStringType.equals(BOOLEAN_LITERAL)) {
                jaTSType = JLiteral.BOOLEAN_LITERAL;
			} else if (jatsStringType.equals(INTEGER_LITERAL)) {
				jaTSType = JLiteral.INTEGER_LITERAL;
            } else if (jatsStringType.equals(IDENTIFIER)) {
                jaTSType = JaTSNode.IDENTIFIER;
            } else if (jatsStringType.equals(PACKAGE_DECLARATION)) {
                jaTSType = JaTSNode.PACKAGE_DECLARATION;
            } else if (jatsStringType.equals(METHOD_DECLARATION)) {
                jaTSType = JaTSNode.METHOD_DECLARATION;
            } else if (jatsStringType.equals(FIELD_DECLARATION)) {
                jaTSType = JaTSNode.FIELD_DECLARATION;
            } else if (jatsStringType.equals(PARAMETER_LIST)) {
                jaTSType = JaTSNode.PARAMETER_LIST;
            } else if (jatsStringType.equals(STATEMENT_LIST)) {
                jaTSType = JaTSNode.STATEMENT_LIST;
            } else {
            	throw new IllegalArgumentException("Unknown JaTS Type: "+jatsStringType);
            }
            jaTSVariableName = newJaTSVariableName;
            jaTSVariableValue = newJaTSVariableValue;
        } else {
        	throw new IllegalArgumentException("null variable name");
        }
    }

    /**
     *  Constructor for the UserJaTSVariable object
     * 
     * @param  newJaTSVariableType   The JaTS variable type
     * @param  newJaTSVariableName   The name to be displayed
     * @param  newJaTSVariableValue  The variable value
     */
    public UserJaTSVariable(int newJaTSVariableType, String newJaTSVariableName,
            String newJaTSVariableValue) {
        jaTSVariableName = newJaTSVariableName;
        jaTSVariableValue = newJaTSVariableValue;
        jaTSType = newJaTSVariableType;
    }

    /**
     * Retrives the variable type
     * 
     * @return A String representing the variable type
     */
    public String getJatsTypeString() {
        String retorno = "";

        if (this.jaTSType == JaTSNode.NAME) {
            retorno = NAME;
        } else if (this.jaTSType == JLiteral.STRING_LITERAL) {
            retorno = STRING_LITERAL;
        } else if (this.jaTSType == JLiteral.BOOLEAN_LITERAL) {
            retorno = BOOLEAN_LITERAL;
        } else if (this.jaTSType == JaTSNode.IDENTIFIER) {
            retorno = IDENTIFIER;
        } else if (this.jaTSType == JaTSNode.PACKAGE_DECLARATION) {
            retorno = PACKAGE_DECLARATION;
        } else if (this.jaTSType == JaTSNode.METHOD_DECLARATION) {
            retorno = METHOD_DECLARATION;
        } else if (this.jaTSType == JaTSNode.FIELD_DECLARATION) {
            retorno = FIELD_DECLARATION;
        } else if (this.jaTSType == JaTSNode.PARAMETER_LIST) {
            retorno = PARAMETER_LIST;
        } else if (this.jaTSType == JaTSNode.STATEMENT_LIST) {
            retorno = STATEMENT_LIST;
        }
        return retorno;
    }

    /**
     * All JaTS types supported by JaTSTester Enviroment
     * 
     * @return A set of variable names (Strings)
     */
	public static Collection getAllJatsTypes() {
       Vector<String> v = new Vector<String>();
       v.addElement(NAME);
       v.addElement(STRING_LITERAL);
       v.addElement(BOOLEAN_LITERAL);
       v.addElement(IDENTIFIER);
       v.addElement(PACKAGE_DECLARATION);
       v.addElement(METHOD_DECLARATION);
       v.addElement(FIELD_DECLARATION);
       v.addElement(PARAMETER_LIST);
       v.addElement(STATEMENT_LIST);
       return v;
    }

    /**
     *  Gets the jaTSType attribute of the UserJaTSVariable object
     *
     * @return    The jaTSType value
     */
    public int getJaTSType() {
        return jaTSType;
    }


    /**
     *  Gets the jaTSVariableName attribute of the UserJaTSVariable object
     *
     * @return    The jaTSVariableName value
     */
    public String getJaTSVariableName() {
        return jaTSVariableName;
    }


    /**
     *  Gets the jaTSVariableValue attribute of the UserJaTSVariable object
     *
     * @return    The jaTSVariableValue value
     */
    public String getJaTSVariableValue() {
        return jaTSVariableValue;
    }


    /**
     *  Description of the Method
     *
     * @return    Description of the Return Value
     */
    public String toString() {
        return jaTSVariableName + " : " + jaTSType;
    }

    public boolean equals(Object o) {
        if (!(o instanceof UserJaTSVariable)) throw new IllegalArgumentException(""+o);
        boolean retorno = false;

        if (this.getJaTSType() == ((UserJaTSVariable) o).getJaTSType()
            && this.getJaTSVariableName().equals(((UserJaTSVariable) o).getJaTSVariableName())) {
            retorno = true;
        }

        return retorno;
    }
}
